/****************************************************************************
** Meta object code from reading C++ file 'radiobrowserservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/radios/radiobrowserservice.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'radiobrowserservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19RadioBrowserServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto RadioBrowserService::qt_create_metaobjectdata<qt_meta_tag_ZN19RadioBrowserServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RadioBrowserService",
        "SearchFinished",
        "",
        "RadioChannelList",
        "channels",
        "has_more",
        "SearchError",
        "error",
        "CountriesLoaded",
        "QList<std::pair<QString,QString>>",
        "countries",
        "GetChannels",
        "ServerTestReply",
        "QNetworkReply*",
        "reply",
        "SearchReply",
        "task_id",
        "limit",
        "CountriesReply"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SearchFinished'
        QtMocHelpers::SignalData<void(const RadioChannelList &, const bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'SearchError'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'CountriesLoaded'
        QtMocHelpers::SignalData<void(const QList<QPair<QString,QString>> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'GetChannels'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ServerTestReply'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'SearchReply'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::Int, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'CountriesReply'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RadioBrowserService, qt_meta_tag_ZN19RadioBrowserServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RadioBrowserService::staticMetaObject = { {
    QMetaObject::SuperData::link<RadioService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19RadioBrowserServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19RadioBrowserServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19RadioBrowserServiceE_t>.metaTypes,
    nullptr
} };

void RadioBrowserService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RadioBrowserService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SearchFinished((*reinterpret_cast<std::add_pointer_t<RadioChannelList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->SearchError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->CountriesLoaded((*reinterpret_cast<std::add_pointer_t<QList<std::pair<QString,QString>>>>(_a[1]))); break;
        case 3: _t->GetChannels(); break;
        case 4: _t->ServerTestReply((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 5: _t->SearchReply((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->CountriesReply((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< RadioChannelList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RadioBrowserService::*)(const RadioChannelList & , const bool )>(_a, &RadioBrowserService::SearchFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RadioBrowserService::*)(const QString & )>(_a, &RadioBrowserService::SearchError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RadioBrowserService::*)(const QList<QPair<QString,QString>> & )>(_a, &RadioBrowserService::CountriesLoaded, 2))
            return;
    }
}

const QMetaObject *RadioBrowserService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RadioBrowserService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19RadioBrowserServiceE_t>.strings))
        return static_cast<void*>(this);
    return RadioService::qt_metacast(_clname);
}

int RadioBrowserService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RadioService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void RadioBrowserService::SearchFinished(const RadioChannelList & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void RadioBrowserService::SearchError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RadioBrowserService::CountriesLoaded(const QList<QPair<QString,QString>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
