/****************************************************************************
** Meta object code from reading C++ file 'playlistmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistmanager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PlaylistManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistManager::qt_create_metaobjectdata<qt_meta_tag_ZN15PlaylistManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistManager",
        "New",
        "",
        "name",
        "SongList",
        "songs",
        "special_type",
        "Load",
        "filename",
        "Save",
        "id",
        "playlist_name",
        "PlaylistSettings::PathType",
        "path_type",
        "SaveWithUI",
        "Rename",
        "new_name",
        "Favorite",
        "favorite",
        "Delete",
        "Close",
        "Open",
        "ChangePlaylistOrder",
        "QList<int>",
        "ids",
        "SetCurrentPlaylist",
        "SetActivePlaylist",
        "SetActiveToCurrent",
        "SelectionChanged",
        "QItemSelection",
        "selection",
        "SetCurrentOrOpen",
        "ClearCurrent",
        "ShuffleCurrent",
        "RemoveDuplicatesCurrent",
        "RemoveUnavailableCurrent",
        "SongChangeRequestProcessed",
        "QUrl",
        "url",
        "valid",
        "InsertUrls",
        "QList<QUrl>",
        "urls",
        "pos",
        "play_now",
        "enqueue",
        "signal",
        "InsertSongs",
        "RemoveItemsWithoutUndo",
        "indices",
        "RemoveCurrentSong",
        "PlaySmartPlaylist",
        "PlaylistGeneratorPtr",
        "generator",
        "as_new",
        "clear",
        "RateCurrentSong",
        "rating",
        "RateCurrentSong2",
        "SaveAllPlaylists",
        "SetActivePlaying",
        "SetActivePaused",
        "SetActiveStopped",
        "OneOfPlaylistsChanged",
        "UpdateSummaryText",
        "UpdateCollectionSongs",
        "ItemsLoadedForSavePlaylist",
        "PlaylistLoaded"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'New'
        QtMocHelpers::SlotData<void(const QString &, const SongList &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'New'
        QtMocHelpers::SlotData<void(const QString &, const SongList &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'New'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'Load'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'Save'
        QtMocHelpers::SlotData<void(const int, const QString &, const QString &, const PlaylistSettings::PathType)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 11 }, { QMetaType::QString, 8 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'SaveWithUI'
        QtMocHelpers::SlotData<void(const int, const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'Rename'
        QtMocHelpers::SlotData<void(const int, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'Favorite'
        QtMocHelpers::SlotData<void(const int, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'Delete'
        QtMocHelpers::SlotData<void(const int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'Close'
        QtMocHelpers::SlotData<bool(const int)>(20, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'Open'
        QtMocHelpers::SlotData<void(const int)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'ChangePlaylistOrder'
        QtMocHelpers::SlotData<void(const QList<int> &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'SetCurrentPlaylist'
        QtMocHelpers::SlotData<void(const int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'SetActivePlaylist'
        QtMocHelpers::SlotData<void(const int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'SetActiveToCurrent'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'SetCurrentOrOpen'
        QtMocHelpers::SlotData<void(const int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'ClearCurrent'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ShuffleCurrent'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RemoveDuplicatesCurrent'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RemoveUnavailableCurrent'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SongChangeRequestProcessed'
        QtMocHelpers::SlotData<void(const QUrl &, const bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 38 }, { QMetaType::Bool, 39 },
        }}),
        // Slot 'InsertUrls'
        QtMocHelpers::SlotData<void(const int, const QList<QUrl> &, const int, const bool, const bool, const bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 41, 42 }, { QMetaType::Int, 43 }, { QMetaType::Bool, 44 },
            { QMetaType::Bool, 45 }, { QMetaType::Bool, 46 },
        }}),
        // Slot 'InsertUrls'
        QtMocHelpers::SlotData<void(const int, const QList<QUrl> &, const int, const bool, const bool)>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 41, 42 }, { QMetaType::Int, 43 }, { QMetaType::Bool, 44 },
            { QMetaType::Bool, 45 },
        }}),
        // Slot 'InsertUrls'
        QtMocHelpers::SlotData<void(const int, const QList<QUrl> &, const int, const bool)>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 41, 42 }, { QMetaType::Int, 43 }, { QMetaType::Bool, 44 },
        }}),
        // Slot 'InsertUrls'
        QtMocHelpers::SlotData<void(const int, const QList<QUrl> &, const int)>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 41, 42 }, { QMetaType::Int, 43 },
        }}),
        // Slot 'InsertUrls'
        QtMocHelpers::SlotData<void(const int, const QList<QUrl> &)>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'InsertSongs'
        QtMocHelpers::SlotData<void(const int, const SongList &, const int, const bool, const bool, const bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 4, 5 }, { QMetaType::Int, 43 }, { QMetaType::Bool, 44 },
            { QMetaType::Bool, 45 }, { QMetaType::Bool, 46 },
        }}),
        // Slot 'InsertSongs'
        QtMocHelpers::SlotData<void(const int, const SongList &, const int, const bool, const bool)>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 4, 5 }, { QMetaType::Int, 43 }, { QMetaType::Bool, 44 },
            { QMetaType::Bool, 45 },
        }}),
        // Slot 'InsertSongs'
        QtMocHelpers::SlotData<void(const int, const SongList &, const int, const bool)>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 4, 5 }, { QMetaType::Int, 43 }, { QMetaType::Bool, 44 },
        }}),
        // Slot 'InsertSongs'
        QtMocHelpers::SlotData<void(const int, const SongList &, const int)>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 4, 5 }, { QMetaType::Int, 43 },
        }}),
        // Slot 'InsertSongs'
        QtMocHelpers::SlotData<void(const int, const SongList &)>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'RemoveItemsWithoutUndo'
        QtMocHelpers::SlotData<void(const int, const QList<int> &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { 0x80000000 | 23, 49 },
        }}),
        // Slot 'RemoveCurrentSong'
        QtMocHelpers::SlotData<void() const>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlaySmartPlaylist'
        QtMocHelpers::SlotData<void(PlaylistGeneratorPtr, const bool, const bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 52, 53 }, { QMetaType::Bool, 54 }, { QMetaType::Bool, 55 },
        }}),
        // Slot 'RateCurrentSong'
        QtMocHelpers::SlotData<void(const float)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 57 },
        }}),
        // Slot 'RateCurrentSong2'
        QtMocHelpers::SlotData<void(const int)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 57 },
        }}),
        // Slot 'SaveAllPlaylists'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetActivePlaying'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetActivePaused'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetActiveStopped'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OneOfPlaylistsChanged'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UpdateSummaryText'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UpdateCollectionSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'ItemsLoadedForSavePlaylist'
        QtMocHelpers::SlotData<void(const QString &, const SongList &, const QString &, const PlaylistSettings::PathType)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 8 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'PlaylistLoaded'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistManager, qt_meta_tag_ZN15PlaylistManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistManager::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistManagerInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlaylistManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlaylistManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PlaylistManagerE_t>.metaTypes,
    nullptr
} };

void PlaylistManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->New((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->New((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2]))); break;
        case 2: _t->New((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->Load((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->Save((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<PlaylistSettings::PathType>>(_a[4]))); break;
        case 5: _t->SaveWithUI((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->Rename((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->Favorite((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->Delete((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: { bool _r = _t->Close((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->Open((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->ChangePlaylistOrder((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 12: _t->SetCurrentPlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->SetActivePlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->SetActiveToCurrent(); break;
        case 15: _t->SelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 16: _t->SetCurrentOrOpen((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->ClearCurrent(); break;
        case 18: _t->ShuffleCurrent(); break;
        case 19: _t->RemoveDuplicatesCurrent(); break;
        case 20: _t->RemoveUnavailableCurrent(); break;
        case 21: _t->SongChangeRequestProcessed((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->InsertUrls((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 23: _t->InsertUrls((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 24: _t->InsertUrls((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 25: _t->InsertUrls((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 26: _t->InsertUrls((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2]))); break;
        case 27: _t->InsertSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 28: _t->InsertSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 29: _t->InsertSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 30: _t->InsertSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 31: _t->InsertSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2]))); break;
        case 32: _t->RemoveItemsWithoutUndo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 33: _t->RemoveCurrentSong(); break;
        case 34: _t->PlaySmartPlaylist((*reinterpret_cast<std::add_pointer_t<PlaylistGeneratorPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 35: _t->RateCurrentSong((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 36: _t->RateCurrentSong2((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->SaveAllPlaylists(); break;
        case 38: _t->SetActivePlaying(); break;
        case 39: _t->SetActivePaused(); break;
        case 40: _t->SetActiveStopped(); break;
        case 41: _t->OneOfPlaylistsChanged(); break;
        case 42: _t->UpdateSummaryText(); break;
        case 43: _t->UpdateCollectionSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 44: _t->ItemsLoadedForSavePlaylist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<PlaylistSettings::PathType>>(_a[4]))); break;
        case 45: _t->PlaylistLoaded(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *PlaylistManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlaylistManagerE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistManagerInterface::qt_metacast(_clname);
}

int PlaylistManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistManagerInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 46)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 46;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 46)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 46;
    }
    return _id;
}
QT_WARNING_POP
