/****************************************************************************
** Meta object code from reading C++ file 'enginebase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/engine/enginebase.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'enginebase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10EngineBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto EngineBase::qt_create_metaobjectdata<qt_meta_tag_ZN10EngineBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EngineBase",
        "TrackAboutToEnd",
        "",
        "TrackEnded",
        "StatusText",
        "text",
        "Error",
        "FatalError",
        "InvalidSongRequested",
        "QUrl",
        "url",
        "ValidSongRequested",
        "MetaData",
        "EngineMetadata",
        "metadata",
        "StateChanged",
        "EngineBase::State",
        "state",
        "VolumeChanged",
        "volume",
        "Finished",
        "ReloadSettings",
        "UpdateVolume",
        "EmitAboutToFinish",
        "UpdateSpotifyAccessToken",
        "spotify_access_token",
        "SetStereoBalancerEnabled",
        "SetStereoBalance",
        "SetEqualizerEnabled",
        "SetEqualizerParameters",
        "QList<int>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'TrackAboutToEnd'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TrackEnded'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'StatusText'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'FatalError'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'InvalidSongRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'ValidSongRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'MetaData'
        QtMocHelpers::SignalData<void(const EngineMetadata &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(const EngineBase::State)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'VolumeChanged'
        QtMocHelpers::SignalData<void(const uint)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 19 },
        }}),
        // Signal 'Finished'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateVolume'
        QtMocHelpers::SlotData<void(const uint)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 19 },
        }}),
        // Slot 'EmitAboutToFinish'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateSpotifyAccessToken'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'SetStereoBalancerEnabled'
        QtMocHelpers::SlotData<void(const bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'SetStereoBalance'
        QtMocHelpers::SlotData<void(const float)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 2 },
        }}),
        // Slot 'SetEqualizerEnabled'
        QtMocHelpers::SlotData<void(const bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'SetEqualizerParameters'
        QtMocHelpers::SlotData<void(const int, const QList<int> &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { 0x80000000 | 30, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EngineBase, qt_meta_tag_ZN10EngineBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EngineBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10EngineBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10EngineBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10EngineBaseE_t>.metaTypes,
    nullptr
} };

void EngineBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EngineBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->TrackAboutToEnd(); break;
        case 1: _t->TrackEnded(); break;
        case 2: _t->StatusText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->FatalError(); break;
        case 5: _t->InvalidSongRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 6: _t->ValidSongRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 7: _t->MetaData((*reinterpret_cast<std::add_pointer_t<EngineMetadata>>(_a[1]))); break;
        case 8: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<EngineBase::State>>(_a[1]))); break;
        case 9: _t->VolumeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 10: _t->Finished(); break;
        case 11: _t->ReloadSettings(); break;
        case 12: _t->UpdateVolume((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 13: _t->EmitAboutToFinish(); break;
        case 14: _t->UpdateSpotifyAccessToken((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->SetStereoBalancerEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->SetStereoBalance((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 17: _t->SetEqualizerEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->SetEqualizerParameters((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EngineBase::State >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)()>(_a, &EngineBase::TrackAboutToEnd, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)()>(_a, &EngineBase::TrackEnded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const QString & )>(_a, &EngineBase::StatusText, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const QString & )>(_a, &EngineBase::Error, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)()>(_a, &EngineBase::FatalError, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const QUrl & )>(_a, &EngineBase::InvalidSongRequested, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const QUrl & )>(_a, &EngineBase::ValidSongRequested, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const EngineMetadata & )>(_a, &EngineBase::MetaData, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const EngineBase::State )>(_a, &EngineBase::StateChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)(const uint )>(_a, &EngineBase::VolumeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBase::*)()>(_a, &EngineBase::Finished, 10))
            return;
    }
}

const QMetaObject *EngineBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EngineBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10EngineBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int EngineBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void EngineBase::TrackAboutToEnd()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void EngineBase::TrackEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void EngineBase::StatusText(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void EngineBase::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void EngineBase::FatalError()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void EngineBase::InvalidSongRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void EngineBase::ValidSongRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void EngineBase::MetaData(const EngineMetadata & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void EngineBase::StateChanged(const EngineBase::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void EngineBase::VolumeChanged(const uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void EngineBase::Finished()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
