/****************************************************************************
** Meta object code from reading C++ file 'mtpdevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/device/mtpdevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mtpdevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9MtpDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto MtpDevice::qt_create_metaobjectdata<qt_meta_tag_ZN9MtpDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MtpDevice",
        "LoadFinished",
        "",
        "success",
        "MtpConnection*",
        "connection",
        "LoaderError",
        "message",
        "QUrl",
        "url",
        "DeviceLister*",
        "lister",
        "unique_id",
        "DeviceManager*",
        "device_manager",
        "SharedPtr<TaskManager>",
        "task_manager",
        "SharedPtr<Database>",
        "database",
        "SharedPtr<TagReaderClient>",
        "tagreader_client",
        "SharedPtr<AlbumCoverLoader>",
        "albumcover_loader",
        "database_id",
        "first_time",
        "parent"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'LoadFinished'
        QtMocHelpers::SlotData<void(bool, MtpConnection *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'LoaderError'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(const QUrl &, DeviceLister *, const QString &, DeviceManager *, const SharedPtr<TaskManager>, const SharedPtr<Database>, const SharedPtr<TagReaderClient>, const SharedPtr<AlbumCoverLoader>, const int, const bool, QObject *)>(2, QMC::AccessPublic, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 }, { QMetaType::QString, 12 }, { 0x80000000 | 13, 14 },
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
            { QMetaType::Int, 23 }, { QMetaType::Bool, 24 }, { QMetaType::QObjectStar, 25 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const QUrl &, DeviceLister *, const QString &, DeviceManager *, const SharedPtr<TaskManager>, const SharedPtr<Database>, const SharedPtr<TagReaderClient>, const SharedPtr<AlbumCoverLoader>, const int, const bool)>(2, QMC::AccessPublic | QMC::MethodCloned, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 }, { QMetaType::QString, 12 }, { 0x80000000 | 13, 14 },
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
            { QMetaType::Int, 23 }, { QMetaType::Bool, 24 },
        }}),
    };
    return QtMocHelpers::metaObjectData<MtpDevice, qt_meta_tag_ZN9MtpDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject MtpDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<ConnectedDevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MtpDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MtpDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9MtpDeviceE_t>.metaTypes,
    nullptr
} };

void MtpDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MtpDevice *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { MtpDevice *_r = new MtpDevice((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeviceLister*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<DeviceManager*>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TaskManager>>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<AlbumCoverLoader>>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[11])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { MtpDevice *_r = new MtpDevice((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeviceLister*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<DeviceManager*>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TaskManager>>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<AlbumCoverLoader>>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) MtpDevice((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeviceLister*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<DeviceManager*>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TaskManager>>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<AlbumCoverLoader>>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[11]))); } break;
        case 1: { new (_a[0]) MtpDevice((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeviceLister*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<DeviceManager*>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TaskManager>>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<AlbumCoverLoader>>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->LoadFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MtpConnection*>>(_a[2]))); break;
        case 1: _t->LoaderError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *MtpDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MtpDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MtpDeviceE_t>.strings))
        return static_cast<void*>(this);
    return ConnectedDevice::qt_metacast(_clname);
}

int MtpDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConnectedDevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
