/****************************************************************************
** Meta object code from reading C++ file 'spotifyservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/spotify/spotifyservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'spotifyservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14SpotifyServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto SpotifyService::qt_create_metaobjectdata<qt_meta_tag_ZN14SpotifyServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SpotifyService",
        "UpdateSpotifyAccessToken",
        "",
        "access_token",
        "Authenticate",
        "ClearSession",
        "GetArtists",
        "GetAlbums",
        "GetSongs",
        "ResetArtistsRequest",
        "ResetAlbumsRequest",
        "ResetSongsRequest",
        "ExitReceived",
        "OAuthFinished",
        "success",
        "error",
        "StartSearch",
        "ArtistsResultsReceived",
        "id",
        "SongMap",
        "songs",
        "AlbumsResultsReceived",
        "SongsResultsReceived",
        "SearchResultsReceived",
        "ArtistsUpdateStatusReceived",
        "text",
        "AlbumsUpdateStatusReceived",
        "SongsUpdateStatusReceived",
        "ArtistsProgressSetMaximumReceived",
        "max",
        "AlbumsProgressSetMaximumReceived",
        "SongsProgressSetMaximumReceived",
        "ArtistsUpdateProgressReceived",
        "progress",
        "AlbumsUpdateProgressReceived",
        "SongsUpdateProgressReceived"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'UpdateSpotifyAccessToken'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'Authenticate'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ClearSession'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetArtists'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetAlbums'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetSongs'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetArtistsRequest'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetAlbumsRequest'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetSongsRequest'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ExitReceived'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OAuthFinished'
        QtMocHelpers::SlotData<void(const bool, const QString &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'OAuthFinished'
        QtMocHelpers::SlotData<void(const bool)>(13, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'StartSearch'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ArtistsResultsReceived'
        QtMocHelpers::SlotData<void(const int, const SongMap &, const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'AlbumsResultsReceived'
        QtMocHelpers::SlotData<void(const int, const SongMap &, const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'SongsResultsReceived'
        QtMocHelpers::SlotData<void(const int, const SongMap &, const QString &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'SearchResultsReceived'
        QtMocHelpers::SlotData<void(const int, const SongMap &, const QString &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'ArtistsUpdateStatusReceived'
        QtMocHelpers::SlotData<void(const int, const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QString, 25 },
        }}),
        // Slot 'AlbumsUpdateStatusReceived'
        QtMocHelpers::SlotData<void(const int, const QString &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QString, 25 },
        }}),
        // Slot 'SongsUpdateStatusReceived'
        QtMocHelpers::SlotData<void(const int, const QString &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QString, 25 },
        }}),
        // Slot 'ArtistsProgressSetMaximumReceived'
        QtMocHelpers::SlotData<void(const int, const int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 29 },
        }}),
        // Slot 'AlbumsProgressSetMaximumReceived'
        QtMocHelpers::SlotData<void(const int, const int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 29 },
        }}),
        // Slot 'SongsProgressSetMaximumReceived'
        QtMocHelpers::SlotData<void(const int, const int)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 29 },
        }}),
        // Slot 'ArtistsUpdateProgressReceived'
        QtMocHelpers::SlotData<void(const int, const int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 33 },
        }}),
        // Slot 'AlbumsUpdateProgressReceived'
        QtMocHelpers::SlotData<void(const int, const int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 33 },
        }}),
        // Slot 'SongsUpdateProgressReceived'
        QtMocHelpers::SlotData<void(const int, const int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SpotifyService, qt_meta_tag_ZN14SpotifyServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SpotifyService::staticMetaObject = { {
    QMetaObject::SuperData::link<StreamingService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SpotifyServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SpotifyServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14SpotifyServiceE_t>.metaTypes,
    nullptr
} };

void SpotifyService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SpotifyService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->UpdateSpotifyAccessToken((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->Authenticate(); break;
        case 2: _t->ClearSession(); break;
        case 3: _t->GetArtists(); break;
        case 4: _t->GetAlbums(); break;
        case 5: _t->GetSongs(); break;
        case 6: _t->ResetArtistsRequest(); break;
        case 7: _t->ResetAlbumsRequest(); break;
        case 8: _t->ResetSongsRequest(); break;
        case 9: _t->ExitReceived(); break;
        case 10: _t->OAuthFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->OAuthFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->StartSearch(); break;
        case 13: _t->ArtistsResultsReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 14: _t->AlbumsResultsReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 15: _t->SongsResultsReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 16: _t->SearchResultsReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 17: _t->ArtistsUpdateStatusReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->AlbumsUpdateStatusReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 19: _t->SongsUpdateStatusReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->ArtistsProgressSetMaximumReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 21: _t->AlbumsProgressSetMaximumReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 22: _t->SongsProgressSetMaximumReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->ArtistsUpdateProgressReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 24: _t->AlbumsUpdateProgressReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 25: _t->SongsUpdateProgressReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SpotifyService::*)(const QString & )>(_a, &SpotifyService::UpdateSpotifyAccessToken, 0))
            return;
    }
}

const QMetaObject *SpotifyService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpotifyService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SpotifyServiceE_t>.strings))
        return static_cast<void*>(this);
    return StreamingService::qt_metacast(_clname);
}

int SpotifyService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StreamingService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void SpotifyService::UpdateSpotifyAccessToken(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
