/****************************************************************************
** Meta object code from reading C++ file 'albumcovermanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/covermanager/albumcovermanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'albumcovermanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17AlbumCoverManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto AlbumCoverManager::qt_create_metaobjectdata<qt_meta_tag_ZN17AlbumCoverManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AlbumCoverManager",
        "Error",
        "",
        "error",
        "AddToPlaylist",
        "QMimeData*",
        "data",
        "ArtistChanged",
        "QListWidgetItem*",
        "current",
        "LoadAlbumCovers",
        "AlbumCoverLoaded",
        "id",
        "AlbumCoverLoaderResult",
        "result",
        "UpdateFilter",
        "FetchAlbumCovers",
        "ExportCovers",
        "AlbumCoverFetched",
        "AlbumCoverImageResult",
        "CoverSearchStatistics",
        "statistics",
        "CancelRequests",
        "FetchSingleCover",
        "LoadCoverFromFile",
        "SaveCoverToFile",
        "LoadCoverFromURL",
        "SearchForCover",
        "UnsetCover",
        "ClearCover",
        "DeleteCover",
        "ShowCover",
        "AlbumDoubleClicked",
        "QModelIndex",
        "idx",
        "AddSelectedToPlaylist",
        "LoadSelectedToPlaylist",
        "UpdateCoverInList",
        "AlbumItem*",
        "album_item",
        "QUrl",
        "cover",
        "UpdateExportStatus",
        "exported",
        "skipped",
        "max",
        "SaveEmbeddedCoverFinished",
        "TagReaderReplyPtr",
        "reply",
        "url",
        "art_embedded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'AddToPlaylist'
        QtMocHelpers::SignalData<void(QMimeData *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'ArtistChanged'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'LoadAlbumCovers'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const quint64, const AlbumCoverLoaderResult &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'UpdateFilter'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FetchAlbumCovers'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ExportCovers'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AlbumCoverFetched'
        QtMocHelpers::SlotData<void(const quint64, const AlbumCoverImageResult &, const CoverSearchStatistics &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 12 }, { 0x80000000 | 19, 14 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'CancelRequests'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FetchSingleCover'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadCoverFromFile'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SaveCoverToFile'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadCoverFromURL'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchForCover'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UnsetCover'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ClearCover'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteCover'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowCover'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AlbumDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'AddSelectedToPlaylist'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadSelectedToPlaylist'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UpdateCoverInList'
        QtMocHelpers::SlotData<void(AlbumItem *, const QUrl &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 39 }, { 0x80000000 | 40, 41 },
        }}),
        // Slot 'UpdateExportStatus'
        QtMocHelpers::SlotData<void(const int, const int, const int)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 43 }, { QMetaType::Int, 44 }, { QMetaType::Int, 45 },
        }}),
        // Slot 'SaveEmbeddedCoverFinished'
        QtMocHelpers::SlotData<void(TagReaderReplyPtr, AlbumItem *, const QUrl &, const bool)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 48 }, { 0x80000000 | 38, 39 }, { 0x80000000 | 40, 49 }, { QMetaType::Bool, 50 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AlbumCoverManager, qt_meta_tag_ZN17AlbumCoverManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AlbumCoverManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AlbumCoverManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AlbumCoverManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17AlbumCoverManagerE_t>.metaTypes,
    nullptr
} };

void AlbumCoverManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AlbumCoverManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->AddToPlaylist((*reinterpret_cast<std::add_pointer_t<QMimeData*>>(_a[1]))); break;
        case 2: _t->ArtistChanged((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 3: _t->LoadAlbumCovers(); break;
        case 4: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 5: _t->UpdateFilter(); break;
        case 6: _t->FetchAlbumCovers(); break;
        case 7: _t->ExportCovers(); break;
        case 8: _t->AlbumCoverFetched((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverImageResult>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CoverSearchStatistics>>(_a[3]))); break;
        case 9: _t->CancelRequests(); break;
        case 10: _t->FetchSingleCover(); break;
        case 11: _t->LoadCoverFromFile(); break;
        case 12: _t->SaveCoverToFile(); break;
        case 13: _t->LoadCoverFromURL(); break;
        case 14: _t->SearchForCover(); break;
        case 15: _t->UnsetCover(); break;
        case 16: _t->ClearCover(); break;
        case 17: _t->DeleteCover(); break;
        case 18: _t->ShowCover(); break;
        case 19: _t->AlbumDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 20: _t->AddSelectedToPlaylist(); break;
        case 21: _t->LoadSelectedToPlaylist(); break;
        case 22: _t->UpdateCoverInList((*reinterpret_cast<std::add_pointer_t<AlbumItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 23: _t->UpdateExportStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 24: _t->SaveEmbeddedCoverFinished((*reinterpret_cast<std::add_pointer_t<TagReaderReplyPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AlbumCoverLoaderResult >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AlbumCoverImageResult >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AlbumCoverManager::*)(const QString & )>(_a, &AlbumCoverManager::Error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AlbumCoverManager::*)(QMimeData * )>(_a, &AlbumCoverManager::AddToPlaylist, 1))
            return;
    }
}

const QMetaObject *AlbumCoverManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AlbumCoverManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AlbumCoverManagerE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int AlbumCoverManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void AlbumCoverManager::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AlbumCoverManager::AddToPlaylist(QMimeData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
