/*
 * Strawberry Music Player
 * Copyright 2024, Jonas Kvinge <jonas@jkvinge.net>
 *
 * Strawberry is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Strawberry is distributed in the hope that it wiLL be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Strawberry.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef TRANSCODERSETTINGS_H
#define TRANSCODERSETTINGS_H

namespace TranscoderSettings {

constexpr char kSettingsGroup[] = "Transcoder";
constexpr char kDefaultLastOutputFormat[] = "audio/x-vorbis";

namespace LameMP3Settings {
constexpr char kTarget[] = "target";
constexpr char kQuality[] = "quality";
constexpr char kBitrate[] = "bitrate";
constexpr char kCbr[] = "cbr";
constexpr char kEncodingEngineQuality[] = "encoding-engine-quality";
constexpr char kMono[] = "mono";
}  // namespace LameMP3Settings

}  // namespace TranscoderSettings

#endif  // TRANSCODERSETTINGS_H
