/****************************************************************************
** Meta object code from reading C++ file 'player.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/core/player.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'player.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6PlayerE_t {};
} // unnamed namespace

template <> constexpr inline auto Player::qt_create_metaobjectdata<qt_meta_tag_ZN6PlayerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Player",
        "ReloadSettings",
        "",
        "LoadVolume",
        "SaveVolume",
        "SavePlaybackStatus",
        "PlaylistsLoaded",
        "PlayAt",
        "index",
        "pause",
        "offset_nanosec",
        "EngineBase::TrackChangeFlags",
        "change",
        "Playlist::AutoScroll",
        "autoscroll",
        "reshuffle",
        "force_inform",
        "PlayPause",
        "PlayPauseHelper",
        "RestartOrPrevious",
        "Next",
        "Previous",
        "PlayPlaylist",
        "playlist_name",
        "SetVolumeFromSlider",
        "value",
        "SetVolumeFromEngine",
        "volume",
        "SetVolume",
        "VolumeUp",
        "VolumeDown",
        "SeekTo",
        "seconds",
        "SeekForward",
        "SeekBackward",
        "CurrentMetadataChanged",
        "Song",
        "metadata",
        "Mute",
        "Pause",
        "Stop",
        "stop_after",
        "StopAfterCurrent",
        "Play",
        "PlayWithPause",
        "PlayHelper",
        "ShowOSD",
        "TogglePrettyOSD",
        "HandleAuthentication",
        "UrlHandlerRegistered",
        "UrlHandler*",
        "url_handler",
        "EngineStateChanged",
        "EngineBase::State",
        "EngineMetadataReceived",
        "EngineMetadata",
        "engine_metadata",
        "TrackAboutToEnd",
        "TrackEnded",
        "NextItem",
        "PreviousItem",
        "NextInternal",
        "PlayPlaylistInternal",
        "FatalError",
        "ValidSongRequested",
        "QUrl",
        "url",
        "InvalidSongRequested",
        "HandleLoadResult",
        "UrlHandler::LoadResult",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'LoadVolume'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SaveVolume'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SavePlaybackStatus'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlaylistsLoaded'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlayAt'
        QtMocHelpers::SlotData<void(const int, const bool, const quint64, EngineBase::TrackChangeFlags, const Playlist::AutoScroll, const bool, const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 9 }, { QMetaType::ULongLong, 10 }, { 0x80000000 | 11, 12 },
            { 0x80000000 | 13, 14 }, { QMetaType::Bool, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'PlayAt'
        QtMocHelpers::SlotData<void(const int, const bool, const quint64, EngineBase::TrackChangeFlags, const Playlist::AutoScroll, const bool)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 9 }, { QMetaType::ULongLong, 10 }, { 0x80000000 | 11, 12 },
            { 0x80000000 | 13, 14 }, { QMetaType::Bool, 15 },
        }}),
        // Slot 'PlayPause'
        QtMocHelpers::SlotData<void(const quint64, const Playlist::AutoScroll)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 10 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'PlayPause'
        QtMocHelpers::SlotData<void(const quint64)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::ULongLong, 10 },
        }}),
        // Slot 'PlayPause'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'PlayPauseHelper'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RestartOrPrevious'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Next'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Previous'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlayPlaylist'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'SetVolumeFromSlider'
        QtMocHelpers::SlotData<void(const int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'SetVolumeFromEngine'
        QtMocHelpers::SlotData<void(const uint)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 27 },
        }}),
        // Slot 'SetVolume'
        QtMocHelpers::SlotData<void(const uint)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 27 },
        }}),
        // Slot 'VolumeUp'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'VolumeDown'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SeekTo'
        QtMocHelpers::SlotData<void(const quint64)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 32 },
        }}),
        // Slot 'SeekForward'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SeekBackward'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'CurrentMetadataChanged'
        QtMocHelpers::SlotData<void(const Song &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'Mute'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Pause'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Stop'
        QtMocHelpers::SlotData<void(const bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 41 },
        }}),
        // Slot 'Stop'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'StopAfterCurrent'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Play'
        QtMocHelpers::SlotData<void(const quint64)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 10 },
        }}),
        // Slot 'Play'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'PlayWithPause'
        QtMocHelpers::SlotData<void(const quint64)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 10 },
        }}),
        // Slot 'PlayHelper'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ShowOSD'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'TogglePrettyOSD'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'HandleAuthentication'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UrlHandlerRegistered'
        QtMocHelpers::SlotData<void(UrlHandler *) const>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'EngineStateChanged'
        QtMocHelpers::SlotData<void(const EngineBase::State)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 53, 2 },
        }}),
        // Slot 'EngineMetadataReceived'
        QtMocHelpers::SlotData<void(const EngineMetadata &)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'TrackAboutToEnd'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'TrackEnded'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'NextItem'
        QtMocHelpers::SlotData<void(const EngineBase::TrackChangeFlags, const Playlist::AutoScroll)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'PreviousItem'
        QtMocHelpers::SlotData<void(const EngineBase::TrackChangeFlags)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'NextInternal'
        QtMocHelpers::SlotData<void(const EngineBase::TrackChangeFlags, const Playlist::AutoScroll)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 2 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'PlayPlaylistInternal'
        QtMocHelpers::SlotData<void(const EngineBase::TrackChangeFlags, const Playlist::AutoScroll, const QString &)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 2 }, { 0x80000000 | 13, 14 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'FatalError'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ValidSongRequested'
        QtMocHelpers::SlotData<void(const QUrl &)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 65, 66 },
        }}),
        // Slot 'InvalidSongRequested'
        QtMocHelpers::SlotData<void(const QUrl &)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 65, 66 },
        }}),
        // Slot 'HandleLoadResult'
        QtMocHelpers::SlotData<void(const UrlHandler::LoadResult &)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 69, 70 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Player, qt_meta_tag_ZN6PlayerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Player::staticMetaObject = { {
    QMetaObject::SuperData::link<PlayerInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6PlayerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6PlayerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6PlayerE_t>.metaTypes,
    nullptr
} };

void Player::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Player *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ReloadSettings(); break;
        case 1: _t->LoadVolume(); break;
        case 2: _t->SaveVolume(); break;
        case 3: _t->SavePlaybackStatus(); break;
        case 4: _t->PlaylistsLoaded(); break;
        case 5: _t->PlayAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7]))); break;
        case 6: _t->PlayAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 7: _t->PlayPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 8: _t->PlayPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 9: _t->PlayPause(); break;
        case 10: _t->PlayPauseHelper(); break;
        case 11: _t->RestartOrPrevious(); break;
        case 12: _t->Next(); break;
        case 13: _t->Previous(); break;
        case 14: _t->PlayPlaylist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->SetVolumeFromSlider((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->SetVolumeFromEngine((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 17: _t->SetVolume((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 18: _t->VolumeUp(); break;
        case 19: _t->VolumeDown(); break;
        case 20: _t->SeekTo((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 21: _t->SeekForward(); break;
        case 22: _t->SeekBackward(); break;
        case 23: _t->CurrentMetadataChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 24: _t->Mute(); break;
        case 25: _t->Pause(); break;
        case 26: _t->Stop((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->Stop(); break;
        case 28: _t->StopAfterCurrent(); break;
        case 29: _t->Play((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 30: _t->Play(); break;
        case 31: _t->PlayWithPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 32: _t->PlayHelper(); break;
        case 33: _t->ShowOSD(); break;
        case 34: _t->TogglePrettyOSD(); break;
        case 35: _t->HandleAuthentication(); break;
        case 36: _t->UrlHandlerRegistered((*reinterpret_cast<std::add_pointer_t<UrlHandler*>>(_a[1]))); break;
        case 37: _t->EngineStateChanged((*reinterpret_cast<std::add_pointer_t<EngineBase::State>>(_a[1]))); break;
        case 38: _t->EngineMetadataReceived((*reinterpret_cast<std::add_pointer_t<EngineMetadata>>(_a[1]))); break;
        case 39: _t->TrackAboutToEnd(); break;
        case 40: _t->TrackEnded(); break;
        case 41: _t->NextItem((*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 42: _t->PreviousItem((*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[1]))); break;
        case 43: _t->NextInternal((*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 44: _t->PlayPlaylistInternal((*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 45: _t->FatalError(); break;
        case 46: _t->ValidSongRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 47: _t->InvalidSongRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 48: _t->HandleLoadResult((*reinterpret_cast<std::add_pointer_t<UrlHandler::LoadResult>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Player::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Player::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6PlayerE_t>.strings))
        return static_cast<void*>(this);
    return PlayerInterface::qt_metacast(_clname);
}

int Player::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlayerInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 49;
    }
    return _id;
}
QT_WARNING_POP
