/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.mpris.MediaPlayer2.TrackList.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "mpris2_tracklist.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Mpris2TrackList
 */

Mpris2TrackList::Mpris2TrackList(mpris::Mpris2 *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Mpris2TrackList::~Mpris2TrackList()
{
    // destructor
}

bool Mpris2TrackList::canEditTracks() const
{
    // get the value of property CanEditTracks
    return qvariant_cast< bool >(parent()->property("CanEditTracks"));
}

QList<QDBusObjectPath> Mpris2TrackList::tracks() const
{
    // get the value of property Tracks
    return qvariant_cast< QList<QDBusObjectPath> >(parent()->property("Tracks"));
}

void Mpris2TrackList::AddTrack(const QString &Uri, const QDBusObjectPath &AfterTrack, bool SetAsCurrent)
{
    // handle method call org.mpris.MediaPlayer2.TrackList.AddTrack
    parent()->AddTrack(Uri, AfterTrack, SetAsCurrent);
}

TrackMetadata Mpris2TrackList::GetTracksMetadata(const QList<QDBusObjectPath> &TrackIds)
{
    // handle method call org.mpris.MediaPlayer2.TrackList.GetTracksMetadata
    return parent()->GetTracksMetadata(TrackIds);
}

void Mpris2TrackList::GoTo(const QDBusObjectPath &TrackId)
{
    // handle method call org.mpris.MediaPlayer2.TrackList.GoTo
    parent()->GoTo(TrackId);
}

void Mpris2TrackList::RemoveTrack(const QDBusObjectPath &TrackId)
{
    // handle method call org.mpris.MediaPlayer2.TrackList.RemoveTrack
    parent()->RemoveTrack(TrackId);
}


#include "moc_mpris2_tracklist.cpp"
