/****************************************************************************
** Meta object code from reading C++ file 'playlistdelegates.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistdelegates.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistdelegates.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QueuedItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto QueuedItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN18QueuedItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QueuedItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QueuedItemDelegate, qt_meta_tag_ZN18QueuedItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QueuedItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QStyledItemDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QueuedItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QueuedItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QueuedItemDelegateE_t>.metaTypes,
    nullptr
} };

void QueuedItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QueuedItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QueuedItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QueuedItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QueuedItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return QStyledItemDelegate::qt_metacast(_clname);
}

int QueuedItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStyledItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20PlaylistDelegateBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistDelegateBase::qt_create_metaobjectdata<qt_meta_tag_ZN20PlaylistDelegateBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistDelegateBase",
        "helpEvent",
        "",
        "QHelpEvent*",
        "event",
        "QAbstractItemView*",
        "view",
        "QStyleOptionViewItem",
        "option",
        "QModelIndex",
        "idx"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'helpEvent'
        QtMocHelpers::SlotData<bool(QHelpEvent *, QAbstractItemView *, const QStyleOptionViewItem &, const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistDelegateBase, qt_meta_tag_ZN20PlaylistDelegateBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistDelegateBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QueuedItemDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PlaylistDelegateBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PlaylistDelegateBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20PlaylistDelegateBaseE_t>.metaTypes,
    nullptr
} };

void PlaylistDelegateBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistDelegateBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->helpEvent((*reinterpret_cast<std::add_pointer_t<QHelpEvent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemView*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStyleOptionViewItem>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *PlaylistDelegateBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistDelegateBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PlaylistDelegateBaseE_t>.strings))
        return static_cast<void*>(this);
    return QueuedItemDelegate::qt_metacast(_clname);
}

int PlaylistDelegateBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QueuedItemDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN18LengthItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto LengthItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN18LengthItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LengthItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LengthItemDelegate, qt_meta_tag_ZN18LengthItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LengthItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18LengthItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18LengthItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18LengthItemDelegateE_t>.metaTypes,
    nullptr
} };

void LengthItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LengthItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LengthItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LengthItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18LengthItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int LengthItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16SizeItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto SizeItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN16SizeItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SizeItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SizeItemDelegate, qt_meta_tag_ZN16SizeItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SizeItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SizeItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SizeItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SizeItemDelegateE_t>.metaTypes,
    nullptr
} };

void SizeItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SizeItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SizeItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SizeItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SizeItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int SizeItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16DateItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto DateItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN16DateItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DateItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DateItemDelegate, qt_meta_tag_ZN16DateItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DateItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DateItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DateItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DateItemDelegateE_t>.metaTypes,
    nullptr
} };

void DateItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DateItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DateItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DateItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DateItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int DateItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN22LastPlayedItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto LastPlayedItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN22LastPlayedItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LastPlayedItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LastPlayedItemDelegate, qt_meta_tag_ZN22LastPlayedItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LastPlayedItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22LastPlayedItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22LastPlayedItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22LastPlayedItemDelegateE_t>.metaTypes,
    nullptr
} };

void LastPlayedItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LastPlayedItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LastPlayedItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LastPlayedItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22LastPlayedItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int LastPlayedItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20FileTypeItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto FileTypeItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN20FileTypeItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileTypeItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileTypeItemDelegate, qt_meta_tag_ZN20FileTypeItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileTypeItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20FileTypeItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20FileTypeItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20FileTypeItemDelegateE_t>.metaTypes,
    nullptr
} };

void FileTypeItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileTypeItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *FileTypeItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileTypeItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20FileTypeItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int FileTypeItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16TextItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto TextItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN16TextItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextItemDelegate, qt_meta_tag_ZN16TextItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TextItemDelegateE_t>.metaTypes,
    nullptr
} };

void TextItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TextItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int TextItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18TagCompletionModelE_t {};
} // unnamed namespace

template <> constexpr inline auto TagCompletionModel::qt_create_metaobjectdata<qt_meta_tag_ZN18TagCompletionModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagCompletionModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagCompletionModel, qt_meta_tag_ZN18TagCompletionModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagCompletionModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStringListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TagCompletionModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TagCompletionModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18TagCompletionModelE_t>.metaTypes,
    nullptr
} };

void TagCompletionModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagCompletionModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TagCompletionModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagCompletionModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TagCompletionModelE_t>.strings))
        return static_cast<void*>(this);
    return QStringListModel::qt_metacast(_clname);
}

int TagCompletionModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStringListModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12TagCompleterE_t {};
} // unnamed namespace

template <> constexpr inline auto TagCompleter::qt_create_metaobjectdata<qt_meta_tag_ZN12TagCompleterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagCompleter",
        "ModelReady",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ModelReady'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagCompleter, qt_meta_tag_ZN12TagCompleterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagCompleter::staticMetaObject = { {
    QMetaObject::SuperData::link<QCompleter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagCompleterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagCompleterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TagCompleterE_t>.metaTypes,
    nullptr
} };

void TagCompleter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagCompleter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ModelReady(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *TagCompleter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagCompleter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagCompleterE_t>.strings))
        return static_cast<void*>(this);
    return QCompleter::qt_metacast(_clname);
}

int TagCompleter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCompleter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN25TagCompletionItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto TagCompletionItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN25TagCompletionItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagCompletionItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagCompletionItemDelegate, qt_meta_tag_ZN25TagCompletionItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagCompletionItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25TagCompletionItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25TagCompletionItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25TagCompletionItemDelegateE_t>.metaTypes,
    nullptr
} };

void TagCompletionItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagCompletionItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TagCompletionItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagCompletionItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25TagCompletionItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int TagCompletionItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24NativeSeparatorsDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto NativeSeparatorsDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN24NativeSeparatorsDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NativeSeparatorsDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NativeSeparatorsDelegate, qt_meta_tag_ZN24NativeSeparatorsDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NativeSeparatorsDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24NativeSeparatorsDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24NativeSeparatorsDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24NativeSeparatorsDelegateE_t>.metaTypes,
    nullptr
} };

void NativeSeparatorsDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NativeSeparatorsDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *NativeSeparatorsDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NativeSeparatorsDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24NativeSeparatorsDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int NativeSeparatorsDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18SongSourceDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto SongSourceDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN18SongSourceDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SongSourceDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SongSourceDelegate, qt_meta_tag_ZN18SongSourceDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SongSourceDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SongSourceDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SongSourceDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18SongSourceDelegateE_t>.metaTypes,
    nullptr
} };

void SongSourceDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SongSourceDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SongSourceDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SongSourceDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SongSourceDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int SongSourceDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18RatingItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto RatingItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN18RatingItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RatingItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RatingItemDelegate, qt_meta_tag_ZN18RatingItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RatingItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18RatingItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18RatingItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18RatingItemDelegateE_t>.metaTypes,
    nullptr
} };

void RatingItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RatingItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *RatingItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RatingItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18RatingItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int RatingItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto Ebur128LoudnessLUFSItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Ebur128LoudnessLUFSItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Ebur128LoudnessLUFSItemDelegate, qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Ebur128LoudnessLUFSItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t>.metaTypes,
    nullptr
} };

void Ebur128LoudnessLUFSItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Ebur128LoudnessLUFSItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Ebur128LoudnessLUFSItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Ebur128LoudnessLUFSItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31Ebur128LoudnessLUFSItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int Ebur128LoudnessLUFSItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto Ebur128LoudnessRangeLUItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Ebur128LoudnessRangeLUItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Ebur128LoudnessRangeLUItemDelegate, qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Ebur128LoudnessRangeLUItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaylistDelegateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t>.metaTypes,
    nullptr
} };

void Ebur128LoudnessRangeLUItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Ebur128LoudnessRangeLUItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Ebur128LoudnessRangeLUItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Ebur128LoudnessRangeLUItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34Ebur128LoudnessRangeLUItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return PlaylistDelegateBase::qt_metacast(_clname);
}

int Ebur128LoudnessRangeLUItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaylistDelegateBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
