/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.mpris.MediaPlayer2.Playlists.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "mpris2_playlists.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Mpris2Playlists
 */

Mpris2Playlists::Mpris2Playlists(mpris::Mpris2 *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Mpris2Playlists::~Mpris2Playlists()
{
    // destructor
}

MaybePlaylist Mpris2Playlists::activePlaylist() const
{
    // get the value of property ActivePlaylist
    return qvariant_cast< MaybePlaylist >(parent()->property("ActivePlaylist"));
}

QStringList Mpris2Playlists::orderings() const
{
    // get the value of property Orderings
    return qvariant_cast< QStringList >(parent()->property("Orderings"));
}

uint Mpris2Playlists::playlistCount() const
{
    // get the value of property PlaylistCount
    return qvariant_cast< uint >(parent()->property("PlaylistCount"));
}

void Mpris2Playlists::ActivatePlaylist(const QDBusObjectPath &PlaylistId)
{
    // handle method call org.mpris.MediaPlayer2.Playlists.ActivatePlaylist
    parent()->ActivatePlaylist(PlaylistId);
}

MprisPlaylistList Mpris2Playlists::GetPlaylists(uint Index, uint MaxCount, const QString &Order, bool ReverseOrder)
{
    // handle method call org.mpris.MediaPlayer2.Playlists.GetPlaylists
    return parent()->GetPlaylists(Index, MaxCount, Order, ReverseOrder);
}


#include "moc_mpris2_playlists.cpp"
