/****************************************************************************
** Meta object code from reading C++ file 'subsonicrequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/subsonic/subsonicrequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'subsonicrequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15SubsonicRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto SubsonicRequest::qt_create_metaobjectdata<qt_meta_tag_ZN15SubsonicRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SubsonicRequest",
        "Results",
        "",
        "SongMap",
        "songs",
        "error",
        "UpdateStatus",
        "text",
        "ProgressSetMaximum",
        "max",
        "UpdateProgress",
        "progress",
        "AlbumsReplyReceived",
        "QNetworkReply*",
        "reply",
        "offset_requested",
        "size_requested",
        "AlbumSongsReplyReceived",
        "artist_id",
        "album_id",
        "album_artist",
        "AlbumCoverReceived",
        "SubsonicRequest::AlbumCoverRequest",
        "request"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Results'
        QtMocHelpers::SignalData<void(const SongMap &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'UpdateStatus'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'ProgressSetMaximum'
        QtMocHelpers::SignalData<void(const int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'UpdateProgress'
        QtMocHelpers::SignalData<void(const int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'AlbumsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'AlbumSongsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const QString &, const QString &, const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 20 },
        }}),
        // Slot 'AlbumCoverReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const SubsonicRequest::AlbumCoverRequest &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 22, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SubsonicRequest, qt_meta_tag_ZN15SubsonicRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SubsonicRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<SubsonicBaseRequest::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SubsonicRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SubsonicRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15SubsonicRequestE_t>.metaTypes,
    nullptr
} };

void SubsonicRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SubsonicRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Results((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->UpdateStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->ProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->UpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->AlbumsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->AlbumSongsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 6: _t->AlbumCoverReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SubsonicRequest::AlbumCoverRequest>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SubsonicRequest::*)(const SongMap & , const QString & )>(_a, &SubsonicRequest::Results, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubsonicRequest::*)(const QString & )>(_a, &SubsonicRequest::UpdateStatus, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubsonicRequest::*)(const int )>(_a, &SubsonicRequest::ProgressSetMaximum, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubsonicRequest::*)(const int )>(_a, &SubsonicRequest::UpdateProgress, 3))
            return;
    }
}

const QMetaObject *SubsonicRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SubsonicRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SubsonicRequestE_t>.strings))
        return static_cast<void*>(this);
    return SubsonicBaseRequest::qt_metacast(_clname);
}

int SubsonicRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SubsonicBaseRequest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void SubsonicRequest::Results(const SongMap & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void SubsonicRequest::UpdateStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SubsonicRequest::ProgressSetMaximum(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SubsonicRequest::UpdateProgress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
