/****************************************************************************
** Meta object code from reading C++ file 'collectionfilterwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/collection/collectionfilterwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionfilterwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22CollectionFilterWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionFilterWidget::qt_create_metaobjectdata<qt_meta_tag_ZN22CollectionFilterWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionFilterWidget",
        "UpPressed",
        "",
        "DownPressed",
        "ReturnPressed",
        "UpdateGroupByActions",
        "SetFilterMode",
        "CollectionFilterOptions::FilterMode",
        "filter_mode",
        "FocusOnFilter",
        "QKeyEvent*",
        "e",
        "GroupingChanged",
        "CollectionModel::Grouping",
        "g",
        "separate_albums_by_grouping",
        "GroupByClicked",
        "QAction*",
        "action",
        "SaveGroupBy",
        "ShowGroupingManager",
        "FilterTextChanged",
        "text",
        "FilterDelayTimeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'UpPressed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DownPressed'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ReturnPressed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateGroupByActions'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetFilterMode'
        QtMocHelpers::SlotData<void(CollectionFilterOptions::FilterMode)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'FocusOnFilter'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'GroupingChanged'
        QtMocHelpers::SlotData<void(const CollectionModel::Grouping, const bool)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::Bool, 15 },
        }}),
        // Slot 'GroupByClicked'
        QtMocHelpers::SlotData<void(QAction *)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'SaveGroupBy'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowGroupingManager'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FilterTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'FilterDelayTimeout'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CollectionFilterWidget, qt_meta_tag_ZN22CollectionFilterWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CollectionFilterWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22CollectionFilterWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22CollectionFilterWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22CollectionFilterWidgetE_t>.metaTypes,
    nullptr
} };

void CollectionFilterWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionFilterWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->UpPressed(); break;
        case 1: _t->DownPressed(); break;
        case 2: _t->ReturnPressed(); break;
        case 3: _t->UpdateGroupByActions(); break;
        case 4: _t->SetFilterMode((*reinterpret_cast<std::add_pointer_t<CollectionFilterOptions::FilterMode>>(_a[1]))); break;
        case 5: _t->FocusOnFilter((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 6: _t->GroupingChanged((*reinterpret_cast<std::add_pointer_t<CollectionModel::Grouping>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->GroupByClicked((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 8: _t->SaveGroupBy(); break;
        case 9: _t->ShowGroupingManager(); break;
        case 10: _t->FilterTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->FilterDelayTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionModel::Grouping >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionFilterWidget::*)()>(_a, &CollectionFilterWidget::UpPressed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFilterWidget::*)()>(_a, &CollectionFilterWidget::DownPressed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFilterWidget::*)()>(_a, &CollectionFilterWidget::ReturnPressed, 2))
            return;
    }
}

const QMetaObject *CollectionFilterWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionFilterWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22CollectionFilterWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int CollectionFilterWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void CollectionFilterWidget::UpPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CollectionFilterWidget::DownPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CollectionFilterWidget::ReturnPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
