/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.mpris.MediaPlayer2.TrackList.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef MPRIS2_TRACKLIST_H
#define MPRIS2_TRACKLIST_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "src/mpris2/mpris2.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.mpris.MediaPlayer2.TrackList
 */
class Mpris2TrackList: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.mpris.MediaPlayer2.TrackList")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.mpris.MediaPlayer2.TrackList\">\n"
"    <method name=\"GetTracksMetadata\">\n"
"      <arg direction=\"in\" type=\"ao\" name=\"TrackIds\"/>\n"
"      <arg direction=\"out\" type=\"aa{sv}\" name=\"Metadata\"/>\n"
"      <annotation value=\"TrackMetadata\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"AddTrack\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"Uri\"/>\n"
"      <arg direction=\"in\" type=\"o\" name=\"AfterTrack\"/>\n"
"      <arg direction=\"in\" type=\"b\" name=\"SetAsCurrent\"/>\n"
"    </method>\n"
"    <method name=\"RemoveTrack\">\n"
"      <arg direction=\"in\" type=\"o\" name=\"TrackId\"/>\n"
"    </method>\n"
"    <method name=\"GoTo\">\n"
"      <arg direction=\"in\" type=\"o\" name=\"TrackId\"/>\n"
"    </method>\n"
"    <signal name=\"TrackListReplaced\">\n"
"      <arg type=\"ao\" name=\"Tracks\"/>\n"
"      <arg type=\"o\" name=\"CurrentTrack\"/>\n"
"    </signal>\n"
"    <signal name=\"TrackAdded\">\n"
"      <arg type=\"a{sv}\" name=\"Metadata\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"      <arg type=\"o\" name=\"AfterTrack\"/>\n"
"    </signal>\n"
"    <signal name=\"TrackRemoved\">\n"
"      <arg type=\"o\" name=\"TrackId\"/>\n"
"    </signal>\n"
"    <signal name=\"TrackMetadataChanged\">\n"
"      <arg type=\"o\" name=\"TrackId\"/>\n"
"      <arg type=\"a{sv}\" name=\"Metadata\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"    </signal>\n"
"    <property access=\"read\" type=\"ao\" name=\"Tracks\"/>\n"
"    <property access=\"read\" type=\"b\" name=\"CanEditTracks\"/>\n"
"  </interface>\n"
        "")
public:
    Mpris2TrackList(mpris::Mpris2 *parent);
    ~Mpris2TrackList() override;

    inline mpris::Mpris2 *parent() const
    { return static_cast<mpris::Mpris2 *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(bool CanEditTracks READ canEditTracks)
    bool canEditTracks() const;

    Q_PROPERTY(QList<QDBusObjectPath> Tracks READ tracks)
    QList<QDBusObjectPath> tracks() const;

public Q_SLOTS: // METHODS
    void AddTrack(const QString &Uri, const QDBusObjectPath &AfterTrack, bool SetAsCurrent);
    TrackMetadata GetTracksMetadata(const QList<QDBusObjectPath> &TrackIds);
    void GoTo(const QDBusObjectPath &TrackId);
    void RemoveTrack(const QDBusObjectPath &TrackId);
Q_SIGNALS: // SIGNALS
    void TrackAdded(const QVariantMap &Metadata, const QDBusObjectPath &AfterTrack);
    void TrackListReplaced(const QList<QDBusObjectPath> &Tracks, const QDBusObjectPath &CurrentTrack);
    void TrackMetadataChanged(const QDBusObjectPath &TrackId, const QVariantMap &Metadata);
    void TrackRemoved(const QDBusObjectPath &TrackId);
};

#endif
