/****************************************************************************
** Meta object code from reading C++ file 'mpris2.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/mpris2/mpris2.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mpris2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mpris6Mpris2E_t {};
} // unnamed namespace

template <> constexpr inline auto mpris::Mpris2::qt_create_metaobjectdata<qt_meta_tag_ZN5mpris6Mpris2E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mpris::Mpris2",
        "Seeked",
        "",
        "position",
        "TrackListReplaced",
        "Track_Ids",
        "tracks",
        "QDBusObjectPath",
        "current_track",
        "TrackAdded",
        "QVariantMap",
        "metadata",
        "after_track",
        "TrackRemoved",
        "track_id",
        "TrackMetadataChanged",
        "RaiseMainWindow",
        "PlaylistChanged",
        "MprisPlaylist",
        "playlist",
        "AlbumCoverLoaded",
        "Song",
        "song",
        "AlbumCoverLoaderResult",
        "result",
        "EngineStateChanged",
        "EngineBase::State",
        "newState",
        "VolumeChanged",
        "PlaylistManagerInitialized",
        "AllPlaylistsLoaded",
        "CurrentSongChanged",
        "ShuffleModeChanged",
        "RepeatModeChanged",
        "PlaylistChangedSlot",
        "Playlist*",
        "PlaylistCollectionChanged",
        "PlaylistItemsAdded",
        "playlist_id",
        "QList<QUuid>",
        "track_ids",
        "QUuid",
        "after_track_id",
        "PlaylistItemsRemoved",
        "PlaylistItemMetadataChanged",
        "CanQuit",
        "CanRaise",
        "HasTrackList",
        "Identity",
        "DesktopEntry",
        "SupportedUriSchemes",
        "SupportedMimeTypes",
        "CanSetFullscreen",
        "Fullscreen",
        "PlaybackStatus",
        "LoopStatus",
        "Rate",
        "Shuffle",
        "Metadata",
        "Volume",
        "Position",
        "MinimumRate",
        "MaximumRate",
        "CanGoNext",
        "CanGoPrevious",
        "CanPlay",
        "CanPause",
        "CanSeek",
        "CanControl",
        "Tracks",
        "CanEditTracks",
        "PlaylistCount",
        "Orderings",
        "ActivePlaylist",
        "MaybePlaylist",
        "Rating"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Seeked'
        QtMocHelpers::SignalData<void(const qint64)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 3 },
        }}),
        // Signal 'TrackListReplaced'
        QtMocHelpers::SignalData<void(const Track_Ids &, const QDBusObjectPath &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'TrackAdded'
        QtMocHelpers::SignalData<void(const QVariantMap &, const QDBusObjectPath &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 7, 12 },
        }}),
        // Signal 'TrackRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 14 },
        }}),
        // Signal 'TrackMetadataChanged'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QVariantMap &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 14 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'RaiseMainWindow'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PlaylistChanged'
        QtMocHelpers::SignalData<void(const MprisPlaylist &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const Song &, const AlbumCoverLoaderResult &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const Song &)>(20, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'EngineStateChanged'
        QtMocHelpers::SlotData<void(EngineBase::State)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'VolumeChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistManagerInitialized'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AllPlaylistsLoaded'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CurrentSongChanged'
        QtMocHelpers::SlotData<void(const Song &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'ShuffleModeChanged'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RepeatModeChanged'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistChangedSlot'
        QtMocHelpers::SlotData<void(Playlist *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 19 },
        }}),
        // Slot 'PlaylistCollectionChanged'
        QtMocHelpers::SlotData<void(Playlist *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 19 },
        }}),
        // Slot 'PlaylistItemsAdded'
        QtMocHelpers::SlotData<void(const int, const QList<QUuid> &, const QUuid &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 38 }, { 0x80000000 | 39, 40 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'PlaylistItemsRemoved'
        QtMocHelpers::SlotData<void(const int, const QList<QUuid> &)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'PlaylistItemMetadataChanged'
        QtMocHelpers::SlotData<void(const int, const QUuid &)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 38 }, { 0x80000000 | 41, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'CanQuit'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanRaise'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'HasTrackList'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Identity'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'DesktopEntry'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'SupportedUriSchemes'
        QtMocHelpers::PropertyData<QStringList>(50, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'SupportedMimeTypes'
        QtMocHelpers::PropertyData<QStringList>(51, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'CanSetFullscreen'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Fullscreen'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'PlaybackStatus'
        QtMocHelpers::PropertyData<QString>(54, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'LoopStatus'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'Rate'
        QtMocHelpers::PropertyData<double>(56, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'Shuffle'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'Metadata'
        QtMocHelpers::PropertyData<QVariantMap>(58, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Volume'
        QtMocHelpers::PropertyData<double>(59, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'Position'
        QtMocHelpers::PropertyData<qint64>(60, QMetaType::LongLong, QMC::DefaultPropertyFlags),
        // property 'MinimumRate'
        QtMocHelpers::PropertyData<double>(61, QMetaType::Double, QMC::DefaultPropertyFlags),
        // property 'MaximumRate'
        QtMocHelpers::PropertyData<double>(62, QMetaType::Double, QMC::DefaultPropertyFlags),
        // property 'CanGoNext'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanGoPrevious'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanPlay'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanPause'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanSeek'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanControl'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Tracks'
        QtMocHelpers::PropertyData<Track_Ids>(69, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'CanEditTracks'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'PlaylistCount'
        QtMocHelpers::PropertyData<quint32>(71, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Orderings'
        QtMocHelpers::PropertyData<QStringList>(72, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'ActivePlaylist'
        QtMocHelpers::PropertyData<MaybePlaylist>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Rating'
        QtMocHelpers::PropertyData<double>(75, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Mpris2, qt_meta_tag_ZN5mpris6Mpris2E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject mpris::Mpris2::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mpris6Mpris2E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mpris6Mpris2E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5mpris6Mpris2E_t>.metaTypes,
    nullptr
} };

void mpris::Mpris2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Mpris2 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Seeked((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 1: _t->TrackListReplaced((*reinterpret_cast<std::add_pointer_t<Track_Ids>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 2: _t->TrackAdded((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 3: _t->TrackRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 4: _t->TrackMetadataChanged((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 5: _t->RaiseMainWindow(); break;
        case 6: _t->PlaylistChanged((*reinterpret_cast<std::add_pointer_t<MprisPlaylist>>(_a[1]))); break;
        case 7: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 8: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 9: _t->EngineStateChanged((*reinterpret_cast<std::add_pointer_t<EngineBase::State>>(_a[1]))); break;
        case 10: _t->VolumeChanged(); break;
        case 11: _t->PlaylistManagerInitialized(); break;
        case 12: _t->AllPlaylistsLoaded(); break;
        case 13: _t->CurrentSongChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 14: _t->ShuffleModeChanged(); break;
        case 15: _t->RepeatModeChanged(); break;
        case 16: _t->PlaylistChangedSlot((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 17: _t->PlaylistCollectionChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 18: _t->PlaylistItemsAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUuid>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[3]))); break;
        case 19: _t->PlaylistItemsRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUuid>>>(_a[2]))); break;
        case 20: _t->PlaylistItemMetadataChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MprisPlaylist >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)(const qint64 )>(_a, &Mpris2::Seeked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)(const Track_Ids & , const QDBusObjectPath & )>(_a, &Mpris2::TrackListReplaced, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)(const QVariantMap & , const QDBusObjectPath & )>(_a, &Mpris2::TrackAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)(const QDBusObjectPath & )>(_a, &Mpris2::TrackRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)(const QDBusObjectPath & , const QVariantMap & )>(_a, &Mpris2::TrackMetadataChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)()>(_a, &Mpris2::RaiseMainWindow, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2::*)(const MprisPlaylist & )>(_a, &Mpris2::PlaylistChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 28:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< MaybePlaylist >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->CanQuit(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->CanRaise(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->HasTrackList(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->Identity(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->DesktopEntry(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->SupportedUriSchemes(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->SupportedMimeTypes(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->CanSetFullscreen(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->Fullscreen(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->PlaybackStatus(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->LoopStatus(); break;
        case 11: *reinterpret_cast<double*>(_v) = _t->Rate(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->Shuffle(); break;
        case 13: *reinterpret_cast<QVariantMap*>(_v) = _t->Metadata(); break;
        case 14: *reinterpret_cast<double*>(_v) = _t->Volume(); break;
        case 15: *reinterpret_cast<qint64*>(_v) = _t->Position(); break;
        case 16: *reinterpret_cast<double*>(_v) = _t->MinimumRate(); break;
        case 17: *reinterpret_cast<double*>(_v) = _t->MaximumRate(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->CanGoNext(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->CanGoPrevious(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->CanPlay(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->CanPause(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->CanSeek(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->CanControl(); break;
        case 24: *reinterpret_cast<Track_Ids*>(_v) = _t->Tracks(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->CanEditTracks(); break;
        case 26: *reinterpret_cast<quint32*>(_v) = _t->PlaylistCount(); break;
        case 27: *reinterpret_cast<QStringList*>(_v) = _t->Orderings(); break;
        case 28: *reinterpret_cast<MaybePlaylist*>(_v) = _t->ActivePlaylist(); break;
        case 29: *reinterpret_cast<double*>(_v) = _t->Rating(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 8: _t->SetFullscreen(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->SetLoopStatus(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->SetRate(*reinterpret_cast<double*>(_v)); break;
        case 12: _t->SetShuffle(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->SetVolume(*reinterpret_cast<double*>(_v)); break;
        case 29: _t->SetRating(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *mpris::Mpris2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mpris::Mpris2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mpris6Mpris2E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int mpris::Mpris2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    return _id;
}

// SIGNAL 0
void mpris::Mpris2::Seeked(const qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void mpris::Mpris2::TrackListReplaced(const Track_Ids & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void mpris::Mpris2::TrackAdded(const QVariantMap & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void mpris::Mpris2::TrackRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void mpris::Mpris2::TrackMetadataChanged(const QDBusObjectPath & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void mpris::Mpris2::RaiseMainWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void mpris::Mpris2::PlaylistChanged(const MprisPlaylist & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
