/****************************************************************************
** Meta object code from reading C++ file 'streamingservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/streaming/streamingservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'streamingservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16StreamingServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto StreamingService::qt_create_metaobjectdata<qt_meta_tag_ZN16StreamingServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StreamingService",
        "ExitFinished",
        "",
        "RequestLogin",
        "RequestLogout",
        "LoginWithCredentials",
        "api_token",
        "username",
        "password",
        "LoginSuccess",
        "LoginFailure",
        "error",
        "LoginFinished",
        "success",
        "TestSuccess",
        "TestFailure",
        "TestComplete",
        "ShowErrorDialog",
        "Results",
        "SongMap",
        "songs",
        "UpdateStatus",
        "text",
        "ProgressSetMaximum",
        "max",
        "UpdateProgress",
        "ArtistsResults",
        "ArtistsUpdateStatus",
        "ArtistsProgressSetMaximum",
        "ArtistsUpdateProgress",
        "AlbumsResults",
        "AlbumsUpdateStatus",
        "AlbumsProgressSetMaximum",
        "AlbumsUpdateProgress",
        "SongsResults",
        "SongsUpdateStatus",
        "SongsProgressSetMaximum",
        "SongsUpdateProgress",
        "SearchResults",
        "id",
        "SearchUpdateStatus",
        "SearchProgressSetMaximum",
        "SearchUpdateProgress",
        "AddArtists",
        "SongList",
        "AddAlbums",
        "AddSongs",
        "RemoveArtists",
        "RemoveAlbums",
        "RemoveSongsByList",
        "RemoveSongsByMap",
        "StreamURLFailure",
        "QUrl",
        "media_url",
        "StreamURLSuccess",
        "stream_url",
        "Song::FileType",
        "filetype",
        "samplerate",
        "bit_depth",
        "duration",
        "StreamURLRequestFinished",
        "OpenSettingsDialog",
        "Song::Source",
        "source",
        "Configure",
        "GetArtists",
        "GetAlbums",
        "GetSongs",
        "ResetArtistsRequest",
        "ResetAlbumsRequest",
        "ResetSongsRequest"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ExitFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RequestLogin'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RequestLogout'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LoginWithCredentials'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'LoginSuccess'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LoginFailure'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'LoginFinished'
        QtMocHelpers::SignalData<void(const bool, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'LoginFinished'
        QtMocHelpers::SignalData<void(const bool)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'TestSuccess'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TestFailure'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'TestComplete'
        QtMocHelpers::SignalData<void(const bool, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'TestComplete'
        QtMocHelpers::SignalData<void(const bool)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'ShowErrorDialog'
        QtMocHelpers::SignalData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'Results'
        QtMocHelpers::SignalData<void(const SongMap &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'UpdateStatus'
        QtMocHelpers::SignalData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'ProgressSetMaximum'
        QtMocHelpers::SignalData<void(const int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'UpdateProgress'
        QtMocHelpers::SignalData<void(const int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'ArtistsResults'
        QtMocHelpers::SignalData<void(const SongMap &, const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'ArtistsUpdateStatus'
        QtMocHelpers::SignalData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'ArtistsProgressSetMaximum'
        QtMocHelpers::SignalData<void(const int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'ArtistsUpdateProgress'
        QtMocHelpers::SignalData<void(const int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'AlbumsResults'
        QtMocHelpers::SignalData<void(const SongMap &, const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'AlbumsUpdateStatus'
        QtMocHelpers::SignalData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'AlbumsProgressSetMaximum'
        QtMocHelpers::SignalData<void(const int)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'AlbumsUpdateProgress'
        QtMocHelpers::SignalData<void(const int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'SongsResults'
        QtMocHelpers::SignalData<void(const SongMap &, const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'SongsUpdateStatus'
        QtMocHelpers::SignalData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'SongsProgressSetMaximum'
        QtMocHelpers::SignalData<void(const int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'SongsUpdateProgress'
        QtMocHelpers::SignalData<void(const int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'SearchResults'
        QtMocHelpers::SignalData<void(const int, const SongMap &, const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'SearchUpdateStatus'
        QtMocHelpers::SignalData<void(const int, const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 }, { QMetaType::QString, 22 },
        }}),
        // Signal 'SearchProgressSetMaximum'
        QtMocHelpers::SignalData<void(const int, const int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 }, { QMetaType::Int, 24 },
        }}),
        // Signal 'SearchUpdateProgress'
        QtMocHelpers::SignalData<void(const int, const int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 }, { QMetaType::Int, 24 },
        }}),
        // Signal 'AddArtists'
        QtMocHelpers::SignalData<void(const SongList &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 20 },
        }}),
        // Signal 'AddAlbums'
        QtMocHelpers::SignalData<void(const SongList &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 20 },
        }}),
        // Signal 'AddSongs'
        QtMocHelpers::SignalData<void(const SongList &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 20 },
        }}),
        // Signal 'RemoveArtists'
        QtMocHelpers::SignalData<void(const SongList &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 20 },
        }}),
        // Signal 'RemoveAlbums'
        QtMocHelpers::SignalData<void(const SongList &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 20 },
        }}),
        // Signal 'RemoveSongsByList'
        QtMocHelpers::SignalData<void(const SongList &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 20 },
        }}),
        // Signal 'RemoveSongsByMap'
        QtMocHelpers::SignalData<void(const SongMap &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'StreamURLFailure'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QString &)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 39 }, { 0x80000000 | 52, 53 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'StreamURLSuccess'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QUrl &, const Song::FileType, const int, const int, const qint64)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 39 }, { 0x80000000 | 52, 53 }, { 0x80000000 | 52, 55 }, { 0x80000000 | 56, 57 },
            { QMetaType::Int, 58 }, { QMetaType::Int, 59 }, { QMetaType::LongLong, 60 },
        }}),
        // Signal 'StreamURLRequestFinished'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const bool, const QUrl &, const QString &)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 39 }, { 0x80000000 | 52, 53 }, { QMetaType::Bool, 13 }, { 0x80000000 | 52, 55 },
            { QMetaType::QString, 11 },
        }}),
        // Signal 'StreamURLRequestFinished'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const bool, const QUrl &)>(61, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::UInt, 39 }, { 0x80000000 | 52, 53 }, { QMetaType::Bool, 13 }, { 0x80000000 | 52, 55 },
        }}),
        // Signal 'OpenSettingsDialog'
        QtMocHelpers::SignalData<void(const Song::Source)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'Configure'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetArtists'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetAlbums'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetSongs'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetArtistsRequest'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetAlbumsRequest'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetSongsRequest'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StreamingService, qt_meta_tag_ZN16StreamingServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StreamingService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16StreamingServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16StreamingServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16StreamingServiceE_t>.metaTypes,
    nullptr
} };

void StreamingService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StreamingService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ExitFinished(); break;
        case 1: _t->RequestLogin(); break;
        case 2: _t->RequestLogout(); break;
        case 3: _t->LoginWithCredentials((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 4: _t->LoginSuccess(); break;
        case 5: _t->LoginFailure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->LoginFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->LoginFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->TestSuccess(); break;
        case 9: _t->TestFailure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->TestComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->TestComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->ShowErrorDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->Results((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->UpdateStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->ProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->UpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->ArtistsResults((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->ArtistsUpdateStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->ArtistsProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->ArtistsUpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->AlbumsResults((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->AlbumsUpdateStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->AlbumsProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->AlbumsUpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->SongsResults((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 26: _t->SongsUpdateStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->SongsProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->SongsUpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->SearchResults((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 30: _t->SearchUpdateStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 31: _t->SearchProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 32: _t->SearchUpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 33: _t->AddArtists((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 34: _t->AddAlbums((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 35: _t->AddSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 36: _t->RemoveArtists((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 37: _t->RemoveAlbums((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 38: _t->RemoveSongsByList((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 39: _t->RemoveSongsByMap((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1]))); break;
        case 40: _t->StreamURLFailure((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 41: _t->StreamURLSuccess((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[7]))); break;
        case 42: _t->StreamURLRequestFinished((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 43: _t->StreamURLRequestFinished((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[4]))); break;
        case 44: _t->OpenSettingsDialog((*reinterpret_cast<std::add_pointer_t<Song::Source>>(_a[1]))); break;
        case 45: _t->Configure(); break;
        case 46: _t->GetArtists(); break;
        case 47: _t->GetAlbums(); break;
        case 48: _t->GetSongs(); break;
        case 49: _t->ResetArtistsRequest(); break;
        case 50: _t->ResetAlbumsRequest(); break;
        case 51: _t->ResetSongsRequest(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)()>(_a, &StreamingService::ExitFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)()>(_a, &StreamingService::RequestLogin, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)()>(_a, &StreamingService::RequestLogout, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & , const QString & , const QString & )>(_a, &StreamingService::LoginWithCredentials, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)()>(_a, &StreamingService::LoginSuccess, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::LoginFailure, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const bool , const QString & )>(_a, &StreamingService::LoginFinished, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)()>(_a, &StreamingService::TestSuccess, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::TestFailure, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const bool , const QString & )>(_a, &StreamingService::TestComplete, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::ShowErrorDialog, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongMap & , const QString & )>(_a, &StreamingService::Results, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::UpdateStatus, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::ProgressSetMaximum, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::UpdateProgress, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongMap & , const QString & )>(_a, &StreamingService::ArtistsResults, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::ArtistsUpdateStatus, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::ArtistsProgressSetMaximum, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::ArtistsUpdateProgress, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongMap & , const QString & )>(_a, &StreamingService::AlbumsResults, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::AlbumsUpdateStatus, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::AlbumsProgressSetMaximum, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::AlbumsUpdateProgress, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongMap & , const QString & )>(_a, &StreamingService::SongsResults, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const QString & )>(_a, &StreamingService::SongsUpdateStatus, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::SongsProgressSetMaximum, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int )>(_a, &StreamingService::SongsUpdateProgress, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int , const SongMap & , const QString & )>(_a, &StreamingService::SearchResults, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int , const QString & )>(_a, &StreamingService::SearchUpdateStatus, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int , const int )>(_a, &StreamingService::SearchProgressSetMaximum, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const int , const int )>(_a, &StreamingService::SearchUpdateProgress, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongList & )>(_a, &StreamingService::AddArtists, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongList & )>(_a, &StreamingService::AddAlbums, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongList & )>(_a, &StreamingService::AddSongs, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongList & )>(_a, &StreamingService::RemoveArtists, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongList & )>(_a, &StreamingService::RemoveAlbums, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongList & )>(_a, &StreamingService::RemoveSongsByList, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const SongMap & )>(_a, &StreamingService::RemoveSongsByMap, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const uint , const QUrl & , const QString & )>(_a, &StreamingService::StreamURLFailure, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const uint , const QUrl & , const QUrl & , const Song::FileType , const int , const int , const qint64 )>(_a, &StreamingService::StreamURLSuccess, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const uint , const QUrl & , const bool , const QUrl & , const QString & )>(_a, &StreamingService::StreamURLRequestFinished, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingService::*)(const Song::Source )>(_a, &StreamingService::OpenSettingsDialog, 44))
            return;
    }
}

const QMetaObject *StreamingService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StreamingService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16StreamingServiceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StreamingService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 52)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 52;
    }
    return _id;
}

// SIGNAL 0
void StreamingService::ExitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void StreamingService::RequestLogin()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void StreamingService::RequestLogout()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void StreamingService::LoginWithCredentials(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void StreamingService::LoginSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void StreamingService::LoginFailure(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StreamingService::LoginFinished(const bool _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 8
void StreamingService::TestSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void StreamingService::TestFailure(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void StreamingService::TestComplete(const bool _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 12
void StreamingService::ShowErrorDialog(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void StreamingService::Results(const SongMap & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void StreamingService::UpdateStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void StreamingService::ProgressSetMaximum(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void StreamingService::UpdateProgress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void StreamingService::ArtistsResults(const SongMap & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2);
}

// SIGNAL 18
void StreamingService::ArtistsUpdateStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void StreamingService::ArtistsProgressSetMaximum(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void StreamingService::ArtistsUpdateProgress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void StreamingService::AlbumsResults(const SongMap & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void StreamingService::AlbumsUpdateStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void StreamingService::AlbumsProgressSetMaximum(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void StreamingService::AlbumsUpdateProgress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void StreamingService::SongsResults(const SongMap & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1, _t2);
}

// SIGNAL 26
void StreamingService::SongsUpdateStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void StreamingService::SongsProgressSetMaximum(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void StreamingService::SongsUpdateProgress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void StreamingService::SearchResults(const int _t1, const SongMap & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1, _t2, _t3);
}

// SIGNAL 30
void StreamingService::SearchUpdateStatus(const int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1, _t2);
}

// SIGNAL 31
void StreamingService::SearchProgressSetMaximum(const int _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1, _t2);
}

// SIGNAL 32
void StreamingService::SearchUpdateProgress(const int _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1, _t2);
}

// SIGNAL 33
void StreamingService::AddArtists(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void StreamingService::AddAlbums(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1);
}

// SIGNAL 35
void StreamingService::AddSongs(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void StreamingService::RemoveArtists(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void StreamingService::RemoveAlbums(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1);
}

// SIGNAL 38
void StreamingService::RemoveSongsByList(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void StreamingService::RemoveSongsByMap(const SongMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}

// SIGNAL 40
void StreamingService::StreamURLFailure(const uint _t1, const QUrl & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1, _t2, _t3);
}

// SIGNAL 41
void StreamingService::StreamURLSuccess(const uint _t1, const QUrl & _t2, const QUrl & _t3, const Song::FileType _t4, const int _t5, const int _t6, const qint64 _t7)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7);
}

// SIGNAL 42
void StreamingService::StreamURLRequestFinished(const uint _t1, const QUrl & _t2, const bool _t3, const QUrl & _t4, const QString & _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 44
void StreamingService::OpenSettingsDialog(const Song::Source _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}
QT_WARNING_POP
