/****************************************************************************
** Meta object code from reading C++ file 'gstenginepipeline.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/engine/gstenginepipeline.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gstenginepipeline.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17GstEnginePipelineE_t {};
} // unnamed namespace

template <> constexpr inline auto GstEnginePipeline::qt_create_metaobjectdata<qt_meta_tag_ZN17GstEnginePipelineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GstEnginePipeline",
        "SetStateFinished",
        "",
        "GstStateChangeReturn",
        "state_change_return",
        "Error",
        "pipeline_id",
        "domain",
        "error_code",
        "message",
        "debug",
        "EndOfStreamReached",
        "has_next_track",
        "MetadataFound",
        "EngineMetadata",
        "bundle",
        "AboutToFinish",
        "Finished",
        "VolumeChanged",
        "volume",
        "FaderFinished",
        "BufferingStarted",
        "BufferingProgress",
        "percent",
        "BufferingFinished",
        "SetStateAsyncSlot",
        "GstState",
        "state",
        "SetStateFinishedSlot",
        "SetFaderVolume",
        "FaderTimelineStateChanged",
        "QTimeLine::State",
        "FaderTimelineFinished",
        "FaderTimelineTimeout",
        "FaderFudgeFinished",
        "SetState",
        "QFuture<GstStateChangeReturn>",
        "Play",
        "pause",
        "offset_nanosec",
        "Seek",
        "nanosec"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SetStateFinished'
        QtMocHelpers::SignalData<void(const GstStateChangeReturn)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const int, const int, const int, const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 9 },
            { QMetaType::QString, 10 },
        }}),
        // Signal 'EndOfStreamReached'
        QtMocHelpers::SignalData<void(const int, const bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'MetadataFound'
        QtMocHelpers::SignalData<void(const int, const EngineMetadata &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'AboutToFinish'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Finished'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'VolumeChanged'
        QtMocHelpers::SignalData<void(const uint)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 19 },
        }}),
        // Signal 'FaderFinished'
        QtMocHelpers::SignalData<void(const int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'BufferingStarted'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'BufferingProgress'
        QtMocHelpers::SignalData<void(const int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Signal 'BufferingFinished'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetStateAsyncSlot'
        QtMocHelpers::SlotData<void(const GstState)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'SetStateFinishedSlot'
        QtMocHelpers::SlotData<void(const GstState, const GstStateChangeReturn)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'SetFaderVolume'
        QtMocHelpers::SlotData<void(const qreal)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 19 },
        }}),
        // Slot 'FaderTimelineStateChanged'
        QtMocHelpers::SlotData<void(const QTimeLine::State)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 27 },
        }}),
        // Slot 'FaderTimelineFinished'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FaderTimelineTimeout'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FaderFudgeFinished'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'SetState'
        QtMocHelpers::MethodData<QFuture<GstStateChangeReturn>(const GstState)>(35, 2, QMC::AccessPublic, 0x80000000 | 36, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'Play'
        QtMocHelpers::MethodData<QFuture<GstStateChangeReturn>(const bool, const quint64)>(37, 2, QMC::AccessPublic, 0x80000000 | 36, {{
            { QMetaType::Bool, 38 }, { QMetaType::ULongLong, 39 },
        }}),
        // Method 'Seek'
        QtMocHelpers::MethodData<bool(const qint64)>(40, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::LongLong, 41 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GstEnginePipeline, qt_meta_tag_ZN17GstEnginePipelineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GstEnginePipeline::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17GstEnginePipelineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17GstEnginePipelineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17GstEnginePipelineE_t>.metaTypes,
    nullptr
} };

void GstEnginePipeline::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GstEnginePipeline *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SetStateFinished((*reinterpret_cast<std::add_pointer_t<GstStateChangeReturn>>(_a[1]))); break;
        case 1: _t->Error((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 2: _t->EndOfStreamReached((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->MetadataFound((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<EngineMetadata>>(_a[2]))); break;
        case 4: _t->AboutToFinish(); break;
        case 5: _t->Finished(); break;
        case 6: _t->VolumeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 7: _t->FaderFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->BufferingStarted(); break;
        case 9: _t->BufferingProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->BufferingFinished(); break;
        case 11: _t->SetStateAsyncSlot((*reinterpret_cast<std::add_pointer_t<GstState>>(_a[1]))); break;
        case 12: _t->SetStateFinishedSlot((*reinterpret_cast<std::add_pointer_t<GstState>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GstStateChangeReturn>>(_a[2]))); break;
        case 13: _t->SetFaderVolume((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 14: _t->FaderTimelineStateChanged((*reinterpret_cast<std::add_pointer_t<QTimeLine::State>>(_a[1]))); break;
        case 15: _t->FaderTimelineFinished(); break;
        case 16: _t->FaderTimelineTimeout(); break;
        case 17: _t->FaderFudgeFinished(); break;
        case 18: { QFuture<GstStateChangeReturn> _r = _t->SetState((*reinterpret_cast<std::add_pointer_t<GstState>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QFuture<GstStateChangeReturn>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QFuture<GstStateChangeReturn> _r = _t->Play((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QFuture<GstStateChangeReturn>*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->Seek((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const GstStateChangeReturn )>(_a, &GstEnginePipeline::SetStateFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const int , const int , const int , const QString & , const QString & )>(_a, &GstEnginePipeline::Error, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const int , const bool )>(_a, &GstEnginePipeline::EndOfStreamReached, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const int , const EngineMetadata & )>(_a, &GstEnginePipeline::MetadataFound, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)()>(_a, &GstEnginePipeline::AboutToFinish, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)()>(_a, &GstEnginePipeline::Finished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const uint )>(_a, &GstEnginePipeline::VolumeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const int )>(_a, &GstEnginePipeline::FaderFinished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)()>(_a, &GstEnginePipeline::BufferingStarted, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)(const int )>(_a, &GstEnginePipeline::BufferingProgress, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (GstEnginePipeline::*)()>(_a, &GstEnginePipeline::BufferingFinished, 10))
            return;
    }
}

const QMetaObject *GstEnginePipeline::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GstEnginePipeline::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17GstEnginePipelineE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GstEnginePipeline::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void GstEnginePipeline::SetStateFinished(const GstStateChangeReturn _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GstEnginePipeline::Error(const int _t1, const int _t2, const int _t3, const QString & _t4, const QString & _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 2
void GstEnginePipeline::EndOfStreamReached(const int _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void GstEnginePipeline::MetadataFound(const int _t1, const EngineMetadata & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void GstEnginePipeline::AboutToFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void GstEnginePipeline::Finished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void GstEnginePipeline::VolumeChanged(const uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void GstEnginePipeline::FaderFinished(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void GstEnginePipeline::BufferingStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void GstEnginePipeline::BufferingProgress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void GstEnginePipeline::BufferingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
