/****************************************************************************
** Meta object code from reading C++ file 'playlistbackend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistbackend.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistbackend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PlaylistBackendE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistBackend::qt_create_metaobjectdata<qt_meta_tag_ZN15PlaylistBackendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistBackend",
        "ExitFinished",
        "",
        "Exit",
        "SavePlaylist",
        "playlist",
        "PlaylistItemPtrList",
        "items",
        "last_played",
        "PlaylistGeneratorPtr",
        "dynamic",
        "SharedPtr<Database>",
        "database",
        "SharedPtr<TagReaderClient>",
        "tagreader_client",
        "SharedPtr<CollectionBackend>",
        "collection_backend",
        "parent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ExitFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Exit'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SavePlaylist'
        QtMocHelpers::SlotData<void(const int, const PlaylistItemPtrList &, const int, PlaylistGeneratorPtr)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Int, 8 }, { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(const SharedPtr<Database>, const SharedPtr<TagReaderClient>, const SharedPtr<CollectionBackend>, QObject *)>(2, QMC::AccessPublic, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 }, { QMetaType::QObjectStar, 17 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const SharedPtr<Database>, const SharedPtr<TagReaderClient>, const SharedPtr<CollectionBackend>)>(2, QMC::AccessPublic | QMC::MethodCloned, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
    };
    return QtMocHelpers::metaObjectData<PlaylistBackend, qt_meta_tag_ZN15PlaylistBackendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject PlaylistBackend::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlaylistBackendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlaylistBackendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PlaylistBackendE_t>.metaTypes,
    nullptr
} };

void PlaylistBackend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistBackend *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { PlaylistBackend *_r = new PlaylistBackend((*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<CollectionBackend>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { PlaylistBackend *_r = new PlaylistBackend((*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<CollectionBackend>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) PlaylistBackend((*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<CollectionBackend>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[4]))); } break;
        case 1: { new (_a[0]) PlaylistBackend((*reinterpret_cast<std::add_pointer_t<SharedPtr<Database>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<TagReaderClient>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SharedPtr<CollectionBackend>>>(_a[3]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ExitFinished(); break;
        case 1: _t->Exit(); break;
        case 2: _t->SavePlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlaylistItemPtrList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<PlaylistGeneratorPtr>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlaylistItemPtrList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistBackend::*)()>(_a, &PlaylistBackend::ExitFinished, 0))
            return;
    }
}

const QMetaObject *PlaylistBackend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistBackend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlaylistBackendE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PlaylistBackend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void PlaylistBackend::ExitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
