/****************************************************************************
** Meta object code from reading C++ file 'fileview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/fileview/fileview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8FileViewE_t {};
} // unnamed namespace

template <> constexpr inline auto FileView::qt_create_metaobjectdata<qt_meta_tag_ZN8FileViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileView",
        "PathChanged",
        "",
        "path",
        "AddToPlaylist",
        "QMimeData*",
        "data",
        "CopyToCollection",
        "QList<QUrl>",
        "urls",
        "MoveToCollection",
        "CopyToDevice",
        "EditTags",
        "FileUp",
        "FileHome",
        "ChangeFilePath",
        "new_path",
        "ItemActivated",
        "QModelIndex",
        "idx",
        "ItemDoubleClick",
        "ToggleViewMode",
        "Delete",
        "filenames",
        "DeleteFinished",
        "SongList",
        "songs_with_errors",
        "AddTreeRootPath",
        "RemoveTreeRootPath"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PathChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'AddToPlaylist'
        QtMocHelpers::SignalData<void(QMimeData *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'CopyToCollection'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'MoveToCollection'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'CopyToDevice'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'EditTags'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'FileUp'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FileHome'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ChangeFilePath'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'ItemActivated'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'ItemDoubleClick'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'ToggleViewMode'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Delete'
        QtMocHelpers::SlotData<void(const QStringList &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 23 },
        }}),
        // Slot 'DeleteFinished'
        QtMocHelpers::SlotData<void(const SongList &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'AddTreeRootPath'
        QtMocHelpers::SlotData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'RemoveTreeRootPath'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileView, qt_meta_tag_ZN8FileViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8FileViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8FileViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8FileViewE_t>.metaTypes,
    nullptr
} };

void FileView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->AddToPlaylist((*reinterpret_cast<std::add_pointer_t<QMimeData*>>(_a[1]))); break;
        case 2: _t->CopyToCollection((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 3: _t->MoveToCollection((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 4: _t->CopyToDevice((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 5: _t->EditTags((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 6: _t->FileUp(); break;
        case 7: _t->FileHome(); break;
        case 8: _t->ChangeFilePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->ItemActivated((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 10: _t->ItemDoubleClick((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->ToggleViewMode(); break;
        case 12: _t->Delete((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 13: _t->DeleteFinished((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 14: _t->AddTreeRootPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->RemoveTreeRootPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileView::*)(const QString & )>(_a, &FileView::PathChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileView::*)(QMimeData * )>(_a, &FileView::AddToPlaylist, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileView::*)(const QList<QUrl> & )>(_a, &FileView::CopyToCollection, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileView::*)(const QList<QUrl> & )>(_a, &FileView::MoveToCollection, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileView::*)(const QList<QUrl> & )>(_a, &FileView::CopyToDevice, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileView::*)(const QList<QUrl> & )>(_a, &FileView::EditTags, 5))
            return;
    }
}

const QMetaObject *FileView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8FileViewE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int FileView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void FileView::PathChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FileView::AddToPlaylist(QMimeData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FileView::CopyToCollection(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FileView::MoveToCollection(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void FileView::CopyToDevice(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void FileView::EditTags(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
