/****************************************************************************
** Meta object code from reading C++ file 'streamingsearchview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/streaming/streamingsearchview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'streamingsearchview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19StreamingSearchViewE_t {};
} // unnamed namespace

template <> constexpr inline auto StreamingSearchView::qt_create_metaobjectdata<qt_meta_tag_ZN19StreamingSearchViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StreamingSearchView",
        "AddToPlaylist",
        "",
        "QMimeData*",
        "mimedata",
        "AddArtistsSignal",
        "SongList",
        "songs",
        "AddAlbumsSignal",
        "AddSongsSignal",
        "OpenSettingsDialog",
        "Song::Source",
        "source",
        "SwapModels",
        "TextEdited",
        "text",
        "StartSearch",
        "query",
        "SearchDone",
        "service_id",
        "SongMap",
        "error",
        "UpdateStatus",
        "ProgressSetMaximum",
        "max",
        "UpdateProgress",
        "progress",
        "AddResults",
        "StreamingSearchView::ResultList",
        "results",
        "FocusOnFilter",
        "QKeyEvent*",
        "e",
        "AddSelectedToPlaylist",
        "LoadSelected",
        "OpenSelectedInNewPlaylist",
        "AddSelectedToPlaylistEnqueue",
        "AddArtists",
        "AddAlbums",
        "AddSongs",
        "SearchForThis",
        "Configure",
        "SearchArtistsClicked",
        "checked",
        "SearchAlbumsClicked",
        "SearchSongsClicked",
        "GroupByClicked",
        "QAction*",
        "action",
        "SetGroupBy",
        "CollectionModel::Grouping",
        "g",
        "AlbumCoverLoaded",
        "id",
        "AlbumCoverLoaderResult",
        "albumcover_result",
        "ReloadSettings"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AddToPlaylist'
        QtMocHelpers::SignalData<void(QMimeData *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'AddArtistsSignal'
        QtMocHelpers::SignalData<void(const SongList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'AddAlbumsSignal'
        QtMocHelpers::SignalData<void(const SongList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'AddSongsSignal'
        QtMocHelpers::SignalData<void(const SongList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'OpenSettingsDialog'
        QtMocHelpers::SignalData<void(const Song::Source)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'SwapModels'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'TextEdited'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'StartSearch'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'SearchDone'
        QtMocHelpers::SlotData<void(const int, const SongMap &, const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { 0x80000000 | 20, 7 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'UpdateStatus'
        QtMocHelpers::SlotData<void(const int, const QString &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'ProgressSetMaximum'
        QtMocHelpers::SlotData<void(const int, const int)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'UpdateProgress'
        QtMocHelpers::SlotData<void(const int, const int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 26 },
        }}),
        // Slot 'AddResults'
        QtMocHelpers::SlotData<void(const int, const StreamingSearchView::ResultList &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'FocusOnFilter'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'AddSelectedToPlaylist'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadSelected'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OpenSelectedInNewPlaylist'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddSelectedToPlaylistEnqueue'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddArtists'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddAlbums'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddSongs'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchForThis'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Configure'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchArtistsClicked'
        QtMocHelpers::SlotData<void(const bool)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'SearchAlbumsClicked'
        QtMocHelpers::SlotData<void(const bool)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'SearchSongsClicked'
        QtMocHelpers::SlotData<void(const bool)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'GroupByClicked'
        QtMocHelpers::SlotData<void(QAction *)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Slot 'SetGroupBy'
        QtMocHelpers::SlotData<void(const CollectionModel::Grouping)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const quint64, const AlbumCoverLoaderResult &)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 53 }, { 0x80000000 | 54, 55 },
        }}),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StreamingSearchView, qt_meta_tag_ZN19StreamingSearchViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StreamingSearchView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19StreamingSearchViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19StreamingSearchViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19StreamingSearchViewE_t>.metaTypes,
    nullptr
} };

void StreamingSearchView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StreamingSearchView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AddToPlaylist((*reinterpret_cast<std::add_pointer_t<QMimeData*>>(_a[1]))); break;
        case 1: _t->AddArtistsSignal((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 2: _t->AddAlbumsSignal((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 3: _t->AddSongsSignal((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 4: _t->OpenSettingsDialog((*reinterpret_cast<std::add_pointer_t<Song::Source>>(_a[1]))); break;
        case 5: _t->SwapModels(); break;
        case 6: _t->TextEdited((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->StartSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->SearchDone((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 9: _t->UpdateStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->ProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->UpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->AddResults((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<StreamingSearchView::ResultList>>(_a[2]))); break;
        case 13: _t->FocusOnFilter((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 14: _t->AddSelectedToPlaylist(); break;
        case 15: _t->LoadSelected(); break;
        case 16: _t->OpenSelectedInNewPlaylist(); break;
        case 17: _t->AddSelectedToPlaylistEnqueue(); break;
        case 18: _t->AddArtists(); break;
        case 19: _t->AddAlbums(); break;
        case 20: _t->AddSongs(); break;
        case 21: _t->SearchForThis(); break;
        case 22: _t->Configure(); break;
        case 23: _t->SearchArtistsClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->SearchAlbumsClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->SearchSongsClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->GroupByClicked((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 27: _t->SetGroupBy((*reinterpret_cast<std::add_pointer_t<CollectionModel::Grouping>>(_a[1]))); break;
        case 28: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 29: _t->ReloadSettings(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StreamingSearchView::ResultList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StreamingSearchView::*)(QMimeData * )>(_a, &StreamingSearchView::AddToPlaylist, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingSearchView::*)(const SongList & )>(_a, &StreamingSearchView::AddArtistsSignal, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingSearchView::*)(const SongList & )>(_a, &StreamingSearchView::AddAlbumsSignal, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingSearchView::*)(const SongList & )>(_a, &StreamingSearchView::AddSongsSignal, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StreamingSearchView::*)(const Song::Source )>(_a, &StreamingSearchView::OpenSettingsDialog, 4))
            return;
    }
}

const QMetaObject *StreamingSearchView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StreamingSearchView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19StreamingSearchViewE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StreamingSearchView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    return _id;
}

// SIGNAL 0
void StreamingSearchView::AddToPlaylist(QMimeData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StreamingSearchView::AddArtistsSignal(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StreamingSearchView::AddAlbumsSignal(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StreamingSearchView::AddSongsSignal(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StreamingSearchView::OpenSettingsDialog(const Song::Source _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
