/****************************************************************************
** Meta object code from reading C++ file 'tidalfavoriterequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/tidal/tidalfavoriterequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tidalfavoriterequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TidalFavoriteRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto TidalFavoriteRequest::qt_create_metaobjectdata<qt_meta_tag_ZN20TidalFavoriteRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TidalFavoriteRequest",
        "ArtistsAdded",
        "",
        "SongList",
        "songs",
        "AlbumsAdded",
        "SongsAdded",
        "ArtistsRemoved",
        "AlbumsRemoved",
        "SongsRemoved",
        "AddFavoritesReply",
        "QNetworkReply*",
        "reply",
        "TidalFavoriteRequest::FavoriteType",
        "type",
        "RemoveFavoritesReply",
        "AddArtists",
        "AddAlbums",
        "AddSongs",
        "SongMap",
        "RemoveArtists",
        "RemoveAlbums",
        "RemoveSongs"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ArtistsAdded'
        QtMocHelpers::SignalData<void(const SongList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'AlbumsAdded'
        QtMocHelpers::SignalData<void(const SongList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SongsAdded'
        QtMocHelpers::SignalData<void(const SongList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'ArtistsRemoved'
        QtMocHelpers::SignalData<void(const SongList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'AlbumsRemoved'
        QtMocHelpers::SignalData<void(const SongList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SongsRemoved'
        QtMocHelpers::SignalData<void(const SongList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddFavoritesReply'
        QtMocHelpers::SlotData<void(QNetworkReply *, const TidalFavoriteRequest::FavoriteType, const SongList &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'RemoveFavoritesReply'
        QtMocHelpers::SlotData<void(QNetworkReply *, const TidalFavoriteRequest::FavoriteType, const SongList &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddArtists'
        QtMocHelpers::SlotData<void(const SongList &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddAlbums'
        QtMocHelpers::SlotData<void(const SongList &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddSongs'
        QtMocHelpers::SlotData<void(const SongMap &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 4 },
        }}),
        // Slot 'RemoveArtists'
        QtMocHelpers::SlotData<void(const SongList &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'RemoveAlbums'
        QtMocHelpers::SlotData<void(const SongList &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'RemoveSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'RemoveSongs'
        QtMocHelpers::SlotData<void(const SongMap &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TidalFavoriteRequest, qt_meta_tag_ZN20TidalFavoriteRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TidalFavoriteRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<TidalBaseRequest::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TidalFavoriteRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TidalFavoriteRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TidalFavoriteRequestE_t>.metaTypes,
    nullptr
} };

void TidalFavoriteRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TidalFavoriteRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ArtistsAdded((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 1: _t->AlbumsAdded((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 2: _t->SongsAdded((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 3: _t->ArtistsRemoved((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 4: _t->AlbumsRemoved((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 5: _t->SongsRemoved((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 6: _t->AddFavoritesReply((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TidalFavoriteRequest::FavoriteType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[3]))); break;
        case 7: _t->RemoveFavoritesReply((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TidalFavoriteRequest::FavoriteType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[3]))); break;
        case 8: _t->AddArtists((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 9: _t->AddAlbums((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 10: _t->AddSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 11: _t->AddSongs((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1]))); break;
        case 12: _t->RemoveArtists((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 13: _t->RemoveAlbums((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 14: _t->RemoveSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 15: _t->RemoveSongs((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TidalFavoriteRequest::*)(const SongList & )>(_a, &TidalFavoriteRequest::ArtistsAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalFavoriteRequest::*)(const SongList & )>(_a, &TidalFavoriteRequest::AlbumsAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalFavoriteRequest::*)(const SongList & )>(_a, &TidalFavoriteRequest::SongsAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalFavoriteRequest::*)(const SongList & )>(_a, &TidalFavoriteRequest::ArtistsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalFavoriteRequest::*)(const SongList & )>(_a, &TidalFavoriteRequest::AlbumsRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalFavoriteRequest::*)(const SongList & )>(_a, &TidalFavoriteRequest::SongsRemoved, 5))
            return;
    }
}

const QMetaObject *TidalFavoriteRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TidalFavoriteRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TidalFavoriteRequestE_t>.strings))
        return static_cast<void*>(this);
    return TidalBaseRequest::qt_metacast(_clname);
}

int TidalFavoriteRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TidalBaseRequest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void TidalFavoriteRequest::ArtistsAdded(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TidalFavoriteRequest::AlbumsAdded(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TidalFavoriteRequest::SongsAdded(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TidalFavoriteRequest::ArtistsRemoved(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TidalFavoriteRequest::AlbumsRemoved(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TidalFavoriteRequest::SongsRemoved(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
