/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.UDisks2.Job.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef UDISKS2JOB_H
#define UDISKS2JOB_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "includes/dbus_metatypes.h"

/*
 * Proxy class for interface org.freedesktop.UDisks2.Job
 */
class OrgFreedesktopUDisks2JobInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.UDisks2.Job"; }

public:
    OrgFreedesktopUDisks2JobInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopUDisks2JobInterface();

    Q_PROPERTY(QList<QDBusObjectPath> Objects READ objects)
    inline QList<QDBusObjectPath> objects() const
    { return qvariant_cast< QList<QDBusObjectPath> >(property("Objects")); }

    Q_PROPERTY(QString Operation READ operation)
    inline QString operation() const
    { return qvariant_cast< QString >(property("Operation")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void Completed(bool success, const QString &message);
};

#endif
