/****************************************************************************
** Meta object code from reading C++ file 'playlisttabbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlisttabbar.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlisttabbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14PlaylistTabBarE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistTabBar::qt_create_metaobjectdata<qt_meta_tag_ZN14PlaylistTabBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistTabBar",
        "CurrentIdChanged",
        "",
        "id",
        "Rename",
        "name",
        "Close",
        "Save",
        "PlaylistOrderChanged",
        "QList<int>",
        "ids",
        "PlaylistFavorited",
        "favorite",
        "CurrentIndexChanged",
        "index",
        "RenameSlot",
        "RenameInline",
        "HideEditor",
        "StarSlot",
        "CloseSlot",
        "CloseFromTabIndex",
        "PlaylistFavoritedSlot",
        "PlaylistManagerInitialized",
        "TabMoved",
        "SaveSlot"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'CurrentIdChanged'
        QtMocHelpers::SignalData<void(const int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'Rename'
        QtMocHelpers::SignalData<void(const int, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'Close'
        QtMocHelpers::SignalData<void(const int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'Save'
        QtMocHelpers::SignalData<void(const int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'PlaylistOrderChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'PlaylistFavorited'
        QtMocHelpers::SignalData<void(const int, const bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'CurrentIndexChanged'
        QtMocHelpers::SlotData<void(const int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'RenameSlot'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RenameInline'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'HideEditor'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'StarSlot'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CloseSlot'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CloseFromTabIndex'
        QtMocHelpers::SlotData<void(const int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'PlaylistFavoritedSlot'
        QtMocHelpers::SlotData<void(const int, const bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'PlaylistManagerInitialized'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'TabMoved'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SaveSlot'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistTabBar, qt_meta_tag_ZN14PlaylistTabBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistTabBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabBar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PlaylistTabBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PlaylistTabBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14PlaylistTabBarE_t>.metaTypes,
    nullptr
} };

void PlaylistTabBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistTabBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->CurrentIdChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->Rename((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->Close((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->Save((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->PlaylistOrderChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 5: _t->PlaylistFavorited((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->CurrentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->RenameSlot(); break;
        case 8: _t->RenameInline(); break;
        case 9: _t->HideEditor(); break;
        case 10: _t->StarSlot(); break;
        case 11: _t->CloseSlot(); break;
        case 12: _t->CloseFromTabIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->PlaylistFavoritedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 14: _t->PlaylistManagerInitialized(); break;
        case 15: _t->TabMoved(); break;
        case 16: _t->SaveSlot(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistTabBar::*)(const int )>(_a, &PlaylistTabBar::CurrentIdChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistTabBar::*)(const int , const QString & )>(_a, &PlaylistTabBar::Rename, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistTabBar::*)(const int )>(_a, &PlaylistTabBar::Close, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistTabBar::*)(const int )>(_a, &PlaylistTabBar::Save, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistTabBar::*)(const QList<int> & )>(_a, &PlaylistTabBar::PlaylistOrderChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistTabBar::*)(const int , const bool )>(_a, &PlaylistTabBar::PlaylistFavorited, 5))
            return;
    }
}

const QMetaObject *PlaylistTabBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistTabBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PlaylistTabBarE_t>.strings))
        return static_cast<void*>(this);
    return QTabBar::qt_metacast(_clname);
}

int PlaylistTabBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void PlaylistTabBar::CurrentIdChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PlaylistTabBar::Rename(const int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void PlaylistTabBar::Close(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PlaylistTabBar::Save(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PlaylistTabBar::PlaylistOrderChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PlaylistTabBar::PlaylistFavorited(const int _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
