/********************************************************************************
** Form generated from reading UI file 'osdpretty.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OSDPRETTY_H
#define UI_OSDPRETTY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_OSDPretty
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *icon;
    QVBoxLayout *verticalLayout;
    QLabel *summary;
    QLabel *message;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *OSDPretty)
    {
        if (OSDPretty->objectName().isEmpty())
            OSDPretty->setObjectName("OSDPretty");
        OSDPretty->resize(396, 80);
        OSDPretty->setStyleSheet(QString::fromUtf8("OSDPretty {\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"#summary {\n"
"  font-weight: bold;\n"
"  font-size: larger;\n"
"}\n"
""));
        horizontalLayout = new QHBoxLayout(OSDPretty);
        horizontalLayout->setSpacing(12);
        horizontalLayout->setObjectName("horizontalLayout");
        icon = new QLabel(OSDPretty);
        icon->setObjectName("icon");

        horizontalLayout->addWidget(icon);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(4);
        verticalLayout->setObjectName("verticalLayout");
        summary = new QLabel(OSDPretty);
        summary->setObjectName("summary");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(summary->sizePolicy().hasHeightForWidth());
        summary->setSizePolicy(sizePolicy);
        summary->setMinimumSize(QSize(300, 0));
        summary->setMaximumSize(QSize(400, 16777215));
        summary->setTextFormat(Qt::RichText);
        summary->setWordWrap(true);

        verticalLayout->addWidget(summary);

        message = new QLabel(OSDPretty);
        message->setObjectName("message");
        sizePolicy.setHeightForWidth(message->sizePolicy().hasHeightForWidth());
        message->setSizePolicy(sizePolicy);
        message->setMaximumSize(QSize(400, 16777215));
        message->setTextFormat(Qt::RichText);
        message->setWordWrap(true);

        verticalLayout->addWidget(message);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout);


        retranslateUi(OSDPretty);

        QMetaObject::connectSlotsByName(OSDPretty);
    } // setupUi

    void retranslateUi(QWidget *OSDPretty)
    {
        (void)OSDPretty;
    } // retranslateUi

};

namespace Ui {
    class OSDPretty: public Ui_OSDPretty {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OSDPRETTY_H
