/****************************************************************************
** Meta object code from reading C++ file 'playlistview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12PlaylistViewE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistView::qt_create_metaobjectdata<qt_meta_tag_ZN12PlaylistViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistView",
        "PlayItem",
        "",
        "QModelIndex",
        "idx",
        "Playlist::AutoScroll",
        "autoscroll",
        "PlayPause",
        "offset_nanosec",
        "RightClicked",
        "QPoint",
        "global_pos",
        "SeekForward",
        "SeekBackward",
        "FocusOnFilterSignal",
        "QKeyEvent*",
        "event",
        "BackgroundPropertyChanged",
        "ColumnAlignmentChanged",
        "ColumnAlignmentMap",
        "alignment",
        "ReloadSettings",
        "SaveSettings",
        "SetColumnAlignment",
        "section",
        "Qt::Alignment",
        "JumpToCurrentlyPlayingTrack",
        "Update",
        "SetHeaderState",
        "HeaderSectionResized",
        "logical_index",
        "old_size",
        "new_size",
        "InhibitAutoscrollTimeout",
        "MaybeAutoscroll",
        "InvalidateCachedCurrentPixmap",
        "PlaylistDestroyed",
        "StretchChanged",
        "stretch",
        "FadePreviousBackgroundImage",
        "value",
        "StopGlowing",
        "StartGlowing",
        "JumpToLastPlayedTrack",
        "CopyCurrentSongToClipboard",
        "Playing",
        "Stopped",
        "SongChanged",
        "Song",
        "song",
        "AlbumCoverLoaded",
        "AlbumCoverLoaderResult",
        "result",
        "DynamicModeChanged",
        "dynamic",
        "SetRatingLockStatus",
        "state",
        "RatingHoverIn",
        "pos",
        "RatingHoverOut"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PlayItem'
        QtMocHelpers::SignalData<void(const QModelIndex, const Playlist::AutoScroll)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'PlayPause'
        QtMocHelpers::SignalData<void(const quint64, const Playlist::AutoScroll)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 8 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'PlayPause'
        QtMocHelpers::SignalData<void(const quint64)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::ULongLong, 8 },
        }}),
        // Signal 'PlayPause'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Signal 'RightClicked'
        QtMocHelpers::SignalData<void(const QPoint, const QModelIndex)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SeekForward'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SeekBackward'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'FocusOnFilterSignal'
        QtMocHelpers::SignalData<void(QKeyEvent *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'BackgroundPropertyChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ColumnAlignmentChanged'
        QtMocHelpers::SignalData<void(const ColumnAlignmentMap)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SaveSettings'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetColumnAlignment'
        QtMocHelpers::SlotData<void(const int, const Qt::Alignment)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { 0x80000000 | 25, 20 },
        }}),
        // Slot 'JumpToCurrentlyPlayingTrack'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Update'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SetHeaderState'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'HeaderSectionResized'
        QtMocHelpers::SlotData<void(const int, const int, const int)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 30 }, { QMetaType::Int, 31 }, { QMetaType::Int, 32 },
        }}),
        // Slot 'InhibitAutoscrollTimeout'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'MaybeAutoscroll'
        QtMocHelpers::SlotData<void(const Playlist::AutoScroll)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'InvalidateCachedCurrentPixmap'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistDestroyed'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'StretchChanged'
        QtMocHelpers::SlotData<void(const bool)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'FadePreviousBackgroundImage'
        QtMocHelpers::SlotData<void(const qreal)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
        // Slot 'StopGlowing'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'StartGlowing'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'JumpToLastPlayedTrack'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CopyCurrentSongToClipboard'
        QtMocHelpers::SlotData<void() const>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Playing'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Stopped'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SongChanged'
        QtMocHelpers::SlotData<void(const Song &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const Song &, const AlbumCoverLoaderResult &)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 }, { 0x80000000 | 51, 52 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const Song &)>(50, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'DynamicModeChanged'
        QtMocHelpers::SlotData<void(const bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 54 },
        }}),
        // Slot 'SetRatingLockStatus'
        QtMocHelpers::SlotData<void(const bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'RatingHoverIn'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 10, 58 },
        }}),
        // Slot 'RatingHoverOut'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistView, qt_meta_tag_ZN12PlaylistViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlaylistViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlaylistViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12PlaylistViewE_t>.metaTypes,
    nullptr
} };

void PlaylistView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PlayItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 1: _t->PlayPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 2: _t->PlayPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 3: _t->PlayPause(); break;
        case 4: _t->RightClicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 5: _t->SeekForward(); break;
        case 6: _t->SeekBackward(); break;
        case 7: _t->FocusOnFilterSignal((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 8: _t->BackgroundPropertyChanged(); break;
        case 9: _t->ColumnAlignmentChanged((*reinterpret_cast<std::add_pointer_t<ColumnAlignmentMap>>(_a[1]))); break;
        case 10: _t->ReloadSettings(); break;
        case 11: _t->SaveSettings(); break;
        case 12: _t->SetColumnAlignment((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::Alignment>>(_a[2]))); break;
        case 13: _t->JumpToCurrentlyPlayingTrack(); break;
        case 14: _t->Update(); break;
        case 15: _t->SetHeaderState(); break;
        case 16: _t->HeaderSectionResized((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 17: _t->InhibitAutoscrollTimeout(); break;
        case 18: _t->MaybeAutoscroll((*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[1]))); break;
        case 19: _t->InvalidateCachedCurrentPixmap(); break;
        case 20: _t->PlaylistDestroyed(); break;
        case 21: _t->StretchChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->FadePreviousBackgroundImage((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 23: _t->StopGlowing(); break;
        case 24: _t->StartGlowing(); break;
        case 25: _t->JumpToLastPlayedTrack(); break;
        case 26: _t->CopyCurrentSongToClipboard(); break;
        case 27: _t->Playing(); break;
        case 28: _t->Stopped(); break;
        case 29: _t->SongChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 30: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 31: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 32: _t->DynamicModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->SetRatingLockStatus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->RatingHoverIn((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 35: _t->RatingHoverOut(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColumnAlignmentMap >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Qt::Alignment >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)(const QModelIndex , const Playlist::AutoScroll )>(_a, &PlaylistView::PlayItem, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)(const quint64 , const Playlist::AutoScroll )>(_a, &PlaylistView::PlayPause, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)(const QPoint , const QModelIndex )>(_a, &PlaylistView::RightClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)()>(_a, &PlaylistView::SeekForward, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)()>(_a, &PlaylistView::SeekBackward, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)(QKeyEvent * )>(_a, &PlaylistView::FocusOnFilterSignal, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)()>(_a, &PlaylistView::BackgroundPropertyChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistView::*)(const ColumnAlignmentMap )>(_a, &PlaylistView::ColumnAlignmentChanged, 9))
            return;
    }
}

const QMetaObject *PlaylistView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlaylistViewE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int PlaylistView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void PlaylistView::PlayItem(const QModelIndex _t1, const Playlist::AutoScroll _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void PlaylistView::PlayPause(const quint64 _t1, const Playlist::AutoScroll _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 4
void PlaylistView::RightClicked(const QPoint _t1, const QModelIndex _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void PlaylistView::SeekForward()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PlaylistView::SeekBackward()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PlaylistView::FocusOnFilterSignal(QKeyEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void PlaylistView::BackgroundPropertyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void PlaylistView::ColumnAlignmentChanged(const ColumnAlignmentMap _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
