/********************************************************************************
** Form generated from reading UI file 'trackselectiondialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRACKSELECTIONDIALOG_H
#define UI_TRACKSELECTIONDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/busyindicator.h"

QT_BEGIN_NAMESPACE

class Ui_TrackSelectionDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QSplitter *splitter;
    QListWidget *song_list;
    QStackedWidget *stack;
    QWidget *loading_page;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    BusyIndicator *progress;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;
    QWidget *error_page;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_3;
    QLabel *label_5;
    QLabel *label_4;
    QSpacerItem *verticalSpacer_4;
    QWidget *results_page;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QTreeWidget *results;
    QHBoxLayout *horizontalLayout_2;
    BusyIndicator *loading_label;
    QDialogButtonBox *button_box;

    void setupUi(QDialog *TrackSelectionDialog)
    {
        if (TrackSelectionDialog->objectName().isEmpty())
            TrackSelectionDialog->setObjectName("TrackSelectionDialog");
        TrackSelectionDialog->resize(773, 375);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/64x64/strawberry.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        TrackSelectionDialog->setWindowIcon(icon);
        verticalLayout_3 = new QVBoxLayout(TrackSelectionDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        splitter = new QSplitter(TrackSelectionDialog);
        splitter->setObjectName("splitter");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(splitter->sizePolicy().hasHeightForWidth());
        splitter->setSizePolicy(sizePolicy);
        splitter->setOrientation(Qt::Horizontal);
        song_list = new QListWidget(splitter);
        song_list->setObjectName("song_list");
        song_list->setUniformItemSizes(true);
        splitter->addWidget(song_list);
        stack = new QStackedWidget(splitter);
        stack->setObjectName("stack");
        loading_page = new QWidget();
        loading_page->setObjectName("loading_page");
        verticalLayout = new QVBoxLayout(loading_page);
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer_2 = new QSpacerItem(20, 133, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        progress = new BusyIndicator(loading_page);
        progress->setObjectName("progress");

        horizontalLayout->addWidget(progress);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 133, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        stack->addWidget(loading_page);
        error_page = new QWidget();
        error_page->setObjectName("error_page");
        verticalLayout_4 = new QVBoxLayout(error_page);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalSpacer_3 = new QSpacerItem(20, 124, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_3);

        label_5 = new QLabel(error_page);
        label_5->setObjectName("label_5");
        label_5->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));
        label_5->setAlignment(Qt::AlignCenter);

        verticalLayout_4->addWidget(label_5);

        label_4 = new QLabel(error_page);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignCenter);
        label_4->setWordWrap(true);

        verticalLayout_4->addWidget(label_4);

        verticalSpacer_4 = new QSpacerItem(20, 124, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_4);

        stack->addWidget(error_page);
        results_page = new QWidget();
        results_page->setObjectName("results_page");
        verticalLayout_2 = new QVBoxLayout(results_page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(results_page);
        label->setObjectName("label");
        label->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));

        verticalLayout_2->addWidget(label);

        results = new QTreeWidget(results_page);
        results->setObjectName("results");
        results->setEditTriggers(QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed);
        results->setRootIsDecorated(false);
        results->header()->setMinimumSectionSize(50);
        results->header()->setDefaultSectionSize(150);

        verticalLayout_2->addWidget(results);

        stack->addWidget(results_page);
        splitter->addWidget(stack);

        verticalLayout_3->addWidget(splitter);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        loading_label = new BusyIndicator(TrackSelectionDialog);
        loading_label->setObjectName("loading_label");

        horizontalLayout_2->addWidget(loading_label);

        button_box = new QDialogButtonBox(TrackSelectionDialog);
        button_box->setObjectName("button_box");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(button_box->sizePolicy().hasHeightForWidth());
        button_box->setSizePolicy(sizePolicy1);
        button_box->setOrientation(Qt::Horizontal);
        button_box->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(button_box);


        verticalLayout_3->addLayout(horizontalLayout_2);


        retranslateUi(TrackSelectionDialog);
        QObject::connect(button_box, &QDialogButtonBox::accepted, TrackSelectionDialog, qOverload<>(&QDialog::accept));
        QObject::connect(button_box, &QDialogButtonBox::rejected, TrackSelectionDialog, qOverload<>(&QDialog::reject));

        stack->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(TrackSelectionDialog);
    } // setupUi

    void retranslateUi(QDialog *TrackSelectionDialog)
    {
        TrackSelectionDialog->setWindowTitle(QCoreApplication::translate("TrackSelectionDialog", "Tag fetcher", nullptr));
        label_5->setText(QCoreApplication::translate("TrackSelectionDialog", "Sorry", nullptr));
        label_4->setText(QCoreApplication::translate("TrackSelectionDialog", "Strawberry was unable to find results for this file", nullptr));
        label->setText(QCoreApplication::translate("TrackSelectionDialog", "Select best possible match", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = results->headerItem();
        ___qtreewidgetitem->setText(5, QCoreApplication::translate("TrackSelectionDialog", "Album", nullptr));
        ___qtreewidgetitem->setText(4, QCoreApplication::translate("TrackSelectionDialog", "Artist", nullptr));
        ___qtreewidgetitem->setText(3, QCoreApplication::translate("TrackSelectionDialog", "Album artist", nullptr));
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("TrackSelectionDialog", "Title", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("TrackSelectionDialog", "Year", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("TrackSelectionDialog", "Track", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TrackSelectionDialog: public Ui_TrackSelectionDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRACKSELECTIONDIALOG_H
