/****************************************************************************
** Meta object code from reading C++ file 'devicemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/device/devicemanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13DeviceManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto DeviceManager::qt_create_metaobjectdata<qt_meta_tag_ZN13DeviceManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeviceManager",
        "ExitFinished",
        "",
        "DevicesLoaded",
        "DeviceDatabaseBackend::DeviceList",
        "devices",
        "DeviceConnected",
        "QModelIndex",
        "idx",
        "DeviceDisconnected",
        "DeviceCreatedFromDB",
        "DeviceInfo*",
        "device_info",
        "DeviceError",
        "error",
        "Unmount",
        "PhysicalDeviceAdded",
        "id",
        "PhysicalDeviceRemoved",
        "PhysicalDeviceChanged",
        "DeviceTaskStarted",
        "TasksChanged",
        "DeviceSongCountUpdated",
        "count",
        "LoadAllDevices",
        "DeviceConnectFinished",
        "success",
        "DeviceCloseFinished",
        "AddDevicesFromDB",
        "BackendClosed",
        "ListerClosed",
        "DeviceDestroyed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ExitFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DevicesLoaded'
        QtMocHelpers::SignalData<void(const DeviceDatabaseBackend::DeviceList &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'DeviceConnected'
        QtMocHelpers::SignalData<void(const QModelIndex)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'DeviceDisconnected'
        QtMocHelpers::SignalData<void(const QModelIndex)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'DeviceCreatedFromDB'
        QtMocHelpers::SignalData<void(DeviceInfo *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'DeviceError'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'Unmount'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'PhysicalDeviceAdded'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'PhysicalDeviceRemoved'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'PhysicalDeviceChanged'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'DeviceTaskStarted'
        QtMocHelpers::SlotData<void(const int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'TasksChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeviceSongCountUpdated'
        QtMocHelpers::SlotData<void(const int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'LoadAllDevices'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeviceConnectFinished'
        QtMocHelpers::SlotData<void(const QString &, bool)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::Bool, 26 },
        }}),
        // Slot 'DeviceCloseFinished'
        QtMocHelpers::SlotData<void(const QString &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'AddDevicesFromDB'
        QtMocHelpers::SlotData<void(const DeviceDatabaseBackend::DeviceList &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'BackendClosed'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ListerClosed'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeviceDestroyed'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DeviceManager, qt_meta_tag_ZN13DeviceManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DeviceManager::staticMetaObject = { {
    QMetaObject::SuperData::link<SimpleTreeModel<DeviceInfo>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DeviceManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DeviceManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13DeviceManagerE_t>.metaTypes,
    nullptr
} };

void DeviceManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeviceManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ExitFinished(); break;
        case 1: _t->DevicesLoaded((*reinterpret_cast<std::add_pointer_t<DeviceDatabaseBackend::DeviceList>>(_a[1]))); break;
        case 2: _t->DeviceConnected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->DeviceDisconnected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->DeviceCreatedFromDB((*reinterpret_cast<std::add_pointer_t<DeviceInfo*>>(_a[1]))); break;
        case 5: _t->DeviceError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->Unmount((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 7: _t->PhysicalDeviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->PhysicalDeviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->PhysicalDeviceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->DeviceTaskStarted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->TasksChanged(); break;
        case 12: _t->DeviceSongCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->LoadAllDevices(); break;
        case 14: _t->DeviceConnectFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 15: _t->DeviceCloseFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->AddDevicesFromDB((*reinterpret_cast<std::add_pointer_t<DeviceDatabaseBackend::DeviceList>>(_a[1]))); break;
        case 17: _t->BackendClosed(); break;
        case 18: _t->ListerClosed(); break;
        case 19: _t->DeviceDestroyed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeviceManager::*)()>(_a, &DeviceManager::ExitFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeviceManager::*)(const DeviceDatabaseBackend::DeviceList & )>(_a, &DeviceManager::DevicesLoaded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeviceManager::*)(const QModelIndex )>(_a, &DeviceManager::DeviceConnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeviceManager::*)(const QModelIndex )>(_a, &DeviceManager::DeviceDisconnected, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeviceManager::*)(DeviceInfo * )>(_a, &DeviceManager::DeviceCreatedFromDB, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeviceManager::*)(const QString & )>(_a, &DeviceManager::DeviceError, 5))
            return;
    }
}

const QMetaObject *DeviceManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeviceManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DeviceManagerE_t>.strings))
        return static_cast<void*>(this);
    return SimpleTreeModel<DeviceInfo>::qt_metacast(_clname);
}

int DeviceManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SimpleTreeModel<DeviceInfo>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void DeviceManager::ExitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DeviceManager::DevicesLoaded(const DeviceDatabaseBackend::DeviceList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeviceManager::DeviceConnected(const QModelIndex _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeviceManager::DeviceDisconnected(const QModelIndex _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DeviceManager::DeviceCreatedFromDB(DeviceInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DeviceManager::DeviceError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
