/****************************************************************************
** Meta object code from reading C++ file 'subsonicservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/subsonic/subsonicservice.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'subsonicservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15SubsonicServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto SubsonicService::qt_create_metaobjectdata<qt_meta_tag_ZN15SubsonicServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SubsonicService",
        "SendPing",
        "",
        "SendPingWithCredentials",
        "QUrl",
        "url",
        "username",
        "password",
        "SubsonicSettings::AuthMethod",
        "auth_method",
        "redirect",
        "GetSongs",
        "DeleteSongs",
        "ResetSongsRequest",
        "HandlePingSSLErrors",
        "QList<QSslError>",
        "ssl_errors",
        "HandlePingReply",
        "QNetworkReply*",
        "reply",
        "SongsResultsReceived",
        "SongMap",
        "songs",
        "error"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'SendPing'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SendPingWithCredentials'
        QtMocHelpers::SlotData<void(QUrl, const QString &, const QString &, const SubsonicSettings::AuthMethod, const bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'SendPingWithCredentials'
        QtMocHelpers::SlotData<void(QUrl, const QString &, const QString &, const SubsonicSettings::AuthMethod)>(3, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'GetSongs'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'DeleteSongs'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetSongsRequest'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'HandlePingSSLErrors'
        QtMocHelpers::SlotData<void(const QList<QSslError> &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'HandlePingReply'
        QtMocHelpers::SlotData<void(QNetworkReply *, const QUrl &, const QString &, const QString &, const SubsonicSettings::AuthMethod)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'SongsResultsReceived'
        QtMocHelpers::SlotData<void(const SongMap &, const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::QString, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SubsonicService, qt_meta_tag_ZN15SubsonicServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SubsonicService::staticMetaObject = { {
    QMetaObject::SuperData::link<StreamingService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SubsonicServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SubsonicServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15SubsonicServiceE_t>.metaTypes,
    nullptr
} };

void SubsonicService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SubsonicService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SendPing(); break;
        case 1: _t->SendPingWithCredentials((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<SubsonicSettings::AuthMethod>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 2: _t->SendPingWithCredentials((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<SubsonicSettings::AuthMethod>>(_a[4]))); break;
        case 3: _t->GetSongs(); break;
        case 4: _t->DeleteSongs(); break;
        case 5: _t->ResetSongsRequest(); break;
        case 6: _t->HandlePingSSLErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 7: _t->HandlePingReply((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<SubsonicSettings::AuthMethod>>(_a[5]))); break;
        case 8: _t->SongsResultsReceived((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *SubsonicService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SubsonicService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SubsonicServiceE_t>.strings))
        return static_cast<void*>(this);
    return StreamingService::qt_metacast(_clname);
}

int SubsonicService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StreamingService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
