/****************************************************************************
** Meta object code from reading C++ file 'fileviewtree.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/fileview/fileviewtree.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileviewtree.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12FileViewTreeE_t {};
} // unnamed namespace

template <> constexpr inline auto FileViewTree::qt_create_metaobjectdata<qt_meta_tag_ZN12FileViewTreeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileViewTree",
        "AddToPlaylist",
        "",
        "QMimeData*",
        "data",
        "CopyToCollection",
        "QList<QUrl>",
        "urls",
        "MoveToCollection",
        "CopyToDevice",
        "Delete",
        "filenames",
        "EditTags",
        "LoadSlot",
        "AddToPlaylistSlot",
        "OpenInNewPlaylistSlot",
        "CopyToCollectionSlot",
        "MoveToCollectionSlot",
        "CopyToDeviceSlot",
        "DeleteSlot",
        "EditTagsSlot",
        "ShowInBrowser"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AddToPlaylist'
        QtMocHelpers::SignalData<void(QMimeData *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CopyToCollection'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'MoveToCollection'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'CopyToDevice'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'Delete'
        QtMocHelpers::SignalData<void(const QStringList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 11 },
        }}),
        // Signal 'EditTags'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'LoadSlot'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddToPlaylistSlot'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OpenInNewPlaylistSlot'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CopyToCollectionSlot'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'MoveToCollectionSlot'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CopyToDeviceSlot'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteSlot'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditTagsSlot'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowInBrowser'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileViewTree, qt_meta_tag_ZN12FileViewTreeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileViewTree::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FileViewTreeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FileViewTreeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12FileViewTreeE_t>.metaTypes,
    nullptr
} };

void FileViewTree::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileViewTree *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AddToPlaylist((*reinterpret_cast<std::add_pointer_t<QMimeData*>>(_a[1]))); break;
        case 1: _t->CopyToCollection((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 2: _t->MoveToCollection((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 3: _t->CopyToDevice((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 4: _t->Delete((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->EditTags((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 6: _t->LoadSlot(); break;
        case 7: _t->AddToPlaylistSlot(); break;
        case 8: _t->OpenInNewPlaylistSlot(); break;
        case 9: _t->CopyToCollectionSlot(); break;
        case 10: _t->MoveToCollectionSlot(); break;
        case 11: _t->CopyToDeviceSlot(); break;
        case 12: _t->DeleteSlot(); break;
        case 13: _t->EditTagsSlot(); break;
        case 14: _t->ShowInBrowser(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileViewTree::*)(QMimeData * )>(_a, &FileViewTree::AddToPlaylist, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileViewTree::*)(const QList<QUrl> & )>(_a, &FileViewTree::CopyToCollection, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileViewTree::*)(const QList<QUrl> & )>(_a, &FileViewTree::MoveToCollection, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileViewTree::*)(const QList<QUrl> & )>(_a, &FileViewTree::CopyToDevice, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileViewTree::*)(const QStringList & )>(_a, &FileViewTree::Delete, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileViewTree::*)(const QList<QUrl> & )>(_a, &FileViewTree::EditTags, 5))
            return;
    }
}

const QMetaObject *FileViewTree::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileViewTree::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FileViewTreeE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int FileViewTree::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void FileViewTree::AddToPlaylist(QMimeData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FileViewTree::CopyToCollection(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FileViewTree::MoveToCollection(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FileViewTree::CopyToDevice(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void FileViewTree::Delete(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void FileViewTree::EditTags(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
