/****************************************************************************
** Meta object code from reading C++ file 'organize.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/organize/organize.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'organize.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8OrganizeE_t {};
} // unnamed namespace

template <> constexpr inline auto Organize::qt_create_metaobjectdata<qt_meta_tag_ZN8OrganizeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Organize",
        "Finished",
        "",
        "files_with_errors",
        "log",
        "FileCopied",
        "database_id",
        "SongPathChanged",
        "Song",
        "song",
        "QFileInfo",
        "new_file",
        "std::optional<int>",
        "new_collection_directory_id",
        "ProcessSomeFiles",
        "FileTranscoded",
        "input",
        "output",
        "success",
        "LogLine",
        "message"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Finished'
        QtMocHelpers::SignalData<void(const QStringList &, const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 }, { QMetaType::QStringList, 4 },
        }}),
        // Signal 'FileCopied'
        QtMocHelpers::SignalData<void(const int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'SongPathChanged'
        QtMocHelpers::SignalData<void(const Song &, const QFileInfo &, const std::optional<int>)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'ProcessSomeFiles'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FileTranscoded'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const bool)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'LogLine'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Organize, qt_meta_tag_ZN8OrganizeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Organize::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8OrganizeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8OrganizeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8OrganizeE_t>.metaTypes,
    nullptr
} };

void Organize::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Organize *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Finished((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 1: _t->FileCopied((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->SongPathChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QFileInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::optional<int>>>(_a[3]))); break;
        case 3: _t->ProcessSomeFiles(); break;
        case 4: _t->FileTranscoded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 5: _t->LogLine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Organize::*)(const QStringList & , const QStringList & )>(_a, &Organize::Finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Organize::*)(const int )>(_a, &Organize::FileCopied, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Organize::*)(const Song & , const QFileInfo & , const std::optional<int> )>(_a, &Organize::SongPathChanged, 2))
            return;
    }
}

const QMetaObject *Organize::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Organize::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8OrganizeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Organize::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Organize::Finished(const QStringList & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Organize::FileCopied(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Organize::SongPathChanged(const Song & _t1, const QFileInfo & _t2, const std::optional<int> _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
