/****************************************************************************
** Meta object code from reading C++ file 'loginstatewidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/widgets/loginstatewidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'loginstatewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16LoginStateWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto LoginStateWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16LoginStateWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LoginStateWidget",
        "LogoutClicked",
        "",
        "LoginClicked",
        "SetLoggedIn",
        "LoginStateWidget::State",
        "state",
        "account_name",
        "HideLoggedInState",
        "SetAccountTypeText",
        "text",
        "SetAccountTypeVisible",
        "visible",
        "SetExpires",
        "expires",
        "Logout",
        "FocusLastCredentialField"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'LogoutClicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LoginClicked'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetLoggedIn'
        QtMocHelpers::SlotData<void(const LoginStateWidget::State, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'SetLoggedIn'
        QtMocHelpers::SlotData<void(const LoginStateWidget::State)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'HideLoggedInState'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetAccountTypeText'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'SetAccountTypeVisible'
        QtMocHelpers::SlotData<void(const bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'SetExpires'
        QtMocHelpers::SlotData<void(const QDate)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 14 },
        }}),
        // Slot 'Logout'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FocusLastCredentialField'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoginStateWidget, qt_meta_tag_ZN16LoginStateWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LoginStateWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16LoginStateWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16LoginStateWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16LoginStateWidgetE_t>.metaTypes,
    nullptr
} };

void LoginStateWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoginStateWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->LogoutClicked(); break;
        case 1: _t->LoginClicked(); break;
        case 2: _t->SetLoggedIn((*reinterpret_cast<std::add_pointer_t<LoginStateWidget::State>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->SetLoggedIn((*reinterpret_cast<std::add_pointer_t<LoginStateWidget::State>>(_a[1]))); break;
        case 4: _t->HideLoggedInState(); break;
        case 5: _t->SetAccountTypeText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->SetAccountTypeVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->SetExpires((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 8: _t->Logout(); break;
        case 9: _t->FocusLastCredentialField(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LoginStateWidget::*)()>(_a, &LoginStateWidget::LogoutClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoginStateWidget::*)()>(_a, &LoginStateWidget::LoginClicked, 1))
            return;
    }
}

const QMetaObject *LoginStateWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoginStateWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16LoginStateWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int LoginStateWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void LoginStateWidget::LogoutClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LoginStateWidget::LoginClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
