/****************************************************************************
** Meta object code from reading C++ file 'playerinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/core/playerinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playerinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PlayerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto PlayerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN15PlayerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlayerInterface",
        "Playing",
        "",
        "Paused",
        "Resumed",
        "Stopped",
        "Error",
        "message",
        "PlaylistFinished",
        "VolumeEnabled",
        "volume_enabled",
        "VolumeChanged",
        "volume",
        "TrackSkipped",
        "PlaylistItemPtr",
        "old_track",
        "Seeked",
        "microseconds",
        "SongChangeRequestProcessed",
        "QUrl",
        "url",
        "valid",
        "ForceShowOSD",
        "Song",
        "song",
        "toggle",
        "Authenticated",
        "ReloadSettings",
        "LoadVolume",
        "SaveVolume",
        "SavePlaybackStatus",
        "PlaylistsLoaded",
        "PlayAt",
        "index",
        "pause",
        "offset_nanosec",
        "EngineBase::TrackChangeFlags",
        "change",
        "Playlist::AutoScroll",
        "autoscroll",
        "reshuffle",
        "force_inform",
        "PlayPause",
        "PlayPauseHelper",
        "RestartOrPrevious",
        "Next",
        "Previous",
        "PlayPlaylist",
        "playlist_name",
        "SetVolumeFromEngine",
        "SetVolumeFromSlider",
        "value",
        "SetVolume",
        "VolumeUp",
        "VolumeDown",
        "SeekTo",
        "seconds",
        "SeekForward",
        "SeekBackward",
        "CurrentMetadataChanged",
        "metadata",
        "Mute",
        "Pause",
        "Stop",
        "stop_after",
        "Play",
        "PlayWithPause",
        "PlayHelper",
        "ShowOSD"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Playing'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Paused'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Resumed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Stopped'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'Error'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Signal 'PlaylistFinished'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'VolumeEnabled'
        QtMocHelpers::SignalData<void(const bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'VolumeChanged'
        QtMocHelpers::SignalData<void(const uint)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 },
        }}),
        // Signal 'TrackSkipped'
        QtMocHelpers::SignalData<void(PlaylistItemPtr)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'Seeked'
        QtMocHelpers::SignalData<void(const qint64)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 17 },
        }}),
        // Signal 'SongChangeRequestProcessed'
        QtMocHelpers::SignalData<void(const QUrl &, const bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Signal 'ForceShowOSD'
        QtMocHelpers::SignalData<void(const Song &, const bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::Bool, 25 },
        }}),
        // Signal 'Authenticated'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'LoadVolume'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SaveVolume'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SavePlaybackStatus'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlaylistsLoaded'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlayAt'
        QtMocHelpers::SlotData<void(const int, const bool, const quint64, EngineBase::TrackChangeFlags, const Playlist::AutoScroll, const bool, const bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 33 }, { QMetaType::Bool, 34 }, { QMetaType::ULongLong, 35 }, { 0x80000000 | 36, 37 },
            { 0x80000000 | 38, 39 }, { QMetaType::Bool, 40 }, { QMetaType::Bool, 41 },
        }}),
        // Slot 'PlayAt'
        QtMocHelpers::SlotData<void(const int, const bool, const quint64, EngineBase::TrackChangeFlags, const Playlist::AutoScroll, const bool)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 33 }, { QMetaType::Bool, 34 }, { QMetaType::ULongLong, 35 }, { 0x80000000 | 36, 37 },
            { 0x80000000 | 38, 39 }, { QMetaType::Bool, 40 },
        }}),
        // Slot 'PlayPause'
        QtMocHelpers::SlotData<void(const quint64, const Playlist::AutoScroll)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 35 }, { 0x80000000 | 38, 39 },
        }}),
        // Slot 'PlayPause'
        QtMocHelpers::SlotData<void(const quint64)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::ULongLong, 35 },
        }}),
        // Slot 'PlayPause'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'PlayPauseHelper'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RestartOrPrevious'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Next'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Previous'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PlayPlaylist'
        QtMocHelpers::SlotData<void(const QString &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 48 },
        }}),
        // Slot 'SetVolumeFromEngine'
        QtMocHelpers::SlotData<void(const uint)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 },
        }}),
        // Slot 'SetVolumeFromSlider'
        QtMocHelpers::SlotData<void(const int)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'SetVolume'
        QtMocHelpers::SlotData<void(const uint)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 },
        }}),
        // Slot 'VolumeUp'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'VolumeDown'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SeekTo'
        QtMocHelpers::SlotData<void(const quint64)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 56 },
        }}),
        // Slot 'SeekForward'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SeekBackward'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'CurrentMetadataChanged'
        QtMocHelpers::SlotData<void(const Song &)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 60 },
        }}),
        // Slot 'Mute'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Pause'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Stop'
        QtMocHelpers::SlotData<void(const bool)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 64 },
        }}),
        // Slot 'Stop'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'Play'
        QtMocHelpers::SlotData<void(const quint64)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 35 },
        }}),
        // Slot 'Play'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'PlayWithPause'
        QtMocHelpers::SlotData<void(const quint64)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 35 },
        }}),
        // Slot 'PlayHelper'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ShowOSD'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlayerInterface, qt_meta_tag_ZN15PlayerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlayerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlayerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlayerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PlayerInterfaceE_t>.metaTypes,
    nullptr
} };

void PlayerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlayerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Playing(); break;
        case 1: _t->Paused(); break;
        case 2: _t->Resumed(); break;
        case 3: _t->Stopped(); break;
        case 4: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->Error(); break;
        case 6: _t->PlaylistFinished(); break;
        case 7: _t->VolumeEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->VolumeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 9: _t->TrackSkipped((*reinterpret_cast<std::add_pointer_t<PlaylistItemPtr>>(_a[1]))); break;
        case 10: _t->Seeked((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 11: _t->SongChangeRequestProcessed((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->ForceShowOSD((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->Authenticated(); break;
        case 14: _t->ReloadSettings(); break;
        case 15: _t->LoadVolume(); break;
        case 16: _t->SaveVolume(); break;
        case 17: _t->SavePlaybackStatus(); break;
        case 18: _t->PlaylistsLoaded(); break;
        case 19: _t->PlayAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7]))); break;
        case 20: _t->PlayAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EngineBase::TrackChangeFlags>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 21: _t->PlayPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 22: _t->PlayPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 23: _t->PlayPause(); break;
        case 24: _t->PlayPauseHelper(); break;
        case 25: _t->RestartOrPrevious(); break;
        case 26: _t->Next(); break;
        case 27: _t->Previous(); break;
        case 28: _t->PlayPlaylist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->SetVolumeFromEngine((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 30: _t->SetVolumeFromSlider((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->SetVolume((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 32: _t->VolumeUp(); break;
        case 33: _t->VolumeDown(); break;
        case 34: _t->SeekTo((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 35: _t->SeekForward(); break;
        case 36: _t->SeekBackward(); break;
        case 37: _t->CurrentMetadataChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 38: _t->Mute(); break;
        case 39: _t->Pause(); break;
        case 40: _t->Stop((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->Stop(); break;
        case 42: _t->Play((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 43: _t->Play(); break;
        case 44: _t->PlayWithPause((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 45: _t->PlayHelper(); break;
        case 46: _t->ShowOSD(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)()>(_a, &PlayerInterface::Playing, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)()>(_a, &PlayerInterface::Paused, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)()>(_a, &PlayerInterface::Resumed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)()>(_a, &PlayerInterface::Stopped, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(const QString & )>(_a, &PlayerInterface::Error, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)()>(_a, &PlayerInterface::PlaylistFinished, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(const bool )>(_a, &PlayerInterface::VolumeEnabled, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(const uint )>(_a, &PlayerInterface::VolumeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(PlaylistItemPtr )>(_a, &PlayerInterface::TrackSkipped, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(const qint64 )>(_a, &PlayerInterface::Seeked, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(const QUrl & , const bool )>(_a, &PlayerInterface::SongChangeRequestProcessed, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)(const Song & , const bool )>(_a, &PlayerInterface::ForceShowOSD, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerInterface::*)()>(_a, &PlayerInterface::Authenticated, 13))
            return;
    }
}

const QMetaObject *PlayerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlayerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlayerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PlayerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 47;
    }
    return _id;
}

// SIGNAL 0
void PlayerInterface::Playing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PlayerInterface::Paused()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PlayerInterface::Resumed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PlayerInterface::Stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PlayerInterface::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 6
void PlayerInterface::PlaylistFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PlayerInterface::VolumeEnabled(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void PlayerInterface::VolumeChanged(const uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void PlayerInterface::TrackSkipped(PlaylistItemPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void PlayerInterface::Seeked(const qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void PlayerInterface::SongChangeRequestProcessed(const QUrl & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void PlayerInterface::ForceShowOSD(const Song & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void PlayerInterface::Authenticated()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}
QT_WARNING_POP
