/****************************************************************************
** Meta object code from reading C++ file 'collectionwatcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/collection/collectionwatcher.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionwatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17CollectionWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN17CollectionWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionWatcher",
        "NewOrUpdatedSongs",
        "",
        "SongList",
        "songs",
        "SongsMTimeUpdated",
        "SongsDeleted",
        "SongsUnavailable",
        "unavailable",
        "SongsReadded",
        "SubdirsDiscovered",
        "CollectionSubdirectoryList",
        "subdirs",
        "SubdirsMTimeUpdated",
        "SubdirsDeleted",
        "CompilationsNeedUpdating",
        "UpdateLastSeen",
        "directory_id",
        "expire_unavailable_songs_days",
        "ExitFinished",
        "ScanStarted",
        "task_id",
        "AddDirectory",
        "CollectionDirectory",
        "dir",
        "RemoveDirectory",
        "SetRescanPaused",
        "pause",
        "ReloadSettings",
        "Exit",
        "DirectoryChanged",
        "subdir",
        "IncrementalScanCheck",
        "IncrementalScanNow",
        "FullScanNow",
        "RescanPathsNow",
        "ScanSubdirectory",
        "path",
        "CollectionSubdirectory",
        "files_count",
        "CollectionWatcher::ScanTransaction*",
        "t",
        "force_noincremental",
        "RescanSongs"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'NewOrUpdatedSongs'
        QtMocHelpers::SignalData<void(const SongList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SongsMTimeUpdated'
        QtMocHelpers::SignalData<void(const SongList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SongsDeleted'
        QtMocHelpers::SignalData<void(const SongList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SongsUnavailable'
        QtMocHelpers::SignalData<void(const SongList &, const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 8 },
        }}),
        // Signal 'SongsUnavailable'
        QtMocHelpers::SignalData<void(const SongList &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SongsReadded'
        QtMocHelpers::SignalData<void(const SongList &, const bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 8 },
        }}),
        // Signal 'SongsReadded'
        QtMocHelpers::SignalData<void(const SongList &)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SubdirsDiscovered'
        QtMocHelpers::SignalData<void(const CollectionSubdirectoryList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'SubdirsMTimeUpdated'
        QtMocHelpers::SignalData<void(const CollectionSubdirectoryList &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'SubdirsDeleted'
        QtMocHelpers::SignalData<void(const CollectionSubdirectoryList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'CompilationsNeedUpdating'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'UpdateLastSeen'
        QtMocHelpers::SignalData<void(const int, const int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
        }}),
        // Signal 'ExitFinished'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ScanStarted'
        QtMocHelpers::SignalData<void(const int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'AddDirectory'
        QtMocHelpers::SlotData<void(const CollectionDirectory &, const CollectionSubdirectoryList &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'RemoveDirectory'
        QtMocHelpers::SlotData<void(const CollectionDirectory &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'SetRescanPaused'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Exit'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DirectoryChanged'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'IncrementalScanCheck'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'IncrementalScanNow'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FullScanNow'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RescanPathsNow'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ScanSubdirectory'
        QtMocHelpers::SlotData<void(const CollectionDirectory &, const QString &, const CollectionSubdirectory &, const quint64, CollectionWatcher::ScanTransaction *, const bool)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::QString, 37 }, { 0x80000000 | 38, 31 }, { QMetaType::ULongLong, 39 },
            { 0x80000000 | 40, 41 }, { QMetaType::Bool, 42 },
        }}),
        // Slot 'ScanSubdirectory'
        QtMocHelpers::SlotData<void(const CollectionDirectory &, const QString &, const CollectionSubdirectory &, const quint64, CollectionWatcher::ScanTransaction *)>(36, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::QString, 37 }, { 0x80000000 | 38, 31 }, { QMetaType::ULongLong, 39 },
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'RescanSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CollectionWatcher, qt_meta_tag_ZN17CollectionWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CollectionWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17CollectionWatcherE_t>.metaTypes,
    nullptr
} };

void CollectionWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->NewOrUpdatedSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 1: _t->SongsMTimeUpdated((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 2: _t->SongsDeleted((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 3: _t->SongsUnavailable((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->SongsUnavailable((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 5: _t->SongsReadded((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->SongsReadded((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 7: _t->SubdirsDiscovered((*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[1]))); break;
        case 8: _t->SubdirsMTimeUpdated((*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[1]))); break;
        case 9: _t->SubdirsDeleted((*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[1]))); break;
        case 10: _t->CompilationsNeedUpdating(); break;
        case 11: _t->UpdateLastSeen((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->ExitFinished(); break;
        case 13: _t->ScanStarted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->AddDirectory((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[2]))); break;
        case 15: _t->RemoveDirectory((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1]))); break;
        case 16: _t->SetRescanPaused((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->ReloadSettings(); break;
        case 18: _t->Exit(); break;
        case 19: _t->DirectoryChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->IncrementalScanCheck(); break;
        case 21: _t->IncrementalScanNow(); break;
        case 22: _t->FullScanNow(); break;
        case 23: _t->RescanPathsNow(); break;
        case 24: _t->ScanSubdirectory((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CollectionSubdirectory>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<CollectionWatcher::ScanTransaction*>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 25: _t->ScanSubdirectory((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CollectionSubdirectory>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<CollectionWatcher::ScanTransaction*>>(_a[5]))); break;
        case 26: _t->RescanSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectory >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectory >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const SongList & )>(_a, &CollectionWatcher::NewOrUpdatedSongs, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const SongList & )>(_a, &CollectionWatcher::SongsMTimeUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const SongList & )>(_a, &CollectionWatcher::SongsDeleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const SongList & , const bool )>(_a, &CollectionWatcher::SongsUnavailable, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const SongList & , const bool )>(_a, &CollectionWatcher::SongsReadded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const CollectionSubdirectoryList & )>(_a, &CollectionWatcher::SubdirsDiscovered, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const CollectionSubdirectoryList & )>(_a, &CollectionWatcher::SubdirsMTimeUpdated, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const CollectionSubdirectoryList & )>(_a, &CollectionWatcher::SubdirsDeleted, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)()>(_a, &CollectionWatcher::CompilationsNeedUpdating, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const int , const int )>(_a, &CollectionWatcher::UpdateLastSeen, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)()>(_a, &CollectionWatcher::ExitFinished, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionWatcher::*)(const int )>(_a, &CollectionWatcher::ScanStarted, 13))
            return;
    }
}

const QMetaObject *CollectionWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CollectionWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void CollectionWatcher::NewOrUpdatedSongs(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CollectionWatcher::SongsMTimeUpdated(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CollectionWatcher::SongsDeleted(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CollectionWatcher::SongsUnavailable(const SongList & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 5
void CollectionWatcher::SongsReadded(const SongList & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 7
void CollectionWatcher::SubdirsDiscovered(const CollectionSubdirectoryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void CollectionWatcher::SubdirsMTimeUpdated(const CollectionSubdirectoryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void CollectionWatcher::SubdirsDeleted(const CollectionSubdirectoryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void CollectionWatcher::CompilationsNeedUpdating()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void CollectionWatcher::UpdateLastSeen(const int _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void CollectionWatcher::ExitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void CollectionWatcher::ScanStarted(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
