/****************************************************************************
** Meta object code from reading C++ file 'collectionbackend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/collection/collectionbackend.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionbackend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26CollectionBackendInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionBackendInterface::qt_create_metaobjectdata<qt_meta_tag_ZN26CollectionBackendInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionBackendInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CollectionBackendInterface, qt_meta_tag_ZN26CollectionBackendInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CollectionBackendInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26CollectionBackendInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26CollectionBackendInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26CollectionBackendInterfaceE_t>.metaTypes,
    nullptr
} };

void CollectionBackendInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionBackendInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CollectionBackendInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionBackendInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26CollectionBackendInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CollectionBackendInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17CollectionBackendE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionBackend::qt_create_metaobjectdata<qt_meta_tag_ZN17CollectionBackendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionBackend",
        "DirectoryAdded",
        "",
        "CollectionDirectory",
        "dir",
        "CollectionSubdirectoryList",
        "subdir",
        "DirectoryDeleted",
        "GotSongs",
        "SongList",
        "songs",
        "id",
        "SongsAdded",
        "SongsDeleted",
        "SongsChanged",
        "SongsStatisticsChanged",
        "save_tags",
        "DatabaseReset",
        "TotalSongCountUpdated",
        "count",
        "TotalArtistCountUpdated",
        "TotalAlbumCountUpdated",
        "SongsRatingChanged",
        "ExitFinished",
        "Error",
        "error",
        "Exit",
        "GetAllSongs",
        "LoadDirectories",
        "UpdateTotalSongCount",
        "UpdateTotalArtistCount",
        "UpdateTotalAlbumCount",
        "AddDirectory",
        "path",
        "RemoveDirectory",
        "AddOrUpdateSongs",
        "UpdateSongsBySongID",
        "SongMap",
        "new_songs",
        "UpdateMTimesOnly",
        "DeleteSongs",
        "DeleteSongsByUrls",
        "QList<QUrl>",
        "url",
        "MarkSongsUnavailable",
        "unavailable",
        "AddOrUpdateSubdirs",
        "subdirs",
        "DeleteSubdirs",
        "CompilationsNeedUpdating",
        "UpdateEmbeddedAlbumArt",
        "effective_albumartist",
        "album",
        "art_embedded",
        "UpdateManualAlbumArt",
        "QUrl",
        "art_manual",
        "UnsetAlbumArt",
        "ClearAlbumArt",
        "art_unset",
        "ForceCompilation",
        "artists",
        "on",
        "IncrementPlayCount",
        "IncrementSkipCount",
        "progress",
        "ResetPlayStatistics",
        "QList<int>",
        "id_list",
        "id_str_list",
        "DeleteAll",
        "SongPathChanged",
        "Song",
        "song",
        "QFileInfo",
        "new_file",
        "std::optional<int>",
        "new_collection_directory_id",
        "GetSongsBy",
        "artist",
        "title",
        "UpdateLastPlayed",
        "lastplayed",
        "UpdatePlayCount",
        "playcount",
        "UpdateSongRating",
        "rating",
        "UpdateSongsRating",
        "UpdateLastSeen",
        "directory_id",
        "expire_unavailable_songs_days",
        "ExpireSongs",
        "parent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'DirectoryAdded'
        QtMocHelpers::SignalData<void(const CollectionDirectory &, const CollectionSubdirectoryList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'DirectoryDeleted'
        QtMocHelpers::SignalData<void(const CollectionDirectory &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'GotSongs'
        QtMocHelpers::SignalData<void(const SongList &, const int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 11 },
        }}),
        // Signal 'SongsAdded'
        QtMocHelpers::SignalData<void(const SongList &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'SongsDeleted'
        QtMocHelpers::SignalData<void(const SongList &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'SongsChanged'
        QtMocHelpers::SignalData<void(const SongList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'SongsStatisticsChanged'
        QtMocHelpers::SignalData<void(const SongList &, const bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 16 },
        }}),
        // Signal 'SongsStatisticsChanged'
        QtMocHelpers::SignalData<void(const SongList &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'DatabaseReset'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TotalSongCountUpdated'
        QtMocHelpers::SignalData<void(const int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Signal 'TotalArtistCountUpdated'
        QtMocHelpers::SignalData<void(const int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Signal 'TotalAlbumCountUpdated'
        QtMocHelpers::SignalData<void(const int)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Signal 'SongsRatingChanged'
        QtMocHelpers::SignalData<void(const SongList &, const bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 16 },
        }}),
        // Signal 'ExitFinished'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'Exit'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetAllSongs'
        QtMocHelpers::SlotData<void(const int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'LoadDirectories'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateTotalSongCount'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateTotalArtistCount'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateTotalAlbumCount'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'AddDirectory'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'RemoveDirectory'
        QtMocHelpers::SlotData<void(const CollectionDirectory &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddOrUpdateSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'UpdateSongsBySongID'
        QtMocHelpers::SlotData<void(const SongMap &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'UpdateMTimesOnly'
        QtMocHelpers::SlotData<void(const SongList &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'DeleteSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'DeleteSongsByUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Slot 'MarkSongsUnavailable'
        QtMocHelpers::SlotData<void(const SongList &, const bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 45 },
        }}),
        // Slot 'MarkSongsUnavailable'
        QtMocHelpers::SlotData<void(const SongList &)>(44, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'AddOrUpdateSubdirs'
        QtMocHelpers::SlotData<void(const CollectionSubdirectoryList &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 47 },
        }}),
        // Slot 'DeleteSubdirs'
        QtMocHelpers::SlotData<void(const CollectionSubdirectoryList &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 47 },
        }}),
        // Slot 'CompilationsNeedUpdating'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'UpdateEmbeddedAlbumArt'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 }, { QMetaType::QString, 52 }, { QMetaType::Bool, 53 },
        }}),
        // Slot 'UpdateManualAlbumArt'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QUrl &)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 }, { QMetaType::QString, 52 }, { 0x80000000 | 55, 56 },
        }}),
        // Slot 'UnsetAlbumArt'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 }, { QMetaType::QString, 52 },
        }}),
        // Slot 'ClearAlbumArt'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const bool)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 }, { QMetaType::QString, 52 }, { QMetaType::Bool, 59 },
        }}),
        // Slot 'ForceCompilation'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &, const bool)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 52 }, { QMetaType::QStringList, 61 }, { QMetaType::Bool, 62 },
        }}),
        // Slot 'IncrementPlayCount'
        QtMocHelpers::SlotData<void(const int)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'IncrementSkipCount'
        QtMocHelpers::SlotData<void(const int, const float)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Float, 65 },
        }}),
        // Slot 'ResetPlayStatistics'
        QtMocHelpers::SlotData<void(const int, const bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'ResetPlayStatistics'
        QtMocHelpers::SlotData<void(const int)>(66, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'ResetPlayStatistics'
        QtMocHelpers::SlotData<void(const QList<int> &, const bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 67, 68 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'ResetPlayStatistics'
        QtMocHelpers::SlotData<void(const QList<int> &)>(66, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 67, 68 },
        }}),
        // Slot 'ResetPlayStatistics'
        QtMocHelpers::SlotData<bool(const QStringList &)>(66, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QStringList, 69 },
        }}),
        // Slot 'DeleteAll'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SongPathChanged'
        QtMocHelpers::SlotData<void(const Song &, const QFileInfo &, const std::optional<int>)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 72, 73 }, { 0x80000000 | 74, 75 }, { 0x80000000 | 76, 77 },
        }}),
        // Slot 'GetSongsBy'
        QtMocHelpers::SlotData<SongList(const QString &, const QString &, const QString &)>(78, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 79 }, { QMetaType::QString, 52 }, { QMetaType::QString, 80 },
        }}),
        // Slot 'UpdateLastPlayed'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const qint64)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 79 }, { QMetaType::QString, 52 }, { QMetaType::QString, 80 }, { QMetaType::LongLong, 82 },
        }}),
        // Slot 'UpdatePlayCount'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const int, const bool)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 79 }, { QMetaType::QString, 80 }, { QMetaType::Int, 84 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'UpdatePlayCount'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const int)>(83, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 79 }, { QMetaType::QString, 80 }, { QMetaType::Int, 84 },
        }}),
        // Slot 'UpdateSongRating'
        QtMocHelpers::SlotData<void(const int, const float, const bool)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Float, 86 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'UpdateSongRating'
        QtMocHelpers::SlotData<void(const int, const float)>(85, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Float, 86 },
        }}),
        // Slot 'UpdateSongsRating'
        QtMocHelpers::SlotData<void(const QList<int> &, const float, const bool)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 67, 68 }, { QMetaType::Float, 86 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'UpdateSongsRating'
        QtMocHelpers::SlotData<void(const QList<int> &, const float)>(87, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 67, 68 }, { QMetaType::Float, 86 },
        }}),
        // Slot 'UpdateLastSeen'
        QtMocHelpers::SlotData<void(const int, const int)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 89 }, { QMetaType::Int, 90 },
        }}),
        // Slot 'ExpireSongs'
        QtMocHelpers::SlotData<void(const int, const int)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 89 }, { QMetaType::Int, 90 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(QObject *)>(2, QMC::AccessPublic, {{
            { QMetaType::QObjectStar, 92 },
        }}),
        QtMocHelpers::ConstructorData<Constructor()>(2, QMC::AccessPublic | QMC::MethodCloned),
    };
    return QtMocHelpers::metaObjectData<CollectionBackend, qt_meta_tag_ZN17CollectionBackendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject CollectionBackend::staticMetaObject = { {
    QMetaObject::SuperData::link<CollectionBackendInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionBackendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionBackendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17CollectionBackendE_t>.metaTypes,
    nullptr
} };

void CollectionBackend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionBackend *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { CollectionBackend *_r = new CollectionBackend((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { CollectionBackend *_r = new CollectionBackend();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) CollectionBackend((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); } break;
        case 1: { new (_a[0]) CollectionBackend(); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->DirectoryAdded((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[2]))); break;
        case 1: _t->DirectoryDeleted((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1]))); break;
        case 2: _t->GotSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->SongsAdded((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 4: _t->SongsDeleted((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 5: _t->SongsChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 6: _t->SongsStatisticsChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->SongsStatisticsChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 8: _t->DatabaseReset(); break;
        case 9: _t->TotalSongCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->TotalArtistCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->TotalAlbumCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->SongsRatingChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->ExitFinished(); break;
        case 14: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->Exit(); break;
        case 16: _t->GetAllSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->LoadDirectories(); break;
        case 18: _t->UpdateTotalSongCount(); break;
        case 19: _t->UpdateTotalArtistCount(); break;
        case 20: _t->UpdateTotalAlbumCount(); break;
        case 21: _t->AddDirectory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->RemoveDirectory((*reinterpret_cast<std::add_pointer_t<CollectionDirectory>>(_a[1]))); break;
        case 23: _t->AddOrUpdateSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 24: _t->UpdateSongsBySongID((*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[1]))); break;
        case 25: _t->UpdateMTimesOnly((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 26: _t->DeleteSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 27: _t->DeleteSongsByUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 28: _t->MarkSongsUnavailable((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 29: _t->MarkSongsUnavailable((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 30: _t->AddOrUpdateSubdirs((*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[1]))); break;
        case 31: _t->DeleteSubdirs((*reinterpret_cast<std::add_pointer_t<CollectionSubdirectoryList>>(_a[1]))); break;
        case 32: _t->CompilationsNeedUpdating(); break;
        case 33: _t->UpdateEmbeddedAlbumArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 34: _t->UpdateManualAlbumArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 35: _t->UnsetAlbumArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 36: _t->ClearAlbumArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 37: _t->ForceCompilation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 38: _t->IncrementPlayCount((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->IncrementSkipCount((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 40: _t->ResetPlayStatistics((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 41: _t->ResetPlayStatistics((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 42: _t->ResetPlayStatistics((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 43: _t->ResetPlayStatistics((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 44: { bool _r = _t->ResetPlayStatistics((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 45: _t->DeleteAll(); break;
        case 46: _t->SongPathChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QFileInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::optional<int>>>(_a[3]))); break;
        case 47: { SongList _r = _t->GetSongsBy((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<SongList*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->UpdateLastPlayed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4]))); break;
        case 49: _t->UpdatePlayCount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 50: _t->UpdatePlayCount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 51: _t->UpdateSongRating((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 52: _t->UpdateSongRating((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 53: _t->UpdateSongsRating((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 54: _t->UpdateSongsRating((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 55: _t->UpdateLastSeen((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 56: _t->ExpireSongs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionDirectory >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionSubdirectoryList >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 43:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 53:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 54:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const CollectionDirectory & , const CollectionSubdirectoryList & )>(_a, &CollectionBackend::DirectoryAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const CollectionDirectory & )>(_a, &CollectionBackend::DirectoryDeleted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const SongList & , const int )>(_a, &CollectionBackend::GotSongs, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const SongList & )>(_a, &CollectionBackend::SongsAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const SongList & )>(_a, &CollectionBackend::SongsDeleted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const SongList & )>(_a, &CollectionBackend::SongsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const SongList & , const bool )>(_a, &CollectionBackend::SongsStatisticsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)()>(_a, &CollectionBackend::DatabaseReset, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const int )>(_a, &CollectionBackend::TotalSongCountUpdated, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const int )>(_a, &CollectionBackend::TotalArtistCountUpdated, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const int )>(_a, &CollectionBackend::TotalAlbumCountUpdated, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const SongList & , const bool )>(_a, &CollectionBackend::SongsRatingChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)()>(_a, &CollectionBackend::ExitFinished, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionBackend::*)(const QString & )>(_a, &CollectionBackend::Error, 14))
            return;
    }
}

const QMetaObject *CollectionBackend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionBackend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionBackendE_t>.strings))
        return static_cast<void*>(this);
    return CollectionBackendInterface::qt_metacast(_clname);
}

int CollectionBackend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CollectionBackendInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    }
    return _id;
}

// SIGNAL 0
void CollectionBackend::DirectoryAdded(const CollectionDirectory & _t1, const CollectionSubdirectoryList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void CollectionBackend::DirectoryDeleted(const CollectionDirectory & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CollectionBackend::GotSongs(const SongList & _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void CollectionBackend::SongsAdded(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void CollectionBackend::SongsDeleted(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void CollectionBackend::SongsChanged(const SongList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void CollectionBackend::SongsStatisticsChanged(const SongList & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 8
void CollectionBackend::DatabaseReset()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void CollectionBackend::TotalSongCountUpdated(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void CollectionBackend::TotalArtistCountUpdated(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void CollectionBackend::TotalAlbumCountUpdated(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void CollectionBackend::SongsRatingChanged(const SongList & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void CollectionBackend::ExitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void CollectionBackend::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}
QT_WARNING_POP
