/****************************************************************************
** Meta object code from reading C++ file 'playlistmanagerinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistmanagerinterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistmanagerinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24PlaylistManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN24PlaylistManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistManagerInterface",
        "PlaylistManagerInitialized",
        "",
        "AllPlaylistsLoaded",
        "PlaylistAdded",
        "id",
        "name",
        "favorite",
        "PlaylistDeleted",
        "PlaylistClosed",
        "PlaylistRenamed",
        "new_name",
        "PlaylistFavorited",
        "CurrentChanged",
        "Playlist*",
        "new_playlist",
        "scroll_position",
        "ActiveChanged",
        "Error",
        "message",
        "SummaryTextChanged",
        "summary",
        "CurrentSongChanged",
        "Song",
        "song",
        "CurrentSongMetadataChanged",
        "PlaylistChanged",
        "playlist",
        "EditingFinished",
        "playlist_id",
        "QModelIndex",
        "idx",
        "PlayRequested",
        "Playlist::AutoScroll",
        "autoscroll",
        "PlaylistItemsAdded",
        "QList<QUuid>",
        "track_ids",
        "QUuid",
        "after_track_id",
        "PlaylistItemsRemoved",
        "PlaylistItemMetadataChanged",
        "track_id",
        "New",
        "SongList",
        "songs",
        "special_type",
        "Load",
        "filename",
        "Save",
        "playlist_name",
        "PlaylistSettings::PathType",
        "path_type",
        "Rename",
        "Delete",
        "Close",
        "Open",
        "ChangePlaylistOrder",
        "QList<int>",
        "ids",
        "SongChangeRequestProcessed",
        "QUrl",
        "url",
        "valid",
        "SetCurrentPlaylist",
        "SetActivePlaylist",
        "SetActiveToCurrent",
        "SelectionChanged",
        "QItemSelection",
        "selection",
        "ClearCurrent",
        "ShuffleCurrent",
        "RemoveDuplicatesCurrent",
        "RemoveUnavailableCurrent",
        "SetActivePlaying",
        "SetActivePaused",
        "SetActiveStopped",
        "RateCurrentSong",
        "rating",
        "RateCurrentSong2"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PlaylistManagerInitialized'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AllPlaylistsLoaded'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PlaylistAdded'
        QtMocHelpers::SignalData<void(const int, const QString &, const bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'PlaylistDeleted'
        QtMocHelpers::SignalData<void(const int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'PlaylistClosed'
        QtMocHelpers::SignalData<void(const int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'PlaylistRenamed'
        QtMocHelpers::SignalData<void(const int, const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'PlaylistFavorited'
        QtMocHelpers::SignalData<void(const int, const bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'CurrentChanged'
        QtMocHelpers::SignalData<void(Playlist *, const int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'CurrentChanged'
        QtMocHelpers::SignalData<void(Playlist *)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'ActiveChanged'
        QtMocHelpers::SignalData<void(Playlist *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'SummaryTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Signal 'CurrentSongChanged'
        QtMocHelpers::SignalData<void(const Song &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'CurrentSongMetadataChanged'
        QtMocHelpers::SignalData<void(const Song &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'PlaylistChanged'
        QtMocHelpers::SignalData<void(Playlist *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 27 },
        }}),
        // Signal 'EditingFinished'
        QtMocHelpers::SignalData<void(const int, const QModelIndex)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { 0x80000000 | 30, 31 },
        }}),
        // Signal 'PlayRequested'
        QtMocHelpers::SignalData<void(const QModelIndex, const Playlist::AutoScroll)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { 0x80000000 | 33, 34 },
        }}),
        // Signal 'PlaylistItemsAdded'
        QtMocHelpers::SignalData<void(const int, const QList<QUuid> &, const QUuid)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { 0x80000000 | 36, 37 }, { 0x80000000 | 38, 39 },
        }}),
        // Signal 'PlaylistItemsRemoved'
        QtMocHelpers::SignalData<void(const int, const QList<QUuid> &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'PlaylistItemMetadataChanged'
        QtMocHelpers::SignalData<void(const int, const QUuid)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { 0x80000000 | 38, 42 },
        }}),
        // Slot 'New'
        QtMocHelpers::SlotData<void(const QString &, const SongList &, const QString &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 44, 45 }, { QMetaType::QString, 46 },
        }}),
        // Slot 'New'
        QtMocHelpers::SlotData<void(const QString &, const SongList &)>(43, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 44, 45 },
        }}),
        // Slot 'New'
        QtMocHelpers::SlotData<void(const QString &)>(43, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'Load'
        QtMocHelpers::SlotData<void(const QString &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 48 },
        }}),
        // Slot 'Save'
        QtMocHelpers::SlotData<void(const int, const QString &, const QString &, const PlaylistSettings::PathType)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 50 }, { QMetaType::QString, 48 }, { 0x80000000 | 51, 52 },
        }}),
        // Slot 'Rename'
        QtMocHelpers::SlotData<void(const int, const QString &)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'Delete'
        QtMocHelpers::SlotData<void(const int)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'Close'
        QtMocHelpers::SlotData<bool(const int)>(55, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'Open'
        QtMocHelpers::SlotData<void(const int)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'ChangePlaylistOrder'
        QtMocHelpers::SlotData<void(const QList<int> &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 58, 59 },
        }}),
        // Slot 'SongChangeRequestProcessed'
        QtMocHelpers::SlotData<void(const QUrl &, const bool)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 61, 62 }, { QMetaType::Bool, 63 },
        }}),
        // Slot 'SetCurrentPlaylist'
        QtMocHelpers::SlotData<void(const int)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'SetActivePlaylist'
        QtMocHelpers::SlotData<void(const int)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'SetActiveToCurrent'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 68, 69 },
        }}),
        // Slot 'ClearCurrent'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ShuffleCurrent'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RemoveDuplicatesCurrent'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RemoveUnavailableCurrent'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetActivePlaying'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetActivePaused'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetActiveStopped'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RateCurrentSong'
        QtMocHelpers::SlotData<void(const float)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 78 },
        }}),
        // Slot 'RateCurrentSong2'
        QtMocHelpers::SlotData<void(const int)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 78 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistManagerInterface, qt_meta_tag_ZN24PlaylistManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24PlaylistManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24PlaylistManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24PlaylistManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void PlaylistManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PlaylistManagerInitialized(); break;
        case 1: _t->AllPlaylistsLoaded(); break;
        case 2: _t->PlaylistAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 3: _t->PlaylistDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->PlaylistClosed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->PlaylistRenamed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->PlaylistFavorited((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->CurrentChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->CurrentChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 9: _t->ActiveChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 10: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->SummaryTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->CurrentSongChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 13: _t->CurrentSongMetadataChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 14: _t->PlaylistChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 15: _t->EditingFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 16: _t->PlayRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 17: _t->PlaylistItemsAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUuid>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[3]))); break;
        case 18: _t->PlaylistItemsRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUuid>>>(_a[2]))); break;
        case 19: _t->PlaylistItemMetadataChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[2]))); break;
        case 20: _t->New((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 21: _t->New((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongList>>(_a[2]))); break;
        case 22: _t->New((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->Load((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->Save((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<PlaylistSettings::PathType>>(_a[4]))); break;
        case 25: _t->Rename((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 26: _t->Delete((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: { bool _r = _t->Close((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->Open((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->ChangePlaylistOrder((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 30: _t->SongChangeRequestProcessed((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 31: _t->SetCurrentPlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->SetActivePlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 33: _t->SetActiveToCurrent(); break;
        case 34: _t->SelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 35: _t->ClearCurrent(); break;
        case 36: _t->ShuffleCurrent(); break;
        case 37: _t->RemoveDuplicatesCurrent(); break;
        case 38: _t->RemoveUnavailableCurrent(); break;
        case 39: _t->SetActivePlaying(); break;
        case 40: _t->SetActivePaused(); break;
        case 41: _t->SetActiveStopped(); break;
        case 42: _t->RateCurrentSong((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 43: _t->RateCurrentSong2((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Playlist* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Playlist* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Playlist* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Playlist* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)()>(_a, &PlaylistManagerInterface::PlaylistManagerInitialized, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)()>(_a, &PlaylistManagerInterface::AllPlaylistsLoaded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const QString & , const bool )>(_a, &PlaylistManagerInterface::PlaylistAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int )>(_a, &PlaylistManagerInterface::PlaylistDeleted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int )>(_a, &PlaylistManagerInterface::PlaylistClosed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const QString & )>(_a, &PlaylistManagerInterface::PlaylistRenamed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const bool )>(_a, &PlaylistManagerInterface::PlaylistFavorited, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(Playlist * , const int )>(_a, &PlaylistManagerInterface::CurrentChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(Playlist * )>(_a, &PlaylistManagerInterface::ActiveChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const QString & )>(_a, &PlaylistManagerInterface::Error, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const QString & )>(_a, &PlaylistManagerInterface::SummaryTextChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const Song & )>(_a, &PlaylistManagerInterface::CurrentSongChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const Song & )>(_a, &PlaylistManagerInterface::CurrentSongMetadataChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(Playlist * )>(_a, &PlaylistManagerInterface::PlaylistChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const QModelIndex )>(_a, &PlaylistManagerInterface::EditingFinished, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const QModelIndex , const Playlist::AutoScroll )>(_a, &PlaylistManagerInterface::PlayRequested, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const QList<QUuid> & , const QUuid )>(_a, &PlaylistManagerInterface::PlaylistItemsAdded, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const QList<QUuid> & )>(_a, &PlaylistManagerInterface::PlaylistItemsRemoved, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistManagerInterface::*)(const int , const QUuid )>(_a, &PlaylistManagerInterface::PlaylistItemMetadataChanged, 19))
            return;
    }
}

const QMetaObject *PlaylistManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24PlaylistManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PlaylistManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    return _id;
}

// SIGNAL 0
void PlaylistManagerInterface::PlaylistManagerInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PlaylistManagerInterface::AllPlaylistsLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PlaylistManagerInterface::PlaylistAdded(const int _t1, const QString & _t2, const bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void PlaylistManagerInterface::PlaylistDeleted(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PlaylistManagerInterface::PlaylistClosed(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PlaylistManagerInterface::PlaylistRenamed(const int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void PlaylistManagerInterface::PlaylistFavorited(const int _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void PlaylistManagerInterface::CurrentChanged(Playlist * _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 9
void PlaylistManagerInterface::ActiveChanged(Playlist * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void PlaylistManagerInterface::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void PlaylistManagerInterface::SummaryTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void PlaylistManagerInterface::CurrentSongChanged(const Song & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void PlaylistManagerInterface::CurrentSongMetadataChanged(const Song & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void PlaylistManagerInterface::PlaylistChanged(Playlist * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void PlaylistManagerInterface::EditingFinished(const int _t1, const QModelIndex _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}

// SIGNAL 16
void PlaylistManagerInterface::PlayRequested(const QModelIndex _t1, const Playlist::AutoScroll _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void PlaylistManagerInterface::PlaylistItemsAdded(const int _t1, const QList<QUuid> & _t2, const QUuid _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2, _t3);
}

// SIGNAL 18
void PlaylistManagerInterface::PlaylistItemsRemoved(const int _t1, const QList<QUuid> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2);
}

// SIGNAL 19
void PlaylistManagerInterface::PlaylistItemMetadataChanged(const int _t1, const QUuid _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2);
}
QT_WARNING_POP
