/****************************************************************************
** Meta object code from reading C++ file 'osdbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/osd/osdbase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'osdbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7OSDBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto OSDBase::qt_create_metaobjectdata<qt_meta_tag_ZN7OSDBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OSDBase",
        "ReloadSettings",
        "",
        "SongChanged",
        "Song",
        "song",
        "Paused",
        "Resumed",
        "Stopped",
        "StopAfterToggle",
        "stop",
        "PlaylistFinished",
        "VolumeChanged",
        "value",
        "RepeatModeChanged",
        "PlaylistSequence::RepeatMode",
        "mode",
        "ShuffleModeChanged",
        "PlaylistSequence::ShuffleMode",
        "ReshowCurrentSong",
        "ShowPreview",
        "OSDSettings::Type",
        "type",
        "line1",
        "line2",
        "AlbumCoverLoaded",
        "QUrl",
        "cover_url",
        "QImage",
        "image"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SongChanged'
        QtMocHelpers::SlotData<void(const Song &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'Paused'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Resumed'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Stopped'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'StopAfterToggle'
        QtMocHelpers::SlotData<void(const bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'PlaylistFinished'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'VolumeChanged'
        QtMocHelpers::SlotData<void(const uint)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 },
        }}),
        // Slot 'RepeatModeChanged'
        QtMocHelpers::SlotData<void(const PlaylistSequence::RepeatMode)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'ShuffleModeChanged'
        QtMocHelpers::SlotData<void(const PlaylistSequence::ShuffleMode)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 16 },
        }}),
        // Slot 'ReshowCurrentSong'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ShowPreview'
        QtMocHelpers::SlotData<void(const OSDSettings::Type, const QString &, const QString &, const Song &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const Song &, const QUrl &, const QImage &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 26, 27 }, { 0x80000000 | 28, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OSDBase, qt_meta_tag_ZN7OSDBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OSDBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OSDBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OSDBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7OSDBaseE_t>.metaTypes,
    nullptr
} };

void OSDBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OSDBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ReloadSettings(); break;
        case 1: _t->SongChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 2: _t->Paused(); break;
        case 3: _t->Resumed(); break;
        case 4: _t->Stopped(); break;
        case 5: _t->StopAfterToggle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->PlaylistFinished(); break;
        case 7: _t->VolumeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 8: _t->RepeatModeChanged((*reinterpret_cast<std::add_pointer_t<PlaylistSequence::RepeatMode>>(_a[1]))); break;
        case 9: _t->ShuffleModeChanged((*reinterpret_cast<std::add_pointer_t<PlaylistSequence::ShuffleMode>>(_a[1]))); break;
        case 10: _t->ReshowCurrentSong(); break;
        case 11: _t->ShowPreview((*reinterpret_cast<std::add_pointer_t<OSDSettings::Type>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Song>>(_a[4]))); break;
        case 12: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QImage>>(_a[3]))); break;
        default: ;
        }
    }
}

const QMetaObject *OSDBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OSDBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OSDBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OSDBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
