/****************************************************************************
** Meta object code from reading C++ file 'globalshortcutsmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/globalshortcuts/globalshortcutsmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'globalshortcutsmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22GlobalShortcutsManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto GlobalShortcutsManager::qt_create_metaobjectdata<qt_meta_tag_ZN22GlobalShortcutsManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GlobalShortcutsManager",
        "Play",
        "",
        "Pause",
        "PlayPause",
        "Stop",
        "StopAfter",
        "Next",
        "Previous",
        "RestartOrPrevious",
        "IncVolume",
        "DecVolume",
        "Mute",
        "SeekForward",
        "SeekBackward",
        "ShowHide",
        "ShowOSD",
        "TogglePrettyOSD",
        "CycleShuffleMode",
        "CycleRepeatMode",
        "RemoveCurrentSong",
        "ToggleScrobbling",
        "Love",
        "ReloadSettings",
        "ShowMacAccessibilityDialog"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Play'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Pause'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PlayPause'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Stop'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'StopAfter'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Next'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Previous'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RestartOrPrevious'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'IncVolume'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DecVolume'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Mute'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SeekForward'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SeekBackward'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ShowHide'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ShowOSD'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TogglePrettyOSD'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CycleShuffleMode'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CycleRepeatMode'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RemoveCurrentSong'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ToggleScrobbling'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Love'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ShowMacAccessibilityDialog'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GlobalShortcutsManager, qt_meta_tag_ZN22GlobalShortcutsManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GlobalShortcutsManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22GlobalShortcutsManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22GlobalShortcutsManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22GlobalShortcutsManagerE_t>.metaTypes,
    nullptr
} };

void GlobalShortcutsManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GlobalShortcutsManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Play(); break;
        case 1: _t->Pause(); break;
        case 2: _t->PlayPause(); break;
        case 3: _t->Stop(); break;
        case 4: _t->StopAfter(); break;
        case 5: _t->Next(); break;
        case 6: _t->Previous(); break;
        case 7: _t->RestartOrPrevious(); break;
        case 8: _t->IncVolume(); break;
        case 9: _t->DecVolume(); break;
        case 10: _t->Mute(); break;
        case 11: _t->SeekForward(); break;
        case 12: _t->SeekBackward(); break;
        case 13: _t->ShowHide(); break;
        case 14: _t->ShowOSD(); break;
        case 15: _t->TogglePrettyOSD(); break;
        case 16: _t->CycleShuffleMode(); break;
        case 17: _t->CycleRepeatMode(); break;
        case 18: _t->RemoveCurrentSong(); break;
        case 19: _t->ToggleScrobbling(); break;
        case 20: _t->Love(); break;
        case 21: _t->ReloadSettings(); break;
        case 22: _t->ShowMacAccessibilityDialog(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Play, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Pause, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::PlayPause, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Stop, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::StopAfter, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Next, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Previous, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::RestartOrPrevious, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::IncVolume, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::DecVolume, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Mute, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::SeekForward, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::SeekBackward, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::ShowHide, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::ShowOSD, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::TogglePrettyOSD, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::CycleShuffleMode, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::CycleRepeatMode, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::RemoveCurrentSong, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::ToggleScrobbling, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (GlobalShortcutsManager::*)()>(_a, &GlobalShortcutsManager::Love, 20))
            return;
    }
}

const QMetaObject *GlobalShortcutsManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GlobalShortcutsManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22GlobalShortcutsManagerE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int GlobalShortcutsManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void GlobalShortcutsManager::Play()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GlobalShortcutsManager::Pause()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GlobalShortcutsManager::PlayPause()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GlobalShortcutsManager::Stop()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GlobalShortcutsManager::StopAfter()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void GlobalShortcutsManager::Next()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void GlobalShortcutsManager::Previous()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void GlobalShortcutsManager::RestartOrPrevious()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void GlobalShortcutsManager::IncVolume()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void GlobalShortcutsManager::DecVolume()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void GlobalShortcutsManager::Mute()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void GlobalShortcutsManager::SeekForward()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void GlobalShortcutsManager::SeekBackward()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void GlobalShortcutsManager::ShowHide()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void GlobalShortcutsManager::ShowOSD()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void GlobalShortcutsManager::TogglePrettyOSD()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void GlobalShortcutsManager::CycleShuffleMode()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void GlobalShortcutsManager::CycleRepeatMode()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void GlobalShortcutsManager::RemoveCurrentSong()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void GlobalShortcutsManager::ToggleScrobbling()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void GlobalShortcutsManager::Love()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}
QT_WARNING_POP
