/****************************************************************************
** Meta object code from reading C++ file 'tidalrequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/tidal/tidalrequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tidalrequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12TidalRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto TidalRequest::qt_create_metaobjectdata<qt_meta_tag_ZN12TidalRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TidalRequest",
        "LoginSuccess",
        "",
        "LoginFailure",
        "failure_reason",
        "Results",
        "id",
        "SongMap",
        "songs",
        "error",
        "UpdateStatus",
        "text",
        "UpdateProgress",
        "max",
        "StreamURLFinished",
        "QUrl",
        "media_url",
        "url",
        "Song::FileType",
        "filetype",
        "ArtistsReplyReceived",
        "QNetworkReply*",
        "reply",
        "limit_requested",
        "offset_requested",
        "AlbumsReplyReceived",
        "AlbumsReceived",
        "TidalRequest::Artist",
        "artist_requested",
        "SongsReplyReceived",
        "SongsReceived",
        "artist",
        "TidalRequest::Album",
        "album",
        "ArtistAlbumsReplyReceived",
        "AlbumSongsReplyReceived",
        "AlbumCoverReceived",
        "album_id",
        "filename"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'LoginSuccess'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LoginFailure'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'Results'
        QtMocHelpers::SignalData<void(const int, const SongMap &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { 0x80000000 | 7, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'Results'
        QtMocHelpers::SignalData<void(const int, const SongMap &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'Results'
        QtMocHelpers::SignalData<void(const int)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'UpdateStatus'
        QtMocHelpers::SignalData<void(const int, const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'UpdateProgress'
        QtMocHelpers::SignalData<void(const int, const int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Int, 13 },
        }}),
        // Signal 'StreamURLFinished'
        QtMocHelpers::SignalData<void(const QUrl &, const QUrl &, const Song::FileType, const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 }, { 0x80000000 | 18, 19 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'StreamURLFinished'
        QtMocHelpers::SignalData<void(const QUrl &, const QUrl &, const Song::FileType)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'ArtistsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'AlbumsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'AlbumsReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const TidalRequest::Artist &, const int, const int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 27, 28 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'SongsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'SongsReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const TidalRequest::Artist &, const TidalRequest::Album &, const int, const int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 27, 31 }, { 0x80000000 | 32, 33 }, { QMetaType::Int, 23 },
            { QMetaType::Int, 24 },
        }}),
        // Slot 'ArtistAlbumsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const TidalRequest::Artist &, const int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 27, 31 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'AlbumSongsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const TidalRequest::Artist &, const TidalRequest::Album &, const int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 27, 31 }, { 0x80000000 | 32, 33 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'AlbumCoverReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const QString &, const QUrl &, const QString &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::QString, 37 }, { 0x80000000 | 15, 17 }, { QMetaType::QString, 38 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TidalRequest, qt_meta_tag_ZN12TidalRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TidalRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<TidalBaseRequest::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TidalRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TidalRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TidalRequestE_t>.metaTypes,
    nullptr
} };

void TidalRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TidalRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->LoginSuccess(); break;
        case 1: _t->LoginFailure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->Results((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 3: _t->Results((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2]))); break;
        case 4: _t->Results((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->UpdateStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->UpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->StreamURLFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 8: _t->StreamURLFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[3]))); break;
        case 9: _t->ArtistsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->AlbumsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 11: _t->AlbumsReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TidalRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 12: _t->SongsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 13: _t->SongsReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TidalRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TidalRequest::Album>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 14: _t->ArtistAlbumsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TidalRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 15: _t->AlbumSongsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TidalRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TidalRequest::Album>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 16: _t->AlbumCoverReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TidalRequest::*)()>(_a, &TidalRequest::LoginSuccess, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalRequest::*)(const QString & )>(_a, &TidalRequest::LoginFailure, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalRequest::*)(const int , const SongMap & , const QString & )>(_a, &TidalRequest::Results, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalRequest::*)(const int , const QString & )>(_a, &TidalRequest::UpdateStatus, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalRequest::*)(const int , const int )>(_a, &TidalRequest::UpdateProgress, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalRequest::*)(const QUrl & , const QUrl & , const Song::FileType , const QString & )>(_a, &TidalRequest::StreamURLFinished, 7))
            return;
    }
}

const QMetaObject *TidalRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TidalRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TidalRequestE_t>.strings))
        return static_cast<void*>(this);
    return TidalBaseRequest::qt_metacast(_clname);
}

int TidalRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TidalBaseRequest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void TidalRequest::LoginSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TidalRequest::LoginFailure(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TidalRequest::Results(const int _t1, const SongMap & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void TidalRequest::UpdateStatus(const int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void TidalRequest::UpdateProgress(const int _t1, const int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void TidalRequest::StreamURLFinished(const QUrl & _t1, const QUrl & _t2, const Song::FileType _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
