/********************************************************************************
** Form generated from reading UI file 'smartplaylistquerysortpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMARTPLAYLISTQUERYSORTPAGE_H
#define UI_SMARTPLAYLISTQUERYSORTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "smartplaylists/smartplaylistsearchpreview.h"

QT_BEGIN_NAMESPACE

class Ui_SmartPlaylistQuerySortPage
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QRadioButton *random;
    QRadioButton *field;
    QHBoxLayout *horizontalLayout;
    QComboBox *field_value;
    QComboBox *order;
    QGroupBox *groupBox_2;
    QFormLayout *formLayout_2;
    QRadioButton *limit_none;
    QRadioButton *limit_limit;
    QSpinBox *limit_value;
    QFrame *line;
    SmartPlaylistSearchPreview *preview;

    void setupUi(QWidget *SmartPlaylistQuerySortPage)
    {
        if (SmartPlaylistQuerySortPage->objectName().isEmpty())
            SmartPlaylistQuerySortPage->setObjectName("SmartPlaylistQuerySortPage");
        SmartPlaylistQuerySortPage->resize(723, 335);
        verticalLayout = new QVBoxLayout(SmartPlaylistQuerySortPage);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(SmartPlaylistQuerySortPage);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        random = new QRadioButton(groupBox);
        random->setObjectName("random");
        random->setChecked(true);

        formLayout->setWidget(0, QFormLayout::ItemRole::SpanningRole, random);

        field = new QRadioButton(groupBox);
        field->setObjectName("field");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, field);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        field_value = new QComboBox(groupBox);
        field_value->setObjectName("field_value");

        horizontalLayout->addWidget(field_value);

        order = new QComboBox(groupBox);
        order->setObjectName("order");
        order->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout->addWidget(order);


        formLayout->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(SmartPlaylistQuerySortPage);
        groupBox_2->setObjectName("groupBox_2");
        formLayout_2 = new QFormLayout(groupBox_2);
        formLayout_2->setObjectName("formLayout_2");
        limit_none = new QRadioButton(groupBox_2);
        limit_none->setObjectName("limit_none");
        limit_none->setChecked(true);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::SpanningRole, limit_none);

        limit_limit = new QRadioButton(groupBox_2);
        limit_limit->setObjectName("limit_limit");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, limit_limit);

        limit_value = new QSpinBox(groupBox_2);
        limit_value->setObjectName("limit_value");
        limit_value->setMaximum(1000);
        limit_value->setValue(15);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, limit_value);


        verticalLayout->addWidget(groupBox_2);

        line = new QFrame(SmartPlaylistQuerySortPage);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        preview = new SmartPlaylistSearchPreview(SmartPlaylistQuerySortPage);
        preview->setObjectName("preview");

        verticalLayout->addWidget(preview);


        retranslateUi(SmartPlaylistQuerySortPage);

        QMetaObject::connectSlotsByName(SmartPlaylistQuerySortPage);
    } // setupUi

    void retranslateUi(QWidget *SmartPlaylistQuerySortPage)
    {
        SmartPlaylistQuerySortPage->setWindowTitle(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Form", nullptr));
        groupBox->setTitle(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Sorting", nullptr));
        random->setText(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Put songs in a random order", nullptr));
        field->setText(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Sort songs by", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Limits", nullptr));
        limit_none->setText(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Show all the songs", nullptr));
        limit_limit->setText(QCoreApplication::translate("SmartPlaylistQuerySortPage", "Only show the first", nullptr));
        limit_value->setSuffix(QCoreApplication::translate("SmartPlaylistQuerySortPage", " songs", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SmartPlaylistQuerySortPage: public Ui_SmartPlaylistQuerySortPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SMARTPLAYLISTQUERYSORTPAGE_H
