/****************************************************************************
** Meta object code from reading C++ file 'mpris2_playlists.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mpris2_playlists.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mpris2_playlists.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15Mpris2PlaylistsE_t {};
} // unnamed namespace

template <> constexpr inline auto Mpris2Playlists::qt_create_metaobjectdata<qt_meta_tag_ZN15Mpris2PlaylistsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Mpris2Playlists",
        "D-Bus Interface",
        "org.mpris.MediaPlayer2.Playlists",
        "D-Bus Introspection",
        "  <interface name=\"org.mpris.MediaPlayer2.Playlists\">\n    <prop"
        "erty access=\"read\" type=\"u\" name=\"PlaylistCount\"/>\n    <pro"
        "perty access=\"read\" type=\"as\" name=\"Orderings\"/>\n    <prope"
        "rty access=\"read\" type=\"(b(oss))\" name=\"ActivePlaylist\">\n  "
        "    <annotation value=\"MaybePlaylist\" name=\"org.qtproject.QtDBu"
        "s.QtTypeName\"/>\n    </property>\n    <method name=\"ActivatePlay"
        "list\">\n      <arg direction=\"in\" type=\"o\" name=\"PlaylistId\""
        "/>\n    </method>\n    <method name=\"GetPlaylists\">\n      <arg "
        "direction=\"in\" type=\"u\" name=\"Index\"/>\n      <arg direction"
        "=\"in\" type=\"u\" name=\"MaxCount\"/>\n      <arg direction=\"in\""
        " type=\"s\" name=\"Order\"/>\n      <arg direction=\"in\" type=\"b"
        "\" name=\"ReverseOrder\"/>\n      <arg direction=\"out\" type=\"a("
        "oss)\" name=\"Playlists\"/>\n      <annotation value=\"MprisPlayli"
        "stList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </me"
        "thod>\n    <signal name=\"PlaylistChanged\">\n      <arg direction"
        "=\"out\" type=\"(oss)\" name=\"Playlist\"/>\n      <annotation val"
        "ue=\"MprisPlaylist\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\""
        "/>\n    </signal>\n    <annotation value=\"MaybePlaylist\" name=\""
        "org.qtproject.QtDBus.QtTypeName\"/>\n  </interface>\n",
        "PlaylistChanged",
        "",
        "MprisPlaylist",
        "Playlist",
        "ActivatePlaylist",
        "QDBusObjectPath",
        "PlaylistId",
        "GetPlaylists",
        "MprisPlaylistList",
        "Index",
        "MaxCount",
        "Order",
        "ReverseOrder",
        "ActivePlaylist",
        "MaybePlaylist",
        "Orderings",
        "PlaylistCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PlaylistChanged'
        QtMocHelpers::SignalData<void(MprisPlaylist)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'ActivatePlaylist'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'GetPlaylists'
        QtMocHelpers::SlotData<MprisPlaylistList(uint, uint, const QString &, bool)>(12, 6, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::UInt, 14 }, { QMetaType::UInt, 15 }, { QMetaType::QString, 16 }, { QMetaType::Bool, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ActivePlaylist'
        QtMocHelpers::PropertyData<MaybePlaylist>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Orderings'
        QtMocHelpers::PropertyData<QStringList>(20, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'PlaylistCount'
        QtMocHelpers::PropertyData<uint>(21, QMetaType::UInt, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Mpris2Playlists, qt_meta_tag_ZN15Mpris2PlaylistsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Mpris2Playlists::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Mpris2PlaylistsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Mpris2PlaylistsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15Mpris2PlaylistsE_t>.metaTypes,
    nullptr
} };

void Mpris2Playlists::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Mpris2Playlists *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PlaylistChanged((*reinterpret_cast<std::add_pointer_t<MprisPlaylist>>(_a[1]))); break;
        case 1: _t->ActivatePlaylist((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 2: { MprisPlaylistList _r = _t->GetPlaylists((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<MprisPlaylistList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Mpris2Playlists::*)(MprisPlaylist )>(_a, &Mpris2Playlists::PlaylistChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<MaybePlaylist*>(_v) = _t->activePlaylist(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->orderings(); break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->playlistCount(); break;
        default: break;
        }
    }
}

const QMetaObject *Mpris2Playlists::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Mpris2Playlists::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Mpris2PlaylistsE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int Mpris2Playlists::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Mpris2Playlists::PlaylistChanged(MprisPlaylist _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
