/****************************************************************************
** Meta object code from reading C++ file 'collectionmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/collection/collectionmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15CollectionModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionModel::qt_create_metaobjectdata<qt_meta_tag_ZN15CollectionModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionModel",
        "TotalSongCountUpdated",
        "",
        "count",
        "TotalArtistCountUpdated",
        "TotalAlbumCountUpdated",
        "GroupingChanged",
        "CollectionModel::Grouping",
        "g",
        "separate_albums_by_grouping",
        "SetFilterMode",
        "CollectionFilterOptions::FilterMode",
        "filter_mode",
        "SetFilterMaxAge",
        "filter_max_age",
        "SetFilterMinRating",
        "filter_min_rating",
        "AddReAddOrUpdate",
        "SongList",
        "songs",
        "RemoveSongs",
        "ClearIconDiskCache",
        "ResetInternal",
        "ScheduleReset",
        "ProcessUpdate",
        "LoadSongsFromSqlAsyncFinished",
        "AlbumCoverLoaded",
        "id",
        "AlbumCoverLoaderResult",
        "result",
        "TotalSongCountUpdatedSlot",
        "TotalArtistCountUpdatedSlot",
        "TotalAlbumCountUpdatedSlot",
        "GroupBy",
        "None",
        "AlbumArtist",
        "Artist",
        "Album",
        "AlbumDisc",
        "YearAlbum",
        "YearAlbumDisc",
        "OriginalYearAlbum",
        "OriginalYearAlbumDisc",
        "Disc",
        "Year",
        "OriginalYear",
        "Genre",
        "Composer",
        "Performer",
        "Grouping",
        "FileType",
        "Format",
        "Samplerate",
        "Bitdepth",
        "Bitrate",
        "GroupByCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'TotalSongCountUpdated'
        QtMocHelpers::SignalData<void(const int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'TotalArtistCountUpdated'
        QtMocHelpers::SignalData<void(const int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'TotalAlbumCountUpdated'
        QtMocHelpers::SignalData<void(const int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'GroupingChanged'
        QtMocHelpers::SignalData<void(const CollectionModel::Grouping, const bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'SetFilterMode'
        QtMocHelpers::SlotData<void(const CollectionFilterOptions::FilterMode)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'SetFilterMaxAge'
        QtMocHelpers::SlotData<void(const int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'SetFilterMinRating'
        QtMocHelpers::SlotData<void(const float)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 16 },
        }}),
        // Slot 'AddReAddOrUpdate'
        QtMocHelpers::SlotData<void(const SongList &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'RemoveSongs'
        QtMocHelpers::SlotData<void(const SongList &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'ClearIconDiskCache'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResetInternal'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ScheduleReset'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ProcessUpdate'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadSongsFromSqlAsyncFinished'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const quint64, const AlbumCoverLoaderResult &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 27 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'TotalSongCountUpdatedSlot'
        QtMocHelpers::SlotData<void(const int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'TotalArtistCountUpdatedSlot'
        QtMocHelpers::SlotData<void(const int)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'TotalAlbumCountUpdatedSlot'
        QtMocHelpers::SlotData<void(const int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GroupBy'
        QtMocHelpers::EnumData<enum GroupBy>(33, 33, QMC::EnumIsScoped).add({
            {   34, GroupBy::None },
            {   35, GroupBy::AlbumArtist },
            {   36, GroupBy::Artist },
            {   37, GroupBy::Album },
            {   38, GroupBy::AlbumDisc },
            {   39, GroupBy::YearAlbum },
            {   40, GroupBy::YearAlbumDisc },
            {   41, GroupBy::OriginalYearAlbum },
            {   42, GroupBy::OriginalYearAlbumDisc },
            {   43, GroupBy::Disc },
            {   44, GroupBy::Year },
            {   45, GroupBy::OriginalYear },
            {   46, GroupBy::Genre },
            {   47, GroupBy::Composer },
            {   48, GroupBy::Performer },
            {   49, GroupBy::Grouping },
            {   50, GroupBy::FileType },
            {   51, GroupBy::Format },
            {   52, GroupBy::Samplerate },
            {   53, GroupBy::Bitdepth },
            {   54, GroupBy::Bitrate },
            {   55, GroupBy::GroupByCount },
        }),
    };
    return QtMocHelpers::metaObjectData<CollectionModel, qt_meta_tag_ZN15CollectionModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CollectionModel::staticMetaObject = { {
    QMetaObject::SuperData::link<SimpleTreeModel<CollectionItem>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CollectionModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CollectionModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15CollectionModelE_t>.metaTypes,
    nullptr
} };

void CollectionModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->TotalSongCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->TotalArtistCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->TotalAlbumCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->GroupingChanged((*reinterpret_cast<std::add_pointer_t<CollectionModel::Grouping>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->SetFilterMode((*reinterpret_cast<std::add_pointer_t<CollectionFilterOptions::FilterMode>>(_a[1]))); break;
        case 5: _t->SetFilterMaxAge((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->SetFilterMinRating((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 7: _t->AddReAddOrUpdate((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 8: _t->RemoveSongs((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 9: _t->ClearIconDiskCache(); break;
        case 10: _t->ResetInternal(); break;
        case 11: _t->ScheduleReset(); break;
        case 12: _t->ProcessUpdate(); break;
        case 13: _t->LoadSongsFromSqlAsyncFinished(); break;
        case 14: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 15: _t->TotalSongCountUpdatedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->TotalArtistCountUpdatedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->TotalAlbumCountUpdatedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CollectionModel::Grouping >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionModel::*)(const int )>(_a, &CollectionModel::TotalSongCountUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionModel::*)(const int )>(_a, &CollectionModel::TotalArtistCountUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionModel::*)(const int )>(_a, &CollectionModel::TotalAlbumCountUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionModel::*)(const CollectionModel::Grouping , const bool )>(_a, &CollectionModel::GroupingChanged, 3))
            return;
    }
}

const QMetaObject *CollectionModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CollectionModelE_t>.strings))
        return static_cast<void*>(this);
    return SimpleTreeModel<CollectionItem>::qt_metacast(_clname);
}

int CollectionModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SimpleTreeModel<CollectionItem>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void CollectionModel::TotalSongCountUpdated(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CollectionModel::TotalArtistCountUpdated(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CollectionModel::TotalAlbumCountUpdated(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CollectionModel::GroupingChanged(const CollectionModel::Grouping _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
