/********************************************************************************
** Form generated from reading UI file 'about.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUT_H
#define UI_ABOUT_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_About
{
public:
    QHBoxLayout *horizontalLayout_2;
    QHBoxLayout *layout_about;
    QVBoxLayout *layout_left;
    QLabel *label_icon;
    QSpacerItem *spacer_left;
    QFrame *line;
    QVBoxLayout *layout_right;
    QLabel *label_title;
    QLabel *label_text;
    QSpacerItem *spacer_middle;
    QTextBrowser *text_contributors;
    QSpacerItem *spacer_bottom;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *About)
    {
        if (About->objectName().isEmpty())
            About->setObjectName("About");
        About->resize(500, 500);
        About->setFocusPolicy(Qt::StrongFocus);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/64x64/strawberry.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        About->setWindowIcon(icon);
        About->setStyleSheet(QString::fromUtf8("#line {\n"
"  color: lightgrey;\n"
"}\n"
""));
        horizontalLayout_2 = new QHBoxLayout(About);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setSizeConstraint(QLayout::SetMinimumSize);
        layout_about = new QHBoxLayout();
        layout_about->setObjectName("layout_about");
        layout_about->setSizeConstraint(QLayout::SetMinimumSize);
        layout_left = new QVBoxLayout();
        layout_left->setObjectName("layout_left");
        label_icon = new QLabel(About);
        label_icon->setObjectName("label_icon");
        label_icon->setPixmap(QPixmap(QString::fromUtf8(":/icons/64x64/strawberry.png")));
        label_icon->setScaledContents(false);

        layout_left->addWidget(label_icon);

        spacer_left = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layout_left->addItem(spacer_left);


        layout_about->addLayout(layout_left);

        line = new QFrame(About);
        line->setObjectName("line");
        line->setMinimumSize(QSize(2, 2));
        line->setBaseSize(QSize(1, 1));
        line->setFrameShadow(QFrame::Plain);
        line->setFrameShape(QFrame::Shape::VLine);

        layout_about->addWidget(line);

        layout_right = new QVBoxLayout();
        layout_right->setObjectName("layout_right");
        layout_right->setSizeConstraint(QLayout::SetMinimumSize);
        label_title = new QLabel(About);
        label_title->setObjectName("label_title");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_title->sizePolicy().hasHeightForWidth());
        label_title->setSizePolicy(sizePolicy);
        label_title->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_title->setWordWrap(true);

        layout_right->addWidget(label_title);

        label_text = new QLabel(About);
        label_text->setObjectName("label_text");
        sizePolicy.setHeightForWidth(label_text->sizePolicy().hasHeightForWidth());
        label_text->setSizePolicy(sizePolicy);
        label_text->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_text->setWordWrap(true);
        label_text->setOpenExternalLinks(true);
        label_text->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        layout_right->addWidget(label_text);

        spacer_middle = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        layout_right->addItem(spacer_middle);

        text_contributors = new QTextBrowser(About);
        text_contributors->setObjectName("text_contributors");
        text_contributors->setMinimumSize(QSize(0, 200));
        text_contributors->setOpenExternalLinks(true);

        layout_right->addWidget(text_contributors);

        spacer_bottom = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        layout_right->addItem(spacer_bottom);

        buttonBox = new QDialogButtonBox(About);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        layout_right->addWidget(buttonBox);


        layout_about->addLayout(layout_right);


        horizontalLayout_2->addLayout(layout_about);


        retranslateUi(About);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, About, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, About, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(About);
    } // setupUi

    void retranslateUi(QDialog *About)
    {
        About->setWindowTitle(QCoreApplication::translate("About", "About", nullptr));
        label_title->setText(QString());
        label_text->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class About: public Ui_About {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUT_H
