/****************************************************************************
** Meta object code from reading C++ file 'smartplaylistsviewcontainer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/smartplaylists/smartplaylistsviewcontainer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smartplaylistsviewcontainer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto SmartPlaylistsViewContainer::qt_create_metaobjectdata<qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SmartPlaylistsViewContainer",
        "AddToPlaylist",
        "",
        "QMimeData*",
        "data",
        "ItemsSelectedChanged",
        "ItemDoubleClicked",
        "QModelIndex",
        "idx",
        "RightClicked",
        "QPoint",
        "global_pos",
        "AppendToPlaylist",
        "ReplaceCurrentPlaylist",
        "OpenInNewPlaylist",
        "AddToPlaylistEnqueue",
        "AddToPlaylistEnqueueNext",
        "NewSmartPlaylist",
        "EditSmartPlaylist",
        "DeleteSmartPlaylist",
        "RestoreDefaults",
        "EditSmartPlaylistFromButton",
        "DeleteSmartPlaylistFromButton",
        "EditSmartPlaylistFromContext",
        "DeleteSmartPlaylistFromContext",
        "RestoreDefaultsFromContext",
        "NewSmartPlaylistFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AddToPlaylist'
        QtMocHelpers::SignalData<void(QMimeData *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'ItemsSelectedChanged'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ItemDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'RightClicked'
        QtMocHelpers::SlotData<void(const QPoint, const QModelIndex &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'AppendToPlaylist'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ReplaceCurrentPlaylist'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OpenInNewPlaylist'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddToPlaylistEnqueue'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddToPlaylistEnqueueNext'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'NewSmartPlaylist'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditSmartPlaylist'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'DeleteSmartPlaylist'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'RestoreDefaults'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditSmartPlaylistFromButton'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteSmartPlaylistFromButton'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditSmartPlaylistFromContext'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteSmartPlaylistFromContext'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RestoreDefaultsFromContext'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'NewSmartPlaylistFinished'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SmartPlaylistsViewContainer, qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SmartPlaylistsViewContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t>.metaTypes,
    nullptr
} };

void SmartPlaylistsViewContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SmartPlaylistsViewContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AddToPlaylist((*reinterpret_cast<std::add_pointer_t<QMimeData*>>(_a[1]))); break;
        case 1: _t->ItemsSelectedChanged(); break;
        case 2: _t->ItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->RightClicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 4: _t->AppendToPlaylist(); break;
        case 5: _t->ReplaceCurrentPlaylist(); break;
        case 6: _t->OpenInNewPlaylist(); break;
        case 7: _t->AddToPlaylistEnqueue(); break;
        case 8: _t->AddToPlaylistEnqueueNext(); break;
        case 9: _t->NewSmartPlaylist(); break;
        case 10: _t->EditSmartPlaylist((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->DeleteSmartPlaylist((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->RestoreDefaults(); break;
        case 13: _t->EditSmartPlaylistFromButton(); break;
        case 14: _t->DeleteSmartPlaylistFromButton(); break;
        case 15: _t->EditSmartPlaylistFromContext(); break;
        case 16: _t->DeleteSmartPlaylistFromContext(); break;
        case 17: _t->RestoreDefaultsFromContext(); break;
        case 18: _t->NewSmartPlaylistFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SmartPlaylistsViewContainer::*)(QMimeData * )>(_a, &SmartPlaylistsViewContainer::AddToPlaylist, 0))
            return;
    }
}

const QMetaObject *SmartPlaylistsViewContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SmartPlaylistsViewContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27SmartPlaylistsViewContainerE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SmartPlaylistsViewContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void SmartPlaylistsViewContainer::AddToPlaylist(QMimeData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
