/****************************************************************************
** Meta object code from reading C++ file 'spotifyrequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/spotify/spotifyrequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'spotifyrequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14SpotifyRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto SpotifyRequest::qt_create_metaobjectdata<qt_meta_tag_ZN14SpotifyRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SpotifyRequest",
        "Results",
        "",
        "id",
        "SongMap",
        "songs",
        "error",
        "UpdateStatus",
        "text",
        "ProgressSetMaximum",
        "max",
        "UpdateProgress",
        "StreamURLFinished",
        "QUrl",
        "original_url",
        "url",
        "Song::FileType",
        "FlushRequests",
        "ArtistsReplyReceived",
        "QNetworkReply*",
        "reply",
        "limit_requested",
        "offset_requested",
        "AlbumsReplyReceived",
        "AlbumsReceived",
        "SpotifyRequest::Artist",
        "artist_artist",
        "SongsReplyReceived",
        "SongsReceived",
        "artist",
        "SpotifyRequest::Album",
        "album",
        "ArtistAlbumsReplyReceived",
        "AlbumSongsReplyReceived",
        "AlbumCoverReceived",
        "album_id",
        "filename"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Results'
        QtMocHelpers::SignalData<void(int, SongMap, QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'UpdateStatus'
        QtMocHelpers::SignalData<void(int, QString)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'ProgressSetMaximum'
        QtMocHelpers::SignalData<void(int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'UpdateProgress'
        QtMocHelpers::SignalData<void(int, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'StreamURLFinished'
        QtMocHelpers::SignalData<void(QUrl, QUrl, Song::FileType, QString)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 }, { 0x80000000 | 16, 2 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'StreamURLFinished'
        QtMocHelpers::SignalData<void(QUrl, QUrl, Song::FileType)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 }, { 0x80000000 | 16, 2 },
        }}),
        // Slot 'FlushRequests'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ArtistsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::Int, 21 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'AlbumsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::Int, 21 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'AlbumsReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const SpotifyRequest::Artist &, const int, const int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 25, 26 }, { QMetaType::Int, 21 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'SongsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const int, const int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::Int, 21 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'SongsReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const SpotifyRequest::Artist &, const SpotifyRequest::Album &, const int, const int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 25, 29 }, { 0x80000000 | 30, 31 }, { QMetaType::Int, 21 },
            { QMetaType::Int, 22 },
        }}),
        // Slot 'ArtistAlbumsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const SpotifyRequest::Artist &, const int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 25, 29 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'AlbumSongsReplyReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const SpotifyRequest::Artist &, const SpotifyRequest::Album &, const int)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 25, 29 }, { 0x80000000 | 30, 31 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'AlbumCoverReceived'
        QtMocHelpers::SlotData<void(QNetworkReply *, const QString &, const QUrl &, const QString &)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 35 }, { 0x80000000 | 13, 15 }, { QMetaType::QString, 36 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SpotifyRequest, qt_meta_tag_ZN14SpotifyRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SpotifyRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<SpotifyBaseRequest::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SpotifyRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SpotifyRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14SpotifyRequestE_t>.metaTypes,
    nullptr
} };

void SpotifyRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SpotifyRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Results((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SongMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->UpdateStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->ProgressSetMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->UpdateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->StreamURLFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 5: _t->StreamURLFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[3]))); break;
        case 6: _t->FlushRequests(); break;
        case 7: _t->ArtistsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->AlbumsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->AlbumsReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SpotifyRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 10: _t->SongsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 11: _t->SongsReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SpotifyRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SpotifyRequest::Album>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 12: _t->ArtistAlbumsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SpotifyRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 13: _t->AlbumSongsReplyReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<SpotifyRequest::Artist>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<SpotifyRequest::Album>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 14: _t->AlbumCoverReceived((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SpotifyRequest::*)(int , SongMap , QString )>(_a, &SpotifyRequest::Results, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpotifyRequest::*)(int , QString )>(_a, &SpotifyRequest::UpdateStatus, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpotifyRequest::*)(int , int )>(_a, &SpotifyRequest::ProgressSetMaximum, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpotifyRequest::*)(int , int )>(_a, &SpotifyRequest::UpdateProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpotifyRequest::*)(QUrl , QUrl , Song::FileType , QString )>(_a, &SpotifyRequest::StreamURLFinished, 4))
            return;
    }
}

const QMetaObject *SpotifyRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpotifyRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SpotifyRequestE_t>.strings))
        return static_cast<void*>(this);
    return SpotifyBaseRequest::qt_metacast(_clname);
}

int SpotifyRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SpotifyBaseRequest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void SpotifyRequest::Results(int _t1, SongMap _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void SpotifyRequest::UpdateStatus(int _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void SpotifyRequest::ProgressSetMaximum(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void SpotifyRequest::UpdateProgress(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void SpotifyRequest::StreamURLFinished(QUrl _t1, QUrl _t2, Song::FileType _t3, QString _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
