/****************************************************************************
** Meta object code from reading C++ file 'gstengine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/engine/gstengine.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gstengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9GstEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto GstEngine::qt_create_metaobjectdata<qt_meta_tag_ZN9GstEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GstEngine",
        "ReloadSettings",
        "",
        "SetStereoBalancerEnabled",
        "enabled",
        "SetStereoBalance",
        "value",
        "SetEqualizerEnabled",
        "SetEqualizerParameters",
        "preamp",
        "QList<int>",
        "band_gains",
        "AddBufferConsumer",
        "GstBufferConsumer*",
        "consumer",
        "RemoveBufferConsumer",
        "EndOfStreamReached",
        "pipeline_id",
        "has_next_track",
        "HandlePipelineError",
        "domain",
        "error_code",
        "message",
        "debugstr",
        "NewMetaData",
        "EngineMetadata",
        "engine_metadata",
        "AddBufferToScope",
        "GstBuffer*",
        "buf",
        "format",
        "FadeoutFinished",
        "FadeoutPauseFinished",
        "SeekNow",
        "PlayDone",
        "GstStateChangeReturn",
        "ret",
        "pause",
        "offset_nanosec",
        "BufferingStarted",
        "BufferingProgress",
        "percent",
        "BufferingFinished",
        "PipelineFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetStereoBalancerEnabled'
        QtMocHelpers::SlotData<void(const bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'SetStereoBalance'
        QtMocHelpers::SlotData<void(const float)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 6 },
        }}),
        // Slot 'SetEqualizerEnabled'
        QtMocHelpers::SlotData<void(const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'SetEqualizerParameters'
        QtMocHelpers::SlotData<void(const int, const QList<int> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'AddBufferConsumer'
        QtMocHelpers::SlotData<void(GstBufferConsumer *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'RemoveBufferConsumer'
        QtMocHelpers::SlotData<void(GstBufferConsumer *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'EndOfStreamReached'
        QtMocHelpers::SlotData<void(const int, const bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'HandlePipelineError'
        QtMocHelpers::SlotData<void(const int, const int, const int, const QString &, const QString &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 }, { QMetaType::QString, 22 },
            { QMetaType::QString, 23 },
        }}),
        // Slot 'NewMetaData'
        QtMocHelpers::SlotData<void(const int, const EngineMetadata &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'AddBufferToScope'
        QtMocHelpers::SlotData<void(GstBuffer *, const int, const QString &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 }, { QMetaType::Int, 17 }, { QMetaType::QString, 30 },
        }}),
        // Slot 'FadeoutFinished'
        QtMocHelpers::SlotData<void(const int)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'FadeoutPauseFinished'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SeekNow'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlayDone'
        QtMocHelpers::SlotData<void(const GstStateChangeReturn, const bool, const quint64, const int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 }, { QMetaType::Bool, 37 }, { QMetaType::ULongLong, 38 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'BufferingStarted'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'BufferingProgress'
        QtMocHelpers::SlotData<void(int)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 41 },
        }}),
        // Slot 'BufferingFinished'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PipelineFinished'
        QtMocHelpers::SlotData<void(const int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GstEngine, qt_meta_tag_ZN9GstEngineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GstEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<EngineBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GstEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GstEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9GstEngineE_t>.metaTypes,
    nullptr
} };

void GstEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GstEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ReloadSettings(); break;
        case 1: _t->SetStereoBalancerEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->SetStereoBalance((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->SetEqualizerEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->SetEqualizerParameters((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 5: _t->AddBufferConsumer((*reinterpret_cast<std::add_pointer_t<GstBufferConsumer*>>(_a[1]))); break;
        case 6: _t->RemoveBufferConsumer((*reinterpret_cast<std::add_pointer_t<GstBufferConsumer*>>(_a[1]))); break;
        case 7: _t->EndOfStreamReached((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->HandlePipelineError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 9: _t->NewMetaData((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<EngineMetadata>>(_a[2]))); break;
        case 10: _t->AddBufferToScope((*reinterpret_cast<std::add_pointer_t<GstBuffer*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 11: _t->FadeoutFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->FadeoutPauseFinished(); break;
        case 13: _t->SeekNow(); break;
        case 14: _t->PlayDone((*reinterpret_cast<std::add_pointer_t<GstStateChangeReturn>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 15: _t->BufferingStarted(); break;
        case 16: _t->BufferingProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->BufferingFinished(); break;
        case 18: _t->PipelineFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *GstEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GstEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GstEngineE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "GstBufferConsumer"))
        return static_cast< GstBufferConsumer*>(this);
    return EngineBase::qt_metacast(_clname);
}

int GstEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EngineBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
