/****************************************************************************
** Meta object code from reading C++ file 'lastfmscrobbler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/scrobbler/lastfmscrobbler.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lastfmscrobbler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15LastFMScrobblerE_t {};
} // unnamed namespace

template <> constexpr inline auto LastFMScrobbler::qt_create_metaobjectdata<qt_meta_tag_ZN15LastFMScrobblerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LastFMScrobbler",
        "AuthenticationComplete",
        "",
        "success",
        "error",
        "WriteCache",
        "RedirectArrived",
        "AuthenticateReplyFinished",
        "QNetworkReply*",
        "reply",
        "UpdateNowPlayingRequestFinished",
        "ScrobbleRequestFinished",
        "ScrobblerCacheItemPtrList",
        "cache_items",
        "SingleScrobbleRequestFinished",
        "ScrobblerCacheItemPtr",
        "cache_item",
        "LoveRequestFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AuthenticationComplete'
        QtMocHelpers::SignalData<void(const bool, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'AuthenticationComplete'
        QtMocHelpers::SignalData<void(const bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'WriteCache'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RedirectArrived'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AuthenticateReplyFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'UpdateNowPlayingRequestFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'ScrobbleRequestFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *, ScrobblerCacheItemPtrList)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'SingleScrobbleRequestFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *, ScrobblerCacheItemPtr)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'LoveRequestFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LastFMScrobbler, qt_meta_tag_ZN15LastFMScrobblerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LastFMScrobbler::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrobblerService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LastFMScrobblerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LastFMScrobblerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15LastFMScrobblerE_t>.metaTypes,
    nullptr
} };

void LastFMScrobbler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LastFMScrobbler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AuthenticationComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->AuthenticationComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->WriteCache(); break;
        case 3: _t->RedirectArrived(); break;
        case 4: _t->AuthenticateReplyFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 5: _t->UpdateNowPlayingRequestFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 6: _t->ScrobbleRequestFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ScrobblerCacheItemPtrList>>(_a[2]))); break;
        case 7: _t->SingleScrobbleRequestFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ScrobblerCacheItemPtr>>(_a[2]))); break;
        case 8: _t->LoveRequestFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ScrobblerCacheItemPtrList >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ScrobblerCacheItemPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LastFMScrobbler::*)(const bool , const QString & )>(_a, &LastFMScrobbler::AuthenticationComplete, 0))
            return;
    }
}

const QMetaObject *LastFMScrobbler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LastFMScrobbler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LastFMScrobblerE_t>.strings))
        return static_cast<void*>(this);
    return ScrobblerService::qt_metacast(_clname);
}

int LastFMScrobbler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrobblerService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void LastFMScrobbler::AuthenticationComplete(const bool _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
