/****************************************************************************
** Meta object code from reading C++ file 'collectionview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/collection/collectionview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14CollectionViewE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionView::qt_create_metaobjectdata<qt_meta_tag_ZN14CollectionViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionView",
        "ShowSettingsDialog",
        "",
        "TotalSongCountUpdated_",
        "TotalArtistCountUpdated_",
        "TotalAlbumCountUpdated_",
        "Error",
        "error",
        "TotalSongCountUpdated",
        "count",
        "TotalArtistCountUpdated",
        "TotalAlbumCountUpdated",
        "ReloadSettings",
        "FilterReturnPressed",
        "SaveFocus",
        "RestoreFocus",
        "EditTagError",
        "message",
        "Load",
        "AddToPlaylist",
        "AddToPlaylistEnqueue",
        "AddToPlaylistEnqueueNext",
        "OpenInNewPlaylist",
        "SearchForThis",
        "Organize",
        "CopyToDevice",
        "EditTracks",
        "RescanSongs",
        "ShowInBrowser",
        "ShowInVarious",
        "NoShowInVarious",
        "Delete",
        "DeleteFilesFinished",
        "SongList",
        "songs_with_errors"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ShowSettingsDialog'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TotalSongCountUpdated_'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TotalArtistCountUpdated_'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'TotalAlbumCountUpdated_'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'TotalSongCountUpdated'
        QtMocHelpers::SlotData<void(const int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'TotalArtistCountUpdated'
        QtMocHelpers::SlotData<void(const int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'TotalAlbumCountUpdated'
        QtMocHelpers::SlotData<void(const int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'FilterReturnPressed'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SaveFocus'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RestoreFocus'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'EditTagError'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'Load'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddToPlaylist'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddToPlaylistEnqueue'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddToPlaylistEnqueueNext'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OpenInNewPlaylist'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchForThis'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Organize'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CopyToDevice'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditTracks'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RescanSongs'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowInBrowser'
        QtMocHelpers::SlotData<void() const>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowInVarious'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'NoShowInVarious'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Delete'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteFilesFinished'
        QtMocHelpers::SlotData<void(const SongList &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CollectionView, qt_meta_tag_ZN14CollectionViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CollectionView::staticMetaObject = { {
    QMetaObject::SuperData::link<AutoExpandingTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CollectionViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CollectionViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14CollectionViewE_t>.metaTypes,
    nullptr
} };

void CollectionView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ShowSettingsDialog(); break;
        case 1: _t->TotalSongCountUpdated_(); break;
        case 2: _t->TotalArtistCountUpdated_(); break;
        case 3: _t->TotalAlbumCountUpdated_(); break;
        case 4: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->TotalSongCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->TotalArtistCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->TotalAlbumCountUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->ReloadSettings(); break;
        case 9: _t->FilterReturnPressed(); break;
        case 10: _t->SaveFocus(); break;
        case 11: _t->RestoreFocus(); break;
        case 12: _t->EditTagError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->Load(); break;
        case 14: _t->AddToPlaylist(); break;
        case 15: _t->AddToPlaylistEnqueue(); break;
        case 16: _t->AddToPlaylistEnqueueNext(); break;
        case 17: _t->OpenInNewPlaylist(); break;
        case 18: _t->SearchForThis(); break;
        case 19: _t->Organize(); break;
        case 20: _t->CopyToDevice(); break;
        case 21: _t->EditTracks(); break;
        case 22: _t->RescanSongs(); break;
        case 23: _t->ShowInBrowser(); break;
        case 24: _t->ShowInVarious(); break;
        case 25: _t->NoShowInVarious(); break;
        case 26: _t->Delete(); break;
        case 27: _t->DeleteFilesFinished((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionView::*)()>(_a, &CollectionView::ShowSettingsDialog, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionView::*)()>(_a, &CollectionView::TotalSongCountUpdated_, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionView::*)()>(_a, &CollectionView::TotalArtistCountUpdated_, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionView::*)()>(_a, &CollectionView::TotalAlbumCountUpdated_, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionView::*)(const QString & )>(_a, &CollectionView::Error, 4))
            return;
    }
}

const QMetaObject *CollectionView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CollectionViewE_t>.strings))
        return static_cast<void*>(this);
    return AutoExpandingTreeView::qt_metacast(_clname);
}

int CollectionView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AutoExpandingTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void CollectionView::ShowSettingsDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CollectionView::TotalSongCountUpdated_()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CollectionView::TotalArtistCountUpdated_()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CollectionView::TotalAlbumCountUpdated_()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void CollectionView::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
