/****************************************************************************
** Meta object code from reading C++ file 'playlistcontainer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistcontainer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistcontainer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17PlaylistContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistContainer::qt_create_metaobjectdata<qt_meta_tag_ZN17PlaylistContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistContainer",
        "UndoRedoActionsChanged",
        "",
        "QAction*",
        "undo",
        "redo",
        "ViewSelectionModelChanged",
        "NewPlaylist",
        "LoadPlaylist",
        "SaveCurrentPlaylist",
        "SavePlaylist",
        "id",
        "ClearPlaylist",
        "GoToNextPlaylistTab",
        "GoToPreviousPlaylistTab",
        "SetViewModel",
        "Playlist*",
        "playlist",
        "scroll_position",
        "PlaylistAdded",
        "name",
        "favorite",
        "PlaylistClosed",
        "PlaylistRenamed",
        "new_name",
        "Started",
        "Save",
        "SetTabBarVisible",
        "visible",
        "SetTabBarHeight",
        "height",
        "SelectionChanged",
        "MaybeUpdateFilter",
        "UpdateFilter",
        "FocusOnFilter",
        "QKeyEvent*",
        "event",
        "UpdateNoMatchesLabel",
        "ActivePlaying",
        "ActivePaused",
        "ActiveStopped"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'UndoRedoActionsChanged'
        QtMocHelpers::SignalData<void(QAction *, QAction *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Signal 'ViewSelectionModelChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'NewPlaylist'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadPlaylist'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SaveCurrentPlaylist'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SavePlaylist'
        QtMocHelpers::SlotData<void(const int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'ClearPlaylist'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'GoToNextPlaylistTab'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'GoToPreviousPlaylistTab'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SetViewModel'
        QtMocHelpers::SlotData<void(Playlist *, const int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'PlaylistAdded'
        QtMocHelpers::SlotData<void(const int, const QString &, bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'PlaylistClosed'
        QtMocHelpers::SlotData<void(const int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'PlaylistRenamed'
        QtMocHelpers::SlotData<void(const int, const QString &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 24 },
        }}),
        // Slot 'Started'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Save'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SetTabBarVisible'
        QtMocHelpers::SlotData<void(const bool)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'SetTabBarHeight'
        QtMocHelpers::SlotData<void(const int)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
        // Slot 'SelectionChanged'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'MaybeUpdateFilter'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UpdateFilter'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FocusOnFilter'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Slot 'UpdateNoMatchesLabel'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ActivePlaying'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ActivePaused'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ActiveStopped'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistContainer, qt_meta_tag_ZN17PlaylistContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PlaylistContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PlaylistContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17PlaylistContainerE_t>.metaTypes,
    nullptr
} };

void PlaylistContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->UndoRedoActionsChanged((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[2]))); break;
        case 1: _t->ViewSelectionModelChanged(); break;
        case 2: _t->NewPlaylist(); break;
        case 3: _t->LoadPlaylist(); break;
        case 4: _t->SaveCurrentPlaylist(); break;
        case 5: _t->SavePlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->ClearPlaylist(); break;
        case 7: _t->GoToNextPlaylistTab(); break;
        case 8: _t->GoToPreviousPlaylistTab(); break;
        case 9: _t->SetViewModel((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->PlaylistAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 11: _t->PlaylistClosed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->PlaylistRenamed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 13: _t->Started(); break;
        case 14: _t->Save(); break;
        case 15: _t->SetTabBarVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->SetTabBarHeight((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->SelectionChanged(); break;
        case 18: _t->MaybeUpdateFilter(); break;
        case 19: _t->UpdateFilter(); break;
        case 20: _t->FocusOnFilter((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 21: _t->UpdateNoMatchesLabel(); break;
        case 22: _t->ActivePlaying(); break;
        case 23: _t->ActivePaused(); break;
        case 24: _t->ActiveStopped(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistContainer::*)(QAction * , QAction * )>(_a, &PlaylistContainer::UndoRedoActionsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistContainer::*)()>(_a, &PlaylistContainer::ViewSelectionModelChanged, 1))
            return;
    }
}

const QMetaObject *PlaylistContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PlaylistContainerE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int PlaylistContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void PlaylistContainer::UndoRedoActionsChanged(QAction * _t1, QAction * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void PlaylistContainer::ViewSelectionModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
