/****************************************************************************
** Meta object code from reading C++ file 'tidalstreamurlrequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/tidal/tidalstreamurlrequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tidalstreamurlrequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21TidalStreamURLRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto TidalStreamURLRequest::qt_create_metaobjectdata<qt_meta_tag_ZN21TidalStreamURLRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TidalStreamURLRequest",
        "StreamURLFailure",
        "",
        "id",
        "QUrl",
        "media_url",
        "error",
        "StreamURLSuccess",
        "stream_url",
        "Song::FileType",
        "filetype",
        "samplerate",
        "bit_depth",
        "duration",
        "StreamURLReceived"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'StreamURLFailure'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'StreamURLSuccess'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QUrl &, const Song::FileType, const int, const int, const qint64)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 8 }, { 0x80000000 | 9, 10 },
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 }, { QMetaType::LongLong, 13 },
        }}),
        // Signal 'StreamURLSuccess'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QUrl &, const Song::FileType, const int, const int)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 8 }, { 0x80000000 | 9, 10 },
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'StreamURLSuccess'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QUrl &, const Song::FileType, const int)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 8 }, { 0x80000000 | 9, 10 },
            { QMetaType::Int, 11 },
        }}),
        // Signal 'StreamURLSuccess'
        QtMocHelpers::SignalData<void(const uint, const QUrl &, const QUrl &, const Song::FileType)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'StreamURLReceived'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TidalStreamURLRequest, qt_meta_tag_ZN21TidalStreamURLRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TidalStreamURLRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<TidalBaseRequest::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21TidalStreamURLRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21TidalStreamURLRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21TidalStreamURLRequestE_t>.metaTypes,
    nullptr
} };

void TidalStreamURLRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TidalStreamURLRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->StreamURLFailure((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->StreamURLSuccess((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[7]))); break;
        case 2: _t->StreamURLSuccess((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6]))); break;
        case 3: _t->StreamURLSuccess((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 4: _t->StreamURLSuccess((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Song::FileType>>(_a[4]))); break;
        case 5: _t->StreamURLReceived(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TidalStreamURLRequest::*)(const uint , const QUrl & , const QString & )>(_a, &TidalStreamURLRequest::StreamURLFailure, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TidalStreamURLRequest::*)(const uint , const QUrl & , const QUrl & , const Song::FileType , const int , const int , const qint64 )>(_a, &TidalStreamURLRequest::StreamURLSuccess, 1))
            return;
    }
}

const QMetaObject *TidalStreamURLRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TidalStreamURLRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21TidalStreamURLRequestE_t>.strings))
        return static_cast<void*>(this);
    return TidalBaseRequest::qt_metacast(_clname);
}

int TidalStreamURLRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TidalBaseRequest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void TidalStreamURLRequest::StreamURLFailure(const uint _t1, const QUrl & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void TidalStreamURLRequest::StreamURLSuccess(const uint _t1, const QUrl & _t2, const QUrl & _t3, const Song::FileType _t4, const int _t5, const int _t6, const qint64 _t7)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7);
}
QT_WARNING_POP
