/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.UDisks2.Drive.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef UDISKS2DRIVE_H
#define UDISKS2DRIVE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "includes/dbus_metatypes.h"

/*
 * Proxy class for interface org.freedesktop.UDisks2.Drive
 */
class OrgFreedesktopUDisks2DriveInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.UDisks2.Drive"; }

public:
    OrgFreedesktopUDisks2DriveInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopUDisks2DriveInterface();

    Q_PROPERTY(QString Id READ id)
    inline QString id() const
    { return qvariant_cast< QString >(property("Id")); }

    Q_PROPERTY(bool MediaRemovable READ mediaRemovable)
    inline bool mediaRemovable() const
    { return qvariant_cast< bool >(property("MediaRemovable")); }

    Q_PROPERTY(QString Model READ model)
    inline QString model() const
    { return qvariant_cast< QString >(property("Model")); }

    Q_PROPERTY(QString Revision READ revision)
    inline QString revision() const
    { return qvariant_cast< QString >(property("Revision")); }

    Q_PROPERTY(QString Serial READ serial)
    inline QString serial() const
    { return qvariant_cast< QString >(property("Serial")); }

    Q_PROPERTY(qulonglong Size READ size)
    inline qulonglong size() const
    { return qvariant_cast< qulonglong >(property("Size")); }

    Q_PROPERTY(QString Vendor READ vendor)
    inline QString vendor() const
    { return qvariant_cast< QString >(property("Vendor")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Eject(const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("Eject"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

#endif
