/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.mpris.MediaPlayer2.Playlists.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef MPRIS2_PLAYLISTS_H
#define MPRIS2_PLAYLISTS_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "src/mpris2/mpris2.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.mpris.MediaPlayer2.Playlists
 */
class Mpris2Playlists: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.mpris.MediaPlayer2.Playlists")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.mpris.MediaPlayer2.Playlists\">\n"
"    <property access=\"read\" type=\"u\" name=\"PlaylistCount\"/>\n"
"    <property access=\"read\" type=\"as\" name=\"Orderings\"/>\n"
"    <property access=\"read\" type=\"(b(oss))\" name=\"ActivePlaylist\">\n"
"      <annotation value=\"MaybePlaylist\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n"
"    </property>\n"
"    <method name=\"ActivatePlaylist\">\n"
"      <arg direction=\"in\" type=\"o\" name=\"PlaylistId\"/>\n"
"    </method>\n"
"    <method name=\"GetPlaylists\">\n"
"      <arg direction=\"in\" type=\"u\" name=\"Index\"/>\n"
"      <arg direction=\"in\" type=\"u\" name=\"MaxCount\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"Order\"/>\n"
"      <arg direction=\"in\" type=\"b\" name=\"ReverseOrder\"/>\n"
"      <arg direction=\"out\" type=\"a(oss)\" name=\"Playlists\"/>\n"
"      <annotation value=\"MprisPlaylistList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <signal name=\"PlaylistChanged\">\n"
"      <arg direction=\"out\" type=\"(oss)\" name=\"Playlist\"/>\n"
"      <annotation value=\"MprisPlaylist\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </signal>\n"
"    <annotation value=\"MaybePlaylist\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n"
"  </interface>\n"
        "")
public:
    Mpris2Playlists(mpris::Mpris2 *parent);
    ~Mpris2Playlists() override;

    inline mpris::Mpris2 *parent() const
    { return static_cast<mpris::Mpris2 *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(MaybePlaylist ActivePlaylist READ activePlaylist)
    MaybePlaylist activePlaylist() const;

    Q_PROPERTY(QStringList Orderings READ orderings)
    QStringList orderings() const;

    Q_PROPERTY(uint PlaylistCount READ playlistCount)
    uint playlistCount() const;

public Q_SLOTS: // METHODS
    void ActivatePlaylist(const QDBusObjectPath &PlaylistId);
    MprisPlaylistList GetPlaylists(uint Index, uint MaxCount, const QString &Order, bool ReverseOrder);
Q_SIGNALS: // SIGNALS
    void PlaylistChanged(MprisPlaylist Playlist);
};

#endif
