/****************************************************************************
** Meta object code from reading C++ file 'coverprovider.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/covermanager/coverprovider.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'coverprovider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13CoverProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto CoverProvider::qt_create_metaobjectdata<qt_meta_tag_ZN13CoverProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CoverProvider",
        "AuthenticationFinished",
        "",
        "success",
        "error",
        "AuthenticationSuccess",
        "AuthenticationFailure",
        "SearchResults",
        "id",
        "CoverProviderSearchResults",
        "results",
        "SearchFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AuthenticationFinished'
        QtMocHelpers::SignalData<void(const bool, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'AuthenticationFinished'
        QtMocHelpers::SignalData<void(const bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'AuthenticationSuccess'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AuthenticationFailure'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'SearchResults'
        QtMocHelpers::SignalData<void(const int, const CoverProviderSearchResults &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'SearchFinished'
        QtMocHelpers::SignalData<void(const int, const CoverProviderSearchResults &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CoverProvider, qt_meta_tag_ZN13CoverProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CoverProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<JsonBaseRequest::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CoverProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CoverProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13CoverProviderE_t>.metaTypes,
    nullptr
} };

void CoverProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CoverProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AuthenticationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->AuthenticationFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->AuthenticationSuccess(); break;
        case 3: _t->AuthenticationFailure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->SearchResults((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverProviderSearchResults>>(_a[2]))); break;
        case 5: _t->SearchFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverProviderSearchResults>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CoverProviderSearchResults >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CoverProviderSearchResults >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CoverProvider::*)(const bool , const QString & )>(_a, &CoverProvider::AuthenticationFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoverProvider::*)()>(_a, &CoverProvider::AuthenticationSuccess, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoverProvider::*)(const QString & )>(_a, &CoverProvider::AuthenticationFailure, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoverProvider::*)(const int , const CoverProviderSearchResults & )>(_a, &CoverProvider::SearchResults, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoverProvider::*)(const int , const CoverProviderSearchResults & )>(_a, &CoverProvider::SearchFinished, 5))
            return;
    }
}

const QMetaObject *CoverProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CoverProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CoverProviderE_t>.strings))
        return static_cast<void*>(this);
    return JsonBaseRequest::qt_metacast(_clname);
}

int CoverProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JsonBaseRequest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void CoverProvider::AuthenticationFinished(const bool _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 2
void CoverProvider::AuthenticationSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CoverProvider::AuthenticationFailure(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void CoverProvider::SearchResults(const int _t1, const CoverProviderSearchResults & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void CoverProvider::SearchFinished(const int _t1, const CoverProviderSearchResults & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
