/****************************************************************************
** Meta object code from reading C++ file 'edittagdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/dialogs/edittagdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'edittagdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13EditTagDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto EditTagDialog::qt_create_metaobjectdata<qt_meta_tag_ZN13EditTagDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EditTagDialog",
        "Error",
        "",
        "message",
        "SetSongsFinished",
        "SaveDataFinished",
        "SelectionChanged",
        "FieldValueEdited",
        "ResetField",
        "ButtonClicked",
        "QAbstractButton*",
        "button",
        "ResetPlayStatistics",
        "SongRated",
        "rating",
        "FetchTag",
        "FetchTagSongChosen",
        "Song",
        "original_song",
        "new_metadata",
        "FetchLyrics",
        "UpdateLyrics",
        "id",
        "provider",
        "lyrics",
        "AlbumCoverLoaded",
        "AlbumCoverLoaderResult",
        "cover_result",
        "LoadCoverFromFile",
        "SaveCoverToFile",
        "LoadCoverFromURL",
        "SearchForCover",
        "UnsetCover",
        "ClearCover",
        "DeleteCover",
        "ShowCover",
        "PreviousSong",
        "NextSong",
        "SongSaveTagsComplete",
        "TagReaderReplyPtr",
        "reply",
        "filename",
        "song",
        "UpdateCoverAction",
        "cover_action"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'SetSongsFinished'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SaveDataFinished'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SelectionChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FieldValueEdited'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ResetField'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ButtonClicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'ResetPlayStatistics'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SongRated'
        QtMocHelpers::SlotData<void(const float)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Float, 14 },
        }}),
        // Slot 'FetchTag'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FetchTagSongChosen'
        QtMocHelpers::SlotData<void(const Song &, const Song &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Slot 'FetchLyrics'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UpdateLyrics'
        QtMocHelpers::SlotData<void(const quint64, const QString &, const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 22 }, { QMetaType::QString, 23 }, { QMetaType::QString, 24 },
        }}),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const quint64, const AlbumCoverLoaderResult &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 22 }, { 0x80000000 | 26, 27 },
        }}),
        // Slot 'LoadCoverFromFile'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SaveCoverToFile'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadCoverFromURL'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchForCover'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UnsetCover'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ClearCover'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteCover'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowCover'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PreviousSong'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'NextSong'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SongSaveTagsComplete'
        QtMocHelpers::SlotData<void(TagReaderReplyPtr, const QString &, Song, const UpdateCoverAction)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { QMetaType::QString, 41 }, { 0x80000000 | 17, 42 }, { 0x80000000 | 43, 44 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EditTagDialog, qt_meta_tag_ZN13EditTagDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EditTagDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13EditTagDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13EditTagDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13EditTagDialogE_t>.metaTypes,
    nullptr
} };

void EditTagDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EditTagDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->SetSongsFinished(); break;
        case 2: _t->SaveDataFinished(); break;
        case 3: _t->SelectionChanged(); break;
        case 4: _t->FieldValueEdited(); break;
        case 5: _t->ResetField(); break;
        case 6: _t->ButtonClicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 7: _t->ResetPlayStatistics(); break;
        case 8: _t->SongRated((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 9: _t->FetchTag(); break;
        case 10: _t->FetchTagSongChosen((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Song>>(_a[2]))); break;
        case 11: _t->FetchLyrics(); break;
        case 12: _t->UpdateLyrics((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 13: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 14: _t->LoadCoverFromFile(); break;
        case 15: _t->SaveCoverToFile(); break;
        case 16: _t->LoadCoverFromURL(); break;
        case 17: _t->SearchForCover(); break;
        case 18: _t->UnsetCover(); break;
        case 19: _t->ClearCover(); break;
        case 20: _t->DeleteCover(); break;
        case 21: _t->ShowCover(); break;
        case 22: _t->PreviousSong(); break;
        case 23: _t->NextSong(); break;
        case 24: _t->SongSaveTagsComplete((*reinterpret_cast<std::add_pointer_t<TagReaderReplyPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Song>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<enum UpdateCoverAction>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EditTagDialog::*)(const QString & )>(_a, &EditTagDialog::Error, 0))
            return;
    }
}

const QMetaObject *EditTagDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditTagDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13EditTagDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int EditTagDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void EditTagDialog::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
