/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/core/mainwindow.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "AlbumCoverReady",
        "",
        "Song",
        "song",
        "QImage",
        "image",
        "SearchCoverInProgress",
        "StopAfterToggled",
        "stop",
        "AuthorizationUrlReceived",
        "QUrl",
        "url",
        "PlaylistsLoaded",
        "FilePathChanged",
        "path",
        "MediaStopped",
        "MediaPaused",
        "MediaPlaying",
        "TrackSkipped",
        "PlaylistItemPtr",
        "item",
        "ForceShowOSD",
        "toggle",
        "PlaylistMenuHidden",
        "PlaylistRightClick",
        "QPoint",
        "global_pos",
        "QModelIndex",
        "index",
        "PlaylistCurrentChanged",
        "current",
        "PlaylistViewSelectionModelChanged",
        "PlaylistPlay",
        "PlaylistStopAfter",
        "PlaylistQueue",
        "PlaylistQueuePlayNext",
        "PlaylistSkip",
        "PlaylistRemoveCurrent",
        "PlaylistEditFinished",
        "playlist_id",
        "idx",
        "PlaylistClearCurrent",
        "RescanSongs",
        "EditTracks",
        "EditTagDialogAccepted",
        "RenumberTracks",
        "SelectionSetValue",
        "EditValue",
        "AutoCompleteTags",
        "AutoCompleteTagsAccepted",
        "PlaylistUndoRedoChanged",
        "QAction*",
        "undo",
        "redo",
        "AddFilesToTranscoder",
        "PlaylistCopyToCollection",
        "PlaylistMoveToCollection",
        "PlaylistCopyToDevice",
        "PlaylistOrganizeSelected",
        "copy",
        "PlaylistOpenInBrowser",
        "PlaylistCopyUrl",
        "ShowInCollection",
        "ChangeCollectionFilterMode",
        "action",
        "PlayIndex",
        "Playlist::AutoScroll",
        "autoscroll",
        "PlaylistDoubleClick",
        "StopAfterCurrent",
        "SongChanged",
        "VolumeChanged",
        "volume",
        "CopyFilesToCollection",
        "QList<QUrl>",
        "urls",
        "MoveFilesToCollection",
        "CopyFilesToDevice",
        "EditFileTags",
        "AddToPlaylist",
        "QMimeData*",
        "q_mimedata",
        "AddToPlaylistFromAction",
        "VolumeWheelEvent",
        "delta",
        "ToggleShowHide",
        "ToggleHide",
        "Seeked",
        "microseconds",
        "UpdateTrackPosition",
        "UpdateTrackSliderPosition",
        "TaskCountChanged",
        "count",
        "OpenCollectionSettingsDialog",
        "OpenServiceSettingsDialog",
        "Song::Source",
        "source",
        "ReloadSettings",
        "ReloadAllSettings",
        "RefreshStyleSheet",
        "SetHiddenInTray",
        "AddFile",
        "AddFolder",
        "AddCDTracks",
        "AddStream",
        "AddStreamAccepted",
        "PlayingWidgetPositionChanged",
        "above_status_bar",
        "SongSaveComplete",
        "TagReaderReplyPtr",
        "reply",
        "QPersistentModelIndex",
        "ShowCoverManager",
        "ShowEqualizer",
        "ShowAboutDialog",
        "ShowErrorDialog",
        "message",
        "CheckShowErrorDialog",
        "ShowTranscodeDialog",
        "CreateSettingsDialog",
        "SettingsDialog*",
        "CreateEditTagDialog",
        "EditTagDialog*",
        "OpenSettingsDialog",
        "OpenSettingsDialogAtPage",
        "SettingsDialog::Page",
        "page",
        "TabSwitched",
        "ToggleSidebar",
        "checked",
        "ToggleSearchCoverAuto",
        "SaveGeometry",
        "DoExit",
        "HandleNotificationPreview",
        "OSDSettings::Type",
        "type",
        "line1",
        "line2",
        "ShowConsole",
        "LoadCoverFromFile",
        "SaveCoverToFile",
        "LoadCoverFromURL",
        "SearchForCover",
        "UnsetCover",
        "ClearCover",
        "DeleteCover",
        "ShowCover",
        "SearchCoverAutomatically",
        "AlbumCoverLoaded",
        "AlbumCoverLoaderResult",
        "result",
        "ScrobblingEnabledChanged",
        "value",
        "ScrobbleButtonVisibilityChanged",
        "LoveButtonVisibilityChanged",
        "SendNowPlaying",
        "Love",
        "ExitFinished",
        "PlaylistDelete",
        "FocusSearchField",
        "DeleteFilesFinished",
        "SongList",
        "songs_with_errors",
        "FetchStreamingMetadata",
        "ProcessMetadataQueue",
        "CommandlineOptionsReceived",
        "string_options",
        "Raise",
        "Exit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AlbumCoverReady'
        QtMocHelpers::SignalData<void(const Song &, const QImage &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'SearchCoverInProgress'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'StopAfterToggled'
        QtMocHelpers::SignalData<void(const bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'AuthorizationUrlReceived'
        QtMocHelpers::SignalData<void(const QUrl &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'PlaylistsLoaded'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FilePathChanged'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'MediaStopped'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'MediaPaused'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'MediaPlaying'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'TrackSkipped'
        QtMocHelpers::SlotData<void(PlaylistItemPtr)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'ForceShowOSD'
        QtMocHelpers::SlotData<void(const Song &, const bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 23 },
        }}),
        // Slot 'PlaylistMenuHidden'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistRightClick'
        QtMocHelpers::SlotData<void(const QPoint, const QModelIndex &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'PlaylistCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 31 },
        }}),
        // Slot 'PlaylistViewSelectionModelChanged'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistPlay'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistStopAfter'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistQueue'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistQueuePlayNext'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistSkip'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistRemoveCurrent'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistEditFinished'
        QtMocHelpers::SlotData<void(const int, const QModelIndex &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { 0x80000000 | 28, 41 },
        }}),
        // Slot 'PlaylistClearCurrent'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RescanSongs'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditTracks'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditTagDialogAccepted'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RenumberTracks'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SelectionSetValue'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'EditValue'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AutoCompleteTags'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AutoCompleteTagsAccepted'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistUndoRedoChanged'
        QtMocHelpers::SlotData<void(QAction *, QAction *)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 52, 53 }, { 0x80000000 | 52, 54 },
        }}),
        // Slot 'AddFilesToTranscoder'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistCopyToCollection'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistMoveToCollection'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistCopyToDevice'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistOrganizeSelected'
        QtMocHelpers::SlotData<void(const bool)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'PlaylistOpenInBrowser'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistCopyUrl'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowInCollection'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ChangeCollectionFilterMode'
        QtMocHelpers::SlotData<void(QAction *)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 52, 65 },
        }}),
        // Slot 'PlayIndex'
        QtMocHelpers::SlotData<void(const QModelIndex &, Playlist::AutoScroll)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 41 }, { 0x80000000 | 67, 68 },
        }}),
        // Slot 'PlaylistDoubleClick'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 41 },
        }}),
        // Slot 'StopAfterCurrent'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SongChanged'
        QtMocHelpers::SlotData<void(const Song &)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'VolumeChanged'
        QtMocHelpers::SlotData<void(const uint)>(72, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 73 },
        }}),
        // Slot 'CopyFilesToCollection'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(74, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Slot 'MoveFilesToCollection'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(77, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Slot 'CopyFilesToDevice'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(78, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Slot 'EditFileTags'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(79, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Slot 'AddToPlaylist'
        QtMocHelpers::SlotData<void(QMimeData *)>(80, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 81, 82 },
        }}),
        // Slot 'AddToPlaylistFromAction'
        QtMocHelpers::SlotData<void(QAction *)>(83, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 52, 65 },
        }}),
        // Slot 'VolumeWheelEvent'
        QtMocHelpers::SlotData<void(const int)>(84, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 85 },
        }}),
        // Slot 'ToggleShowHide'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ToggleHide'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Seeked'
        QtMocHelpers::SlotData<void(const qint64)>(88, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 89 },
        }}),
        // Slot 'UpdateTrackPosition'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UpdateTrackSliderPosition'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'TaskCountChanged'
        QtMocHelpers::SlotData<void(const int)>(92, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'OpenCollectionSettingsDialog'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OpenServiceSettingsDialog'
        QtMocHelpers::SlotData<void(const Song::Source)>(95, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 96, 97 },
        }}),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ReloadAllSettings'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'RefreshStyleSheet'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SetHiddenInTray'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddFile'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddFolder'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddCDTracks'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddStream'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AddStreamAccepted'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlayingWidgetPositionChanged'
        QtMocHelpers::SlotData<void(const bool)>(107, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 108 },
        }}),
        // Slot 'SongSaveComplete'
        QtMocHelpers::SlotData<void(TagReaderReplyPtr, const QPersistentModelIndex &)>(109, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 110, 111 }, { 0x80000000 | 112, 41 },
        }}),
        // Slot 'ShowCoverManager'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowEqualizer'
        QtMocHelpers::SlotData<void()>(114, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowAboutDialog'
        QtMocHelpers::SlotData<void()>(115, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowErrorDialog'
        QtMocHelpers::SlotData<void(const QString &)>(116, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 117 },
        }}),
        // Slot 'CheckShowErrorDialog'
        QtMocHelpers::SlotData<void()>(118, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowTranscodeDialog'
        QtMocHelpers::SlotData<void()>(119, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CreateSettingsDialog'
        QtMocHelpers::SlotData<SettingsDialog *()>(120, 2, QMC::AccessPrivate, 0x80000000 | 121),
        // Slot 'CreateEditTagDialog'
        QtMocHelpers::SlotData<EditTagDialog *()>(122, 2, QMC::AccessPrivate, 0x80000000 | 123),
        // Slot 'OpenSettingsDialog'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OpenSettingsDialogAtPage'
        QtMocHelpers::SlotData<void(const SettingsDialog::Page)>(125, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'TabSwitched'
        QtMocHelpers::SlotData<void()>(128, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ToggleSidebar'
        QtMocHelpers::SlotData<void(const bool)>(129, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 130 },
        }}),
        // Slot 'ToggleSearchCoverAuto'
        QtMocHelpers::SlotData<void(const bool)>(131, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 130 },
        }}),
        // Slot 'SaveGeometry'
        QtMocHelpers::SlotData<void()>(132, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DoExit'
        QtMocHelpers::SlotData<void()>(133, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'HandleNotificationPreview'
        QtMocHelpers::SlotData<void(const OSDSettings::Type, const QString &, const QString &)>(134, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 135, 136 }, { QMetaType::QString, 137 }, { QMetaType::QString, 138 },
        }}),
        // Slot 'ShowConsole'
        QtMocHelpers::SlotData<void()>(139, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadCoverFromFile'
        QtMocHelpers::SlotData<void()>(140, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SaveCoverToFile'
        QtMocHelpers::SlotData<void()>(141, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'LoadCoverFromURL'
        QtMocHelpers::SlotData<void()>(142, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchForCover'
        QtMocHelpers::SlotData<void()>(143, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'UnsetCover'
        QtMocHelpers::SlotData<void()>(144, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ClearCover'
        QtMocHelpers::SlotData<void()>(145, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteCover'
        QtMocHelpers::SlotData<void()>(146, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ShowCover'
        QtMocHelpers::SlotData<void()>(147, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SearchCoverAutomatically'
        QtMocHelpers::SlotData<void()>(148, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'AlbumCoverLoaded'
        QtMocHelpers::SlotData<void(const Song &, const AlbumCoverLoaderResult &)>(149, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 150, 151 },
        }}),
        // Slot 'ScrobblingEnabledChanged'
        QtMocHelpers::SlotData<void(const bool)>(152, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 153 },
        }}),
        // Slot 'ScrobbleButtonVisibilityChanged'
        QtMocHelpers::SlotData<void(const bool)>(154, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 153 },
        }}),
        // Slot 'LoveButtonVisibilityChanged'
        QtMocHelpers::SlotData<void(const bool)>(155, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 153 },
        }}),
        // Slot 'SendNowPlaying'
        QtMocHelpers::SlotData<void()>(156, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Love'
        QtMocHelpers::SlotData<void()>(157, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ExitFinished'
        QtMocHelpers::SlotData<void()>(158, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'PlaylistDelete'
        QtMocHelpers::SlotData<void()>(159, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'FocusSearchField'
        QtMocHelpers::SlotData<void()>(160, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'DeleteFilesFinished'
        QtMocHelpers::SlotData<void(const SongList &)>(161, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 162, 163 },
        }}),
        // Slot 'FetchStreamingMetadata'
        QtMocHelpers::SlotData<void()>(164, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ProcessMetadataQueue'
        QtMocHelpers::SlotData<void()>(165, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CommandlineOptionsReceived'
        QtMocHelpers::SlotData<void(const QByteArray &)>(166, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 167 },
        }}),
        // Slot 'Raise'
        QtMocHelpers::SlotData<void()>(168, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Exit'
        QtMocHelpers::SlotData<void()>(169, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AlbumCoverReady((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QImage>>(_a[2]))); break;
        case 1: _t->SearchCoverInProgress(); break;
        case 2: _t->StopAfterToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->AuthorizationUrlReceived((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->PlaylistsLoaded(); break;
        case 5: _t->FilePathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->MediaStopped(); break;
        case 7: _t->MediaPaused(); break;
        case 8: _t->MediaPlaying(); break;
        case 9: _t->TrackSkipped((*reinterpret_cast<std::add_pointer_t<PlaylistItemPtr>>(_a[1]))); break;
        case 10: _t->ForceShowOSD((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->PlaylistMenuHidden(); break;
        case 12: _t->PlaylistRightClick((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 13: _t->PlaylistCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 14: _t->PlaylistViewSelectionModelChanged(); break;
        case 15: _t->PlaylistPlay(); break;
        case 16: _t->PlaylistStopAfter(); break;
        case 17: _t->PlaylistQueue(); break;
        case 18: _t->PlaylistQueuePlayNext(); break;
        case 19: _t->PlaylistSkip(); break;
        case 20: _t->PlaylistRemoveCurrent(); break;
        case 21: _t->PlaylistEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 22: _t->PlaylistClearCurrent(); break;
        case 23: _t->RescanSongs(); break;
        case 24: _t->EditTracks(); break;
        case 25: _t->EditTagDialogAccepted(); break;
        case 26: _t->RenumberTracks(); break;
        case 27: _t->SelectionSetValue(); break;
        case 28: _t->EditValue(); break;
        case 29: _t->AutoCompleteTags(); break;
        case 30: _t->AutoCompleteTagsAccepted(); break;
        case 31: _t->PlaylistUndoRedoChanged((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[2]))); break;
        case 32: _t->AddFilesToTranscoder(); break;
        case 33: _t->PlaylistCopyToCollection(); break;
        case 34: _t->PlaylistMoveToCollection(); break;
        case 35: _t->PlaylistCopyToDevice(); break;
        case 36: _t->PlaylistOrganizeSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->PlaylistOpenInBrowser(); break;
        case 38: _t->PlaylistCopyUrl(); break;
        case 39: _t->ShowInCollection(); break;
        case 40: _t->ChangeCollectionFilterMode((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 41: _t->PlayIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Playlist::AutoScroll>>(_a[2]))); break;
        case 42: _t->PlaylistDoubleClick((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 43: _t->StopAfterCurrent(); break;
        case 44: _t->SongChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 45: _t->VolumeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 46: _t->CopyFilesToCollection((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 47: _t->MoveFilesToCollection((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 48: _t->CopyFilesToDevice((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 49: _t->EditFileTags((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 50: _t->AddToPlaylist((*reinterpret_cast<std::add_pointer_t<QMimeData*>>(_a[1]))); break;
        case 51: _t->AddToPlaylistFromAction((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 52: _t->VolumeWheelEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 53: _t->ToggleShowHide(); break;
        case 54: _t->ToggleHide(); break;
        case 55: _t->Seeked((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 56: _t->UpdateTrackPosition(); break;
        case 57: _t->UpdateTrackSliderPosition(); break;
        case 58: _t->TaskCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 59: _t->OpenCollectionSettingsDialog(); break;
        case 60: _t->OpenServiceSettingsDialog((*reinterpret_cast<std::add_pointer_t<Song::Source>>(_a[1]))); break;
        case 61: _t->ReloadSettings(); break;
        case 62: _t->ReloadAllSettings(); break;
        case 63: _t->RefreshStyleSheet(); break;
        case 64: _t->SetHiddenInTray(); break;
        case 65: _t->AddFile(); break;
        case 66: _t->AddFolder(); break;
        case 67: _t->AddCDTracks(); break;
        case 68: _t->AddStream(); break;
        case 69: _t->AddStreamAccepted(); break;
        case 70: _t->PlayingWidgetPositionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 71: _t->SongSaveComplete((*reinterpret_cast<std::add_pointer_t<TagReaderReplyPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[2]))); break;
        case 72: _t->ShowCoverManager(); break;
        case 73: _t->ShowEqualizer(); break;
        case 74: _t->ShowAboutDialog(); break;
        case 75: _t->ShowErrorDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 76: _t->CheckShowErrorDialog(); break;
        case 77: _t->ShowTranscodeDialog(); break;
        case 78: { SettingsDialog* _r = _t->CreateSettingsDialog();
            if (_a[0]) *reinterpret_cast<SettingsDialog**>(_a[0]) = std::move(_r); }  break;
        case 79: { EditTagDialog* _r = _t->CreateEditTagDialog();
            if (_a[0]) *reinterpret_cast<EditTagDialog**>(_a[0]) = std::move(_r); }  break;
        case 80: _t->OpenSettingsDialog(); break;
        case 81: _t->OpenSettingsDialogAtPage((*reinterpret_cast<std::add_pointer_t<SettingsDialog::Page>>(_a[1]))); break;
        case 82: _t->TabSwitched(); break;
        case 83: _t->ToggleSidebar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 84: _t->ToggleSearchCoverAuto((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 85: _t->SaveGeometry(); break;
        case 86: _t->DoExit(); break;
        case 87: _t->HandleNotificationPreview((*reinterpret_cast<std::add_pointer_t<OSDSettings::Type>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 88: _t->ShowConsole(); break;
        case 89: _t->LoadCoverFromFile(); break;
        case 90: _t->SaveCoverToFile(); break;
        case 91: _t->LoadCoverFromURL(); break;
        case 92: _t->SearchForCover(); break;
        case 93: _t->UnsetCover(); break;
        case 94: _t->ClearCover(); break;
        case 95: _t->DeleteCover(); break;
        case 96: _t->ShowCover(); break;
        case 97: _t->SearchCoverAutomatically(); break;
        case 98: _t->AlbumCoverLoaded((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AlbumCoverLoaderResult>>(_a[2]))); break;
        case 99: _t->ScrobblingEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 100: _t->ScrobbleButtonVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 101: _t->LoveButtonVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 102: _t->SendNowPlaying(); break;
        case 103: _t->Love(); break;
        case 104: _t->ExitFinished(); break;
        case 105: _t->PlaylistDelete(); break;
        case 106: _t->FocusSearchField(); break;
        case 107: _t->DeleteFilesFinished((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 108: _t->FetchStreamingMetadata(); break;
        case 109: _t->ProcessMetadataQueue(); break;
        case 110: _t->CommandlineOptionsReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 111: _t->Raise(); break;
        case 112: _t->Exit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(const Song & , const QImage & )>(_a, &MainWindow::AlbumCoverReady, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::SearchCoverInProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(const bool )>(_a, &MainWindow::StopAfterToggled, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(const QUrl & )>(_a, &MainWindow::AuthorizationUrlReceived, 3))
            return;
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PlatformInterface"))
        return static_cast< PlatformInterface*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 113)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 113;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 113)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 113;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::AlbumCoverReady(const Song & _t1, const QImage & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void MainWindow::SearchCoverInProgress()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MainWindow::StopAfterToggled(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MainWindow::AuthorizationUrlReceived(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
