/****************************************************************************
** Meta object code from reading C++ file 'playlistlistcontainer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/playlist/playlistlistcontainer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistlistcontainer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21PlaylistListContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistListContainer::qt_create_metaobjectdata<qt_meta_tag_ZN21PlaylistListContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistListContainer",
        "ActivePlaying",
        "",
        "ActivePaused",
        "ActiveStopped",
        "NewFolderClicked",
        "ItemDoubleClicked",
        "QModelIndex",
        "proxy_idx",
        "ItemMimeDataDropped",
        "const QMimeData*",
        "q_mimedata",
        "PlaylistPathChanged",
        "id",
        "new_path",
        "PlaylistRenamed",
        "new_name",
        "AddPlaylist",
        "name",
        "favorite",
        "RemovePlaylist",
        "PlaylistFavoriteStateChanged",
        "CurrentChanged",
        "Playlist*",
        "new_playlist",
        "ActiveChanged",
        "ItemsSelectedChanged",
        "selected",
        "SavePlaylist",
        "Delete",
        "CopyToDevice"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ActivePlaying'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ActivePaused'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ActiveStopped'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'NewFolderClicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ItemDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'ItemMimeDataDropped'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QMimeData *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'PlaylistPathChanged'
        QtMocHelpers::SlotData<void(const int, const QString &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { QMetaType::QString, 14 },
        }}),
        // Slot 'PlaylistRenamed'
        QtMocHelpers::SlotData<void(const int, const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'AddPlaylist'
        QtMocHelpers::SlotData<void(const int, const QString &, const bool)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { QMetaType::QString, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'RemovePlaylist'
        QtMocHelpers::SlotData<void(const int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'PlaylistFavoriteStateChanged'
        QtMocHelpers::SlotData<void(const int, const bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'CurrentChanged'
        QtMocHelpers::SlotData<void(Playlist *)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'ActiveChanged'
        QtMocHelpers::SlotData<void(Playlist *)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'ItemsSelectedChanged'
        QtMocHelpers::SlotData<void(const bool)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'SavePlaylist'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'Delete'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'CopyToDevice'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistListContainer, qt_meta_tag_ZN21PlaylistListContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistListContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21PlaylistListContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21PlaylistListContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21PlaylistListContainerE_t>.metaTypes,
    nullptr
} };

void PlaylistListContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistListContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ActivePlaying(); break;
        case 1: _t->ActivePaused(); break;
        case 2: _t->ActiveStopped(); break;
        case 3: _t->NewFolderClicked(); break;
        case 4: _t->ItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->ItemMimeDataDropped((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const QMimeData*>>(_a[2]))); break;
        case 6: _t->PlaylistPathChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->PlaylistRenamed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->AddPlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 9: _t->RemovePlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->PlaylistFavoriteStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->CurrentChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 12: _t->ActiveChanged((*reinterpret_cast<std::add_pointer_t<Playlist*>>(_a[1]))); break;
        case 13: _t->ItemsSelectedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->SavePlaylist(); break;
        case 15: _t->Delete(); break;
        case 16: _t->CopyToDevice(); break;
        default: ;
        }
    }
}

const QMetaObject *PlaylistListContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistListContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21PlaylistListContainerE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int PlaylistListContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
