/****************************************************************************
** Meta object code from reading C++ file 'collectionlibrary.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../strawberry/src/collection/collectionlibrary.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionlibrary.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17CollectionLibraryE_t {};
} // unnamed namespace

template <> constexpr inline auto CollectionLibrary::qt_create_metaobjectdata<qt_meta_tag_ZN17CollectionLibraryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CollectionLibrary",
        "Error",
        "",
        "error",
        "ExitFinished",
        "ReloadSettings",
        "PauseWatcher",
        "ResumeWatcher",
        "FullScan",
        "StopScan",
        "Rescan",
        "SongList",
        "songs",
        "IncrementalScan",
        "CurrentSongChanged",
        "Song",
        "song",
        "Stopped",
        "ExitReceived",
        "SongsPlaycountChanged",
        "save_tags",
        "SongsRatingChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Error'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'ExitFinished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ReloadSettings'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'PauseWatcher'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ResumeWatcher'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'FullScan'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'StopScan'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Rescan'
        QtMocHelpers::SlotData<void(const SongList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'IncrementalScan'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'CurrentSongChanged'
        QtMocHelpers::SlotData<void(const Song &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'Stopped'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ExitReceived'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'SongsPlaycountChanged'
        QtMocHelpers::SlotData<void(const SongList &, const bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'SongsPlaycountChanged'
        QtMocHelpers::SlotData<void(const SongList &)>(19, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'SongsRatingChanged'
        QtMocHelpers::SlotData<void(const SongList &, const bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'SongsRatingChanged'
        QtMocHelpers::SlotData<void(const SongList &)>(21, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CollectionLibrary, qt_meta_tag_ZN17CollectionLibraryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CollectionLibrary::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionLibraryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionLibraryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17CollectionLibraryE_t>.metaTypes,
    nullptr
} };

void CollectionLibrary::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionLibrary *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->ExitFinished(); break;
        case 2: _t->ReloadSettings(); break;
        case 3: _t->PauseWatcher(); break;
        case 4: _t->ResumeWatcher(); break;
        case 5: _t->FullScan(); break;
        case 6: _t->StopScan(); break;
        case 7: _t->Rescan((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 8: _t->IncrementalScan(); break;
        case 9: _t->CurrentSongChanged((*reinterpret_cast<std::add_pointer_t<Song>>(_a[1]))); break;
        case 10: _t->Stopped(); break;
        case 11: _t->ExitReceived(); break;
        case 12: _t->SongsPlaycountChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->SongsPlaycountChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        case 14: _t->SongsRatingChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 15: _t->SongsRatingChanged((*reinterpret_cast<std::add_pointer_t<SongList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionLibrary::*)(const QString & )>(_a, &CollectionLibrary::Error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionLibrary::*)()>(_a, &CollectionLibrary::ExitFinished, 1))
            return;
    }
}

const QMetaObject *CollectionLibrary::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionLibrary::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CollectionLibraryE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CollectionLibrary::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void CollectionLibrary::Error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CollectionLibrary::ExitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
