/****************************************************************************
** Meta object code from reading C++ file 'mpris2_tracklist.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mpris2_tracklist.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mpris2_tracklist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15Mpris2TrackListE_t {};
} // unnamed namespace

template <> constexpr inline auto Mpris2TrackList::qt_create_metaobjectdata<qt_meta_tag_ZN15Mpris2TrackListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Mpris2TrackList",
        "D-Bus Interface",
        "org.mpris.MediaPlayer2.TrackList",
        "D-Bus Introspection",
        "  <interface name=\"org.mpris.MediaPlayer2.TrackList\">\n    <meth"
        "od name=\"GetTracksMetadata\">\n      <arg direction=\"in\" type=\""
        "ao\" name=\"TrackIds\"/>\n      <arg direction=\"out\" type=\"aa{s"
        "v}\" name=\"Metadata\"/>\n      <annotation value=\"TrackMetadata\""
        " name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n "
        "   <method name=\"AddTrack\">\n      <arg direction=\"in\" type=\""
        "s\" name=\"Uri\"/>\n      <arg direction=\"in\" type=\"o\" name=\""
        "AfterTrack\"/>\n      <arg direction=\"in\" type=\"b\" name=\"SetA"
        "sCurrent\"/>\n    </method>\n    <method name=\"RemoveTrack\">\n  "
        "    <arg direction=\"in\" type=\"o\" name=\"TrackId\"/>\n    </met"
        "hod>\n    <method name=\"GoTo\">\n      <arg direction=\"in\" type"
        "=\"o\" name=\"TrackId\"/>\n    </method>\n    <signal name=\"Track"
        "ListReplaced\">\n      <arg type=\"ao\" name=\"Tracks\"/>\n      <"
        "arg type=\"o\" name=\"CurrentTrack\"/>\n    </signal>\n    <signal"
        " name=\"TrackAdded\">\n      <arg type=\"a{sv}\" name=\"Metadata\""
        "/>\n      <annotation value=\"QVariantMap\" name=\"org.qtproject.Q"
        "tDBus.QtTypeName.Out0\"/>\n      <arg type=\"o\" name=\"AfterTrack"
        "\"/>\n    </signal>\n    <signal name=\"TrackRemoved\">\n      <ar"
        "g type=\"o\" name=\"TrackId\"/>\n    </signal>\n    <signal name=\""
        "TrackMetadataChanged\">\n      <arg type=\"o\" name=\"TrackId\"/>\n"
        "      <arg type=\"a{sv}\" name=\"Metadata\"/>\n      <annotation v"
        "alue=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\""
        "/>\n    </signal>\n    <property access=\"read\" type=\"ao\" name="
        "\"Tracks\"/>\n    <property access=\"read\" type=\"b\" name=\"CanE"
        "ditTracks\"/>\n  </interface>\n",
        "TrackAdded",
        "",
        "QVariantMap",
        "Metadata",
        "QDBusObjectPath",
        "AfterTrack",
        "TrackListReplaced",
        "QList<QDBusObjectPath>",
        "Tracks",
        "CurrentTrack",
        "TrackMetadataChanged",
        "TrackId",
        "TrackRemoved",
        "AddTrack",
        "Uri",
        "SetAsCurrent",
        "GetTracksMetadata",
        "TrackMetadata",
        "TrackIds",
        "GoTo",
        "RemoveTrack",
        "CanEditTracks"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'TrackAdded'
        QtMocHelpers::SignalData<void(const QVariantMap &, const QDBusObjectPath &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'TrackListReplaced'
        QtMocHelpers::SignalData<void(const QList<QDBusObjectPath> &, const QDBusObjectPath &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 9, 14 },
        }}),
        // Signal 'TrackMetadataChanged'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QVariantMap &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 16 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'TrackRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 16 },
        }}),
        // Slot 'AddTrack'
        QtMocHelpers::SlotData<void(const QString &, const QDBusObjectPath &, bool)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { 0x80000000 | 9, 10 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'GetTracksMetadata'
        QtMocHelpers::SlotData<TrackMetadata(const QList<QDBusObjectPath> &)>(21, 6, QMC::AccessPublic, 0x80000000 | 22, {{
            { 0x80000000 | 12, 23 },
        }}),
        // Slot 'GoTo'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 16 },
        }}),
        // Slot 'RemoveTrack'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'CanEditTracks'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Tracks'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Mpris2TrackList, qt_meta_tag_ZN15Mpris2TrackListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Mpris2TrackList::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Mpris2TrackListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Mpris2TrackListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15Mpris2TrackListE_t>.metaTypes,
    nullptr
} };

void Mpris2TrackList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Mpris2TrackList *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->TrackAdded((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 1: _t->TrackListReplaced((*reinterpret_cast<std::add_pointer_t<QList<QDBusObjectPath>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 2: _t->TrackMetadataChanged((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 3: _t->TrackRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 4: _t->AddTrack((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 5: { TrackMetadata _r = _t->GetTracksMetadata((*reinterpret_cast<std::add_pointer_t<QList<QDBusObjectPath>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<TrackMetadata*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->GoTo((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 7: _t->RemoveTrack((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Mpris2TrackList::*)(const QVariantMap & , const QDBusObjectPath & )>(_a, &Mpris2TrackList::TrackAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2TrackList::*)(const QList<QDBusObjectPath> & , const QDBusObjectPath & )>(_a, &Mpris2TrackList::TrackListReplaced, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2TrackList::*)(const QDBusObjectPath & , const QVariantMap & )>(_a, &Mpris2TrackList::TrackMetadataChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Mpris2TrackList::*)(const QDBusObjectPath & )>(_a, &Mpris2TrackList::TrackRemoved, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->canEditTracks(); break;
        case 1: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->tracks(); break;
        default: break;
        }
    }
}

const QMetaObject *Mpris2TrackList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Mpris2TrackList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Mpris2TrackListE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int Mpris2TrackList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Mpris2TrackList::TrackAdded(const QVariantMap & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Mpris2TrackList::TrackListReplaced(const QList<QDBusObjectPath> & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Mpris2TrackList::TrackMetadataChanged(const QDBusObjectPath & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Mpris2TrackList::TrackRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
